<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class IpSetBuildHelper {

var $logger;
var $workdir;
var $processorLogged;
var $processor;
var $profiler;
var $spawner;
var $timeout = 3;
var $existen = array();
var $must = array();

function setProfiler($obj){
	$this->profiler = $obj;
}

function setSpawner($obj){
	$this->spawner = $obj;
}

function setLogFactory($logFactory){
	$this->logger = $logFactory->getLogger('ipset.build.helper');
}

function setProcessorLogged($obj){
	$this->processorLogged = $obj;
}

function setProcessor($obj){
	$this->processor = $obj;
}

function setWorkdir($str){
	$this->workdir = $str;
}

function prepare(){
	exec('rm -rf '.$this->workdir.'/make');
}

function process(){
	$pids = array();
	if($d = @dir($this->workdir.'/make/')){
		while (false !== ($entry = $d->read())) {
			if($entry != '.' && $entry !='..' && is_dir($d->path.'/'.$entry)){
				$cmd = $_SERVER['argv'][0].' -a make_server -s '.$entry;
				$pids[] = $this->spawner->spawn($cmd);
			}
		}
		$d->close();
	}
	$this->spawner->waitFor($pids);
}

function processServer(){
	$opt = getopt('w:a:s:');
	if(!isset($opt['s'])){
		$this->logger->error('set server by -s option');
		return;
	}
	$server = $opt['s'];
	$this->script = array();
	$d = dir($this->workdir.'/make/'.$opt['s']);
	while (false !== ($entry = $d->read())) {
		if($entry != '.' && $entry !='..'){
			$this->processHash($d->path.'/'.$entry);
		}
	}
	$this->buildScript();
	$filename = $this->workdir.'/build-'.$server;
	if($file = @fopen($filename, "w+")){
		$this->logger->debug("Writing build to '%s'", $filename);
		fputs($file, implode("\n", $this->script));
		fclose($file);
		chmod($filename, 0711);
		if(!empty($this->script)){
			if($server!='localhost'){
				$copy = sprintf("scp -o BatchMode=yes -o ConnectTimeout=%s %s %s@%s:%s 2>&1", $this->timeout, $filename, 'root', $server, $filename);
				$this->logger->debug($copy);
				exec($copy, $out, $err);
			}else{
				$err = 0;
			}
			if($err){
				$this->logger->error("can't send script: %s, exit code: %s, output:\n%s", $copy, $err, implode("\n", $out));
			}else{
				$cmd = $filename;
				if($server!='localhost'){
					$cmd =  sprintf("ssh -o BatchMode=yes -o ConnectTimeout=%s %s@%s \"%s\" 2>&1", $this->timeout, 'root', $server, $cmd);
				}
				$this->logger->debug($cmd);
				$err = $this->processorLogged->process($cmd, $this->logger);
				if($err){
					$this->logger->error("can't build: %s, exit code: %s", $cmd, $err);
				}
			}
		}else{
			$this->logger->debug("skip executing empty script '%s'", $filename);
		}
	}else{
		$this->logger->error("Can't write to '%s'", $filename);
	}
}

function buildScript(){
	foreach($this->must as $hash=>$arr){
		foreach($arr as $addr){
			if(!isset($this->existen[$hash][$addr])){
				$this->script[] = sprintf("ipset -A %s %s", $hash, $addr);
			}else{
				unset($this->existen[$hash][$addr]);
			}
		}
	}
	foreach($this->existen as $hash=>$arr){
		foreach($arr as $addr=>$x){
			$this->script[] = sprintf("ipset -D %s %s", $hash, $addr);
		}
	}
}

function processHash($path){
	if($data = unserialize(file_get_contents($path))){
		if(!isset($this->existen[$data['name']])){
			$this->existen[$data['name']] = $this->readHash($data['read_hash_cmd']);
		}
		foreach($data['hash'] as $addr){
			$this->must[$data['name']][] = $addr;
		}
	}
}

function readHash($cmd){
	$this->hash = array();
	$this->logger->debug($cmd);
	$err = $this->processor->process($cmd,
		new NoneReader(),
		new LineWriter(new CallbackWriter(array($this, 'readHashLine'))),
		new LineWriter(new CallbackWriter(array($this->logger, 'error')))
	);
	if($err){
		$this->logger->error("can't read hash: %s, exit code: %s", $cmd, $err);
		return array();
	}
	return $this->hash;
}

function readHashLine($line){
	if(count(explode('.', $line))==4){
		$this->hash[trim($line)] = 1;
	}
}

}
?>