<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class IpSetBuilder {

var $host = 'localhost';
var $logger;
var $workdir;
var $ipHashName;
var $netHashName;
var $started;
var $finished;
var $processorLogged;
var $processor;
var $profiler;

function setProfiler($obj){
	$this->profiler = $obj;
}

function setLogFactory($logFactory){
	$this->logger = $logFactory->getLogger('ipset.builder');
}

function setProcessorLogged($obj){
	$this->processorLogged = $obj;
}

function setProcessor($obj){
	$this->processor = $obj;
}

function setIpHashName($str){
	$this->ipHashName = $str;
}

function setNetHashName($str){
	$this->netHashName = $str;
}

function setWorkdir($str){
	$this->workdir = $str;
}

function beginSetup(){
	if(!$this->started){
		$this->logger->debug("begin setup %s, %s", $this->ipHashName, $this->netHashName);
		$this->ipHash = array();
		$this->netHash = array();
		$this->started = 1;
	}
}

function add($ip){
	if(strpos($ip, '/') > 0){
		$this->netHash[] = $ip;
	}else{
		$this->ipHash[] = $ip;
	}
}

function finishSetup(){
	if(!$this->finished){
		$this->logger->debug("finish setup %s, %s", $this->ipHashName, $this->netHashName);
		if(isset($this->ipHashName)){
			$this->make($this->ipHashName, $this->ipHash); //'iphash', 
		}
		if(isset($this->netHashName)){
			$this->make($this->netHashName, $this->netHash); //'nethash', 
		}
		$this->finished = 1;
	}
}

function make($name, $hash){
	$data = array(
		'name'=>$name,
		'read_hash_cmd'=>$this->buildHashReadCommand($name),
		'hash'=>&$hash,
	);
	$target = $this->workdir.'/make/'.$this->host.'/'.$name.'-'.getmypid();
	@mkdir(dirname($target), 0777, true);
	file_put_contents($target, serialize($data));
	return; //   IpSetBuildHelper
	
	
	$this->profiler->enter('ipset.builder.make');
	$existen = $this->readHash($name);
	if(!is_array($existen)){
		$this->logger->error("can't build set '%s' cause it can't be read", $name);
		$this->profiler->leave('ipset.builder.make');
		return;
	}
	$script = array();
	foreach($hash as $addr){
		if(!isset($existen[$addr])){
			$script[] = sprintf("ipset -A %s %s", $name, $addr);
		}else{
			unset($existen[$addr]);
		}
	}
	foreach($existen as $addr=>$x){
		$script[] = sprintf("ipset -D %s %s", $name, $addr);
	}
	$filename = $this->workdir.'/build-'.$this->getScriptFileName($name);
	if($file = @fopen($filename, "w+")){
		$this->logger->debug("Writing build to '%s'", $filename);
		fputs($file, implode("\n", $script));
		fclose($file);
		chmod($filename, 0711);
		if(!empty($script)){
			$cmd = $this->buildScriptCommand($filename);
			$this->logger->debug($cmd);
			$err = $this->processorLogged->process($cmd, $this->logger);
			//exec($cmd, $out, $err);
			if($err){
				$this->logger->error("can't build: %s, exit code: %s", $cmd, $err);
			}
		}else{
			$this->logger->debug("skip executing empty script '%s'", $filename);
		}
	}else{
		$this->logger->error("Can't write to '%s'", $filename);
	}
	$this->profiler->leave('ipset.builder.make');
}

function buildHashReadCommand($name){
	return sprintf("ipset -L %s", $name);
}

function readHash($name){
	$this->hash = array();
	$cmd = $this->buildHashReadCommand($name);
	$this->logger->debug($cmd);
	$err = $this->processor->process($cmd,
		new NoneReader(),
		new LineWriter(new CallbackWriter(array($this, 'readHashLine'))),
		new LineWriter(new CallbackWriter(array($this->logger, 'error')))
	);
	//exec($cmd, $out, $err);
	if($err){
		$this->logger->error("can't read hash: %s, exit code: %s", $cmd, $err);
		return;
	}
	return $this->hash;
}

function readHashLine($line){
	if(count(explode('.', $line))==4){
		$this->hash[trim($line)] = 1;
	}
}

function getScriptFileName($name){
	return $name;
}

function buildScriptCommand($filename){
	return $filename;
}

}
?>