<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class IptNames {

var $names = array();
var $logger;
var $profiler;
var $workDir;
var $servers = array();
var $spawner;
var $activeServer;

function setLogFactory($logFactory){
	$this->logger = $logFactory->getLogger('ipt.names');
}

function setProfiler($obj){
	$this->profiler = $obj;
}

function setSpawner($obj){
	$this->spawner = $obj;
}

function setWorkDir($str){
	$this->workDir = $str;
}

function detectActive(){
	$opt = getopt('w:a:s:');
	if(!isset($opt['s'])){
		$this->logger->error('set server by -s option');
		return;
	}
	if(!isset($this->servers[$opt['s']])){
		$this->logger->error('server %s not found', $opt['s']);
		return;
	}
	$this->activeServer = $opt['s'];
}

function isActive($server){
	return $this->activeServer == $server;
}

function launchByServerRead(){
	$pids = array();
	foreach($this->servers as $host=>$list_cmd){
		$cmd = $_SERVER['argv'][0].' -a read_counters -s '.$host;
		$pids[] = $this->spawner->spawn($cmd);
	}
	$this->spawner->waitFor($pids);
}

function registerServer($host, $list_cmd){
	$this->servers[$host] = $list_cmd;
}

function readNames(){
	$this->logger->debug("begin read");
	$pids = array();
	foreach($this->servers as $host=>$list_cmd){
		$cmd = $_SERVER['argv'][0].' -a read_ipt_names -s '.$host;
		$pids[] = $this->spawner->spawn($cmd);
	}
	$this->spawner->waitFor($pids);
	$this->loadNames();
	$this->logger->debug("end read");
}

function loadNames(){
	foreach($this->servers as $host=>$list_cmd){
		if($arr = unserialize(file_get_contents($this->workDir.'/ipt.names-'.$host))){
			$this->names[$host] = $arr;
		}
	}
}

function readServerNames(){
	$opt = getopt('w:a:s:');
	if(!isset($opt['s'])){
		$this->logger->error('set server by -s option');
		return;
	}
	$host = $opt['s'];
	if(!isset($this->servers[$host])){
		$this->logger->error('server %s not found', $host);
		return;
	}
	$list_cmd = $this->servers[$host];
	$this->logger->debug($list_cmd);
	if($f = popen($list_cmd, "r")){
		$names = array();
		while (!feof ($f)) {
			if($line = trim(fgets($f, 255))){
				if(strstr($line, 'Found table:')){
					$table = substr($line, 13);
					$names[] = $table;
				}
			}
		}
		fclose($f);
		file_put_contents($this->workDir.'/ipt.names-'.$host, serialize($names));
	}else{
		$this->logger->error("can't open '%s' for reading", $list_cmd);
	}
}

function getNames($host){
	if(isset($this->names[$host])){
		return $this->names[$host];
	}else{
		$this->logger->warning("names not found for %s", $host);
		return array();
	}
}

}
?>