<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class IpUseLogPackerDAO {

var $connection;
var $queries = array(
	'drop_old_intervals'=>"DROP TABLE IF EXISTS _old_intervals",
	'drop_new_intervals'=>"DROP TABLE IF EXISTS _new_intervals",
	'drop_old_log'=>"DROP TABLE IF EXISTS _old_log",
	'drop_new_log'=>"DROP TABLE IF EXISTS _new_log",
	'prepare_new_intervals'=>"CREATE TABLE _new_intervals (
		id INT UNSIGNED NOT NULL,
		from_time DATETIME NOT NULL,
		to_time DATETIME NOT NULL,
		link BIGINT,
		INDEX (link))
		TYPE=InnoDB",
	'prepare_new_log'=>"CREATE TABLE _new_log (
		ip_use INT UNSIGNED NOT NULL,
		ip_use_interval INT UNSIGNED NOT NULL,
		bytes BIGINT UNSIGNED)
		TYPE=InnoDB",
	'lock'=>'LOCK TABLES ip_use_log WRITE, ip_use_intervals WRITE, _new_log WRITE, _new_intervals WRITE',
	'unlock'=>"UNLOCK TABLES",
	'move_packed_intervals'=>"INSERT INTO _new_intervals (id, from_time, to_time, link)
		SELECT max(ip_use_interval), min(from_time), max(to_time), floor((UNIX_TIMESTAMP(to_time) - (%s))/%s)
		FROM ip_use_log
		LEFT JOIN ip_use_intervals ON ip_use_intervals.id=ip_use_log.ip_use_interval
		WHERE (to_time > FROM_UNIXTIME(%s)) AND (to_time <= FROM_UNIXTIME(%s))
		GROUP BY 4",
	'move_packed_log'=>"INSERT INTO _new_log (ip_use, ip_use_interval, bytes)
		SELECT ip_use, _new_intervals.id, sum(bytes)
		FROM ip_use_log
		LEFT JOIN ip_use_intervals ON ip_use_intervals.id=ip_use_log.ip_use_interval
		LEFT JOIN _new_intervals ON _new_intervals.link = floor((UNIX_TIMESTAMP(ip_use_intervals.to_time) - (%s))/%s)
		WHERE (ip_use_intervals.to_time > FROM_UNIXTIME(%s)) AND (ip_use_intervals.to_time <= FROM_UNIXTIME(%s))
		GROUP BY 1, 2",
	'move_unpacked_intervals'=>"INSERT INTO _new_intervals (id, from_time, to_time)
		SELECT ip_use_interval, from_time, to_time FROM ip_use_log
		LEFT JOIN ip_use_intervals ON ip_use_intervals.id=ip_use_log.ip_use_interval
		WHERE to_time > FROM_UNIXTIME(%s)
		GROUP BY ip_use_interval",
	'move_unpacked_log'=>"INSERT INTO _new_log (ip_use, ip_use_interval, bytes)
		SELECT ip_use, ip_use_interval, bytes FROM ip_use_log
		LEFT JOIN ip_use_intervals ON ip_use_intervals.id=ip_use_log.ip_use_interval
		WHERE to_time > FROM_UNIXTIME(%s)",
	'swap'=>"RENAME TABLE 
		ip_use_log TO _old_log,
		ip_use_intervals TO _old_intervals,
		_new_intervals TO ip_use_intervals,
		_new_log TO ip_use_log",
	'finalize_intervals'=>"ALTER TABLE ip_use_intervals
		CHANGE COLUMN id id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
		DROP COLUMN link,
		ADD INDEX (from_time),
		ADD INDEX (to_time)",
	'finalize_log'=>array(
	    /*
		"ALTER TABLE ip_use_log add PRIMARY KEY (ip_use, ip_use_interval)",
	    "ALTER TABLE ip_use_log ADD INDEX (ip_use_interval)",
	    "ALTER TABLE ip_use_log ADD FOREIGN KEY (ip_use_interval) REFERENCES ip_use_intervals (id)",
	    "ALTER TABLE ip_use_log ADD FOREIGN KEY (ip_use) REFERENCES ip_uses (id) ON DELETE CASCADE",
		*/
		"ALTER TABLE ip_use_log add PRIMARY KEY (ip_use, ip_use_interval),
			ADD INDEX (ip_use_interval),
			ADD FOREIGN KEY (ip_use_interval) REFERENCES ip_use_intervals (id),
			ADD FOREIGN KEY (ip_use) REFERENCES ip_uses (id) ON DELETE CASCADE"
	),
);

function setConnection($connection){
	$this->connection = $connection;
}

function prepare(){
	$this->connection->query($this->queries['drop_old_log']);
	$this->connection->query($this->queries['drop_new_log']);
	$this->connection->query($this->queries['prepare_new_log']);
	$this->connection->query($this->queries['drop_old_intervals']);
	$this->connection->query($this->queries['drop_new_intervals']);
	$this->connection->query($this->queries['prepare_new_intervals']);
	$this->connection->query($this->queries['lock']);
}

function movePackedData($from_time, $to_time, $granula){
	$gmt_shift = mktime(0,0,0,1,1,1970);
	$this->connection->query($this->queries['move_packed_intervals'], $gmt_shift, $granula, $from_time, $to_time);
	$this->connection->query($this->queries['move_packed_log'], $gmt_shift, $granula, $from_time, $to_time);
}

function moveUnpackedData($from_time){
	$this->connection->query($this->queries['move_unpacked_intervals'], $from_time);
	$this->connection->query($this->queries['move_unpacked_log'], $from_time);
}

function finalize(){
	$this->connection->query($this->queries['unlock']);
	$this->connection->query($this->queries['swap']);
	$this->connection->query($this->queries['finalize_intervals']);
	foreach($this->queries['finalize_log'] as $q){
	    $this->connection->query($q);
	}
	$this->connection->query($this->queries['drop_old_log']);
	$this->connection->query($this->queries['drop_old_intervals']);
}

}
?>