<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class MySqliConnection {

var $host = 'localhost';
var $dieOnError = false;
var $link;
var $external_link = false;
var $logger;
var $errors = array();
var $isolationLevel;
var $nextAsync = false;
var $profiler;
var $nulls = '';
var $autoquote = false;

function setProfiler($obj){
	$this->profiler = $obj;
}

function setHost($host){
	$this->host = $host;
}

function setLink($link){
	if($link){
		$this->external_link = true;
	}else{
		$this->external_link = false;
	}
	$this->link = $link;
}

function setDatabase($database){
	$this->database = $database;
}

function setUser($user){
	$this->user = $user;
}

function setPassword($password){
	$this->password = $password;
}

function setCharset($charset){
	$this->charset = $charset;
}

function setLogFactory($logFactory){
	$this->logger = $logFactory->getLogger('mysql');
}

function setDieOnError($bool){
	$this->dieOnError = $bool;
}

function setIsolationLevel($str){
    $this->isolationLevel = $str;
}

function setNulls($str){
    $this->nulls = $str;
}

function setAutoquote($bool){
    $this->autoquote = $bool;
}

function initBean(){
	$this->connect();
}

function clearErrors(){
	$this->errors = array();
}

function hasErrors(){
	return !empty($this->errors);
}

function connect(){
	if(!$this->external_link){
		$this->link = mysqli_connect($this->host, $this->user, $this->password, $this->database) 
		or die(sprintf("Could not connect to %s as %s with%s password specified", $this->host, $this->user, $this->password ? "":"out"));
		if(isset($this->charset)){
			$this->query("SET NAMES %s", $this->charset);
		}
		if($this->isolationLevel){
		    $this->query("SET SESSION TRANSACTION ISOLATION LEVEL %s", $this->isolationLevel);
		}
	}
}

function close(){
	 mysqli_close($this->link);
}

function nextAsync(){
	$this->nextAsync = true;
}

function query($sql){
	$nextAsync = $this->nextAsync;
	$this->nextAsync = false;
	if(func_num_args() > 1){
		$args = func_get_args();
		$c = count($args);
		for($i=1; $i < $c; $i++){
			$v = $args[$i];
			if(strlen($v)==0){
				$args[$i] = $this->nulls;
			}elseif(!is_numeric($v)){
				$v = addslashes($v);
				if($this->autoquote){
					$v = "'".$v."'";
				}
				$args[$i] = $v;
			}
		}
		$sql = call_user_func_array('sprintf', $args);
	}
	$this->logger->debug($this->skip_extra_whitespace($sql).";");
	
	$flags = MYSQLI_STORE_RESULT;
	if($nextAsync){
		//$flags += MYSQLI_ASYNC;
	}
	$this->profiler->enter('mysqli_query');
	$res = mysqli_query($this->link, $sql, $flags);
	$this->profiler->leave('mysqli_query');
	if($res){
		return $res;
	}else{
		$err = mysqli_error($this->link);
		if(strstr($err, 'has gone away')){
		    $this->logger->warning('disconnected trying to reconnect...');
		    $this->connect();
		    if($res = mysqli_query($this->link, $sql, $flags)){
		        return $res;
		    }
		}
		$this->logger->error($err.": ".$this->skip_extra_whitespace($sql).";");
		$this->errors[] = $err;
		if($this->dieOnError){
			die($err);
		}
	}
}

function select($sql){
	$args = func_get_args();
	if($res = call_user_func_array(array($this, 'query'), $args)){
		return new MySqliSelect($res);
	}
}

function update($sql){
	$args = func_get_args();
	if($res = call_user_func_array(array($this, 'query'), $args)){
		return mysqli_affected_rows($this->link);
	}
}

function insert($sql){
	$args = func_get_args();
	if($res = call_user_func_array(array($this, 'query'), $args)){
		return mysqli_affected_rows($this->link);
	}
}

function insert_auto($sql){
	$args = func_get_args();
	if($res = call_user_func_array(array($this, 'query'), $args)){
		return $this->getInsertedId();
	}
}

function getInsertedId(){
	$select = $this->select("SELECT LAST_INSERT_ID() as id");
	$arr = $select->next();
	return $arr['id'];
}

function skip_extra_whitespace($str){
	return preg_replace('|\s+|',' ', $str);
}

}

class MySqliSelect {

var $res;

function MySqliSelect($res){
	$this->res = $res;
}

function next(){
	return mysqli_fetch_assoc($this->res);
}

}
?>