<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class PersonalPeriodPlansDAO {

var $profiler;
var $connection;
var $connectionHelper;
var $splitter;
var $queries = array(
	'select'=>"SELECT ppp.personal_period, ppp.resource_type, ppp.use_plan, ppp.use_limit, ppp.use_actual, ppp.use_value
				FROM personal_period_plans ppp
				LEFT JOIN personal_periods pp ON pp.id=ppp.personal_period
				WHERE (pp.state=1 || pp.marker=1)",
	'select_by_period'=>"SELECT ppp.personal_period, ppp.resource_type, ppp.use_plan, ppp.use_limit, ppp.use_actual, ppp.use_value
				FROM personal_period_plans ppp
				LEFT JOIN personal_periods pp ON pp.id=ppp.personal_period
				WHERE pp.id=%s AND (pp.state=1 || pp.marker=1)",
	'update'=>"UPDATE personal_period_plans SET use_actual=%s WHERE personal_period=%s AND resource_type=%s",
	'insert'=>"INSERT INTO personal_period_plans (personal_period, resource_type, use_plan, use_limit) VALUES (%s, %s, %s, %s)",
	'prepare_update_actual_uses'=>'CREATE TEMPORARY table pp_uses (
		id INT UNSIGNED,
		resource_type INT UNSIGNED,
		value BIGINT UNSIGNED,
		cost BIGINT UNSIGNED,
		PRIMARY KEY(id, resource_type)
	) ENGINE=Memory',
	'select_actual_uses'=>'SELECT id, resource_type, SUM(value) as value, SUM(cost) cost FROM (
			SELECT pp.id, ru.resource_type, SUM(ppru.amount) AS value, CEIL(SUM(ppru.amount) * ru.price / ru.unit_size) AS cost
			FROM personal_period_resource_uses ppru
			LEFT JOIN personal_periods pp ON pp.id=ppru.personal_period
			LEFT JOIN resource_uses ru ON ru.id=ppru.resource_use
			LEFT JOIN clients c ON c.id=ru.client
			WHERE ru.src = c.account AND pp.state=1 AND pp.period_end > SUBDATE(NOW(), INTERVAL 1 HOUR)
			GROUP BY pp.id, ru.id
		) x GROUP BY id, resource_type',
	'update_actual_uses'=>'UPDATE pp_uses t
		LEFT JOIN personal_period_plans ppp ON t.id=ppp.personal_period AND t.resource_type=ppp.resource_type
		LEFT JOIN personal_periods pp ON pp.id=ppp.personal_period
		SET ppp.use_value=t.value, ppp.use_actual=t.cost
		WHERE pp.state=1',
);

function setConnection($connection){
	$this->connection = $connection;
}

function setConnectionHelper($obj){
	$this->connectionHelper = $obj;
}

function setProfiler($obj){
	$this->profiler = $obj;
}

function setSplitter($splitter){
	$this->splitter = $splitter;
}

function iterator(){
	$q = $this->queries['select'];
	$q = $this->splitter->patch($q." AND pp.client >= [client_from] AND pp.client < [client_to]", $q);
	return $this->connection->select($q);
}

function iteratorByPeriod(){
	return $this->connection->select($this->queries['select_by_period']);
}

function update($pid, $rtid, $amount){
//	$this->profiler->enter('dao.ppp.update');
	$res = $this->connection->update($this->queries['update'], $amount, $pid, $rtid);
//	$this->profiler->leave('dao.ppp.update');
	return $res;
}

function insert($pid, $rtid, $plan, $limit){
	return $this->connection->insert($this->queries['insert'], $pid, $rtid, $plan, $limit);
}

function updateActualUses(){
	$this->connection->update($this->queries['prepare_update_actual_uses']);
	$q = $this->queries['select_actual_uses'];
	$q = $this->splitter->patch(str_replace("WHERE", "WHERE pp.client >= [client_from] AND pp.client < [client_to] AND ", $q), $q);
	$this->connectionHelper->insertSelectOverFile('pp_uses', $q);
	$this->connection->update($this->queries['update_actual_uses']);
}

}
?>