<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class MailProcessor {

var $server;
var $user;
var $password;
var $messageParser;
var $logger;
var $connection;
var $operator;
var $delegates = array();

function setLoggerFactory($obj){
	$this->logger = $obj->getLogger('mail.processor');
}

function setServer($str){
	$this->server = $str;
}

function setUser($str){
	$this->user = $str;
}

function setPassword($str){
	$this->password = $str;
}

function setMessageParser($obj){
	$this->messageParser = $obj;
}

function setConnection($obj){
	$this->connection = $obj;
}

function setOperator($i){
	$this->operator = $i;
}

function setDelegates($arr){
	$this->delegates = $arr;
}

function process(){
	if($mbox = imap_open($this->server, $this->user, $this->password, OP_SILENT)){
		$this->messageParser->setMbox($mbox);
		if ($check = imap_check($mbox)) {
			if($check->Nmsgs > 0){
				$overview = imap_fetch_overview($mbox,"1:".$check->Nmsgs, 0);
				foreach($overview as $info){
					$msg = $this->messageParser->parse($info);
					$this->processMessage($msg);
				}
				imap_delete($mbox, "1:*");
				$this->logger->info('%s messages processed', $check->Nmsgs);
			}else{
				$this->logger->info('now new messages');
			}
		} else {
			$this->logger->error("Can't check mailbox %s @ %s", $this->user, $this->server);
		}
		imap_expunge($mbox);
		imap_close($mbox);
	}else{
		$this->logger->error("Can't open mailbox %s @ %s", $this->user, $this->server);
	}
}

function processMessage($msg){
	foreach($this->delegates as $delegate){
		if($delegate->process($msg)){
			return;
		}
	}
	$sel = $this->connection->select("SELECT id FROM mail_messages WHERE msg_id='%s'", $msg['msg_id']);
	if($sel->next()){
		$this->logger->warning("message %s already exists", $msg['msg_id']);
	}else{
		//print_r($msg);
		if($msg['answer_id']){
			$sel = $this->connection->select("SELECT id FROM mail_messages WHERE answer_id='%s'", $msg['answer_id']);
			if($arr = $sel->next()){
				$this->logger->info("found a message with answer id %s, writing message id %s", $msg['answer_id'], $msg['msg_id']);
				$this->connection->update("UPDATE mail_messages SET msg_id='%s' WHERE id='%s'", $msg['msg_id'], $arr['id']);
			}else{
				$this->logger->warning("message with answer id %s not found", $msg['answer_id']);
				$this->storeMessage($msg);
			}
		}else{
			$this->storeMessage($msg);
		}
	}
}

function storeMessage($msg){
	$this->logger->info("storing message %s", $msg['msg_id']);
	$this->connection->query("BEGIN");
	// detect client
	$client = 'NULL';
	if(preg_match('|([a-z0-9])+([.\w-])*@([\w-])+(\.[\w-]+)+|i', $msg['addr_from'], $m)){
		$sel = $this->connection->select("SELECT client FROM mail_messages WHERE addr_from LIKE '%%%s%%' AND client IS NOT NULL LIMIT 1", $m[0]);
		if($arr = $sel->next()){
			$client = $arr['client'] ? $arr['client'] : 'NULL';
		}
	}
	$stamp = $this->connection->insert_auto("INSERT INTO stamps(created, created_by) VALUES (now(), '%s')", $this->operator);
	$mid = $this->connection->insert_auto("INSERT INTO mail_messages
		(subject, addr_from, addr_to, reply_to, date, msg_id, in_reply_to, body, attachments, stamp, client) VALUES
		('%s', '%s', '%s', '%s', FROM_UNIXTIME(%s), '%s', '%s', '%s', %s, %s, %s)",
		$msg['subject'], $msg['addr_from'], $msg['addr_to'], $msg['reply_to'], $msg['date'], $msg['msg_id'], $msg['in_reply_to'], $msg['body'], count($msg['attachments']), $stamp, $client);
	foreach($msg['attachments'] as $att){
		$fid = $this->connection->insert_auto("INSERT INTO billing_storage.files (name, size, type, filedate)
			VALUES ('%s', '%s', '%s', FROM_UNIXTIME(%s))",
			$att['name'], $att['size'], strtolower($att['type']), $msg['date']);
		$pointer = 0;
		while($pointer < $att['size']){
			$data = substr($att['body'], $pointer, 65535);
			$this->connection->insert("INSERT INTO billing_storage.file_chunks (file_id, chunk) VALUES ('%s', '%s')", $fid, $data);
			$pointer += 65535;
		}
		$this->connection->insert("INSERT INTO mail_attachments (message, file) VALUES ('%s', '%s')", $mid, $fid);
	}
	if(!$this->connection->hasErrors()){
		$this->connection->query("COMMIT");
	}else{
		$this->logger->info("message %s can't be stored", $msg['msg_id']);
		$this->connection->query("ROLLBACK");
	}
}

}
?>