<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class AccessTrackerService {

var $logger;
var $accessTrackerDAO;
var $connectionPolicy;
var $clientsManager;
var $hostsManager;
var $timeService;
var $personalPeriodsManager;
var $delayedTasksManager;

var $states;
var $queue;
var $time;
var $loaded;

function setLoggerFactory($obj){
	$this->logger = $obj->getLogger('access.tracker');
}

function setAccessTrackerDAO($obj){
	$this->accessTrackerDAO = $obj;
}

function setConnectionPolicy($obj){
	$this->connectionPolicy = $obj;
}

function setClientsManager($obj){
	$this->clientsManager = $obj;
}

function setHostsManager($obj){
	$this->hostsManager = $obj;
}

function setPersonalPeriodsManager($obj){
	$this->personalPeriodsManager = $obj;
}

function setDelayedTasksManager($obj){
	$this->delayedTasksManager = $obj;
}

function setTimeService($obj){
	$this->timeService = $obj;
}

function load(){
	if(!$this->loaded){	
		$this->clientsManager->load();
		$this->personalPeriodsManager->load();
		$this->connectionPolicy->load();
		$this->hostsManager->load();
		$this->states = array();
		$it = $this->accessTrackerDAO->listStates();
		while($arr = $it->next()){
			$this->states[$arr['client']] = $arr;
		}
		$this->logger->debug("%s track states loaded", count($this->states));
		$this->loaded = true;
	}
}

function getState($clid){
	if(isset($this->states[$clid])){
		return $this->states[$clid];
	}
}

function track(){
	$this->logger->info("starting tracking");
	$this->load();
	$this->time = $this->timeService->getTime();
	$this->queue = array();
	$it = $this->accessTrackerDAO->listQueue();
	while($arr = $it->next()){
		$this->queue[$arr['modem']] = $arr;
	}
	$this->logger->debug("%s queue items loaded", count($this->queue));
	$this->clientsManager->walk(array($this, 'trackClient'));
	$succeed = 0;
	$failed = 0;
	$new = 0;
	$waiting = 0;
	foreach($this->queue as $modem=>$arr){
		$this->accessTrackerDAO->touch();
		$create_task = false;
		if($arr['status']==2){ // success
			$succeed++;
			if(!isset($arr['re'])){
				$this->accessTrackerDAO->removeFromQueue($modem);
			}
		}
		if(isset($arr['re']) || !$arr['task']){ //    ,      ,   
			$create_task = true;
			$new++;
		}
		if($arr['status']==3){ // fail
			$failed++;
			$create_task = true;
		}
		if($create_task){
			$task = $this->delayedTasksManager->appendOffline('3', 'task.build.profile '.$modem, 1, 1);
			$this->accessTrackerDAO->appendToQueue($modem, $arr['added'], $task);
		}
		if($arr['status']==1){
			$waiting++;
		}
	}
	$this->logger->info("tracking finished: %d modems successfully restarted, %d modems failed, %d modems added, %d modems in process", $succeed, $failed, $new, $waiting);
}

function trackClient($client){
	$clid = $client['id'];
	$state = $this->connectionPolicy->isClientDisconnected($clid) ? 0 : 1;
	if($period = $this->personalPeriodsManager->getClientPersonalPeriod($clid)){
		$con_type = $period['connection_type'];
	}else{
		$con_type = 0;
	}
	if(!isset($this->states[$clid]) || $this->states[$clid]['state']!=$state){
		$this->hostsManager->walkClientHosts($clid, array($this, 'scheduleHostModem'));
		$this->accessTrackerDAO->saveState($clid, $state, $con_type);
	}elseif($this->states[$clid]['connection_type']!=$con_type){
		if(strlen($this->states[$clid]['connection_type']) > 0){ //   , ,    (    )
			$this->hostsManager->walkClientHosts($clid, array($this, 'scheduleHostModem'));
		}
		$this->accessTrackerDAO->saveState($clid, $state, $con_type);
	}
}

function scheduleHostModem($host){
	if(!isset($this->queue[$host['modem']])){
		$this->queue[$host['modem']] = array(
			'modem' => $host['modem'],
			'added' => $this->time,
			'task' => '',
			'status' => '',
		);
	}else{
		$this->queue[$host['modem']]['re'] = 1;
	}
}

}
?>