<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class BonusesManager {

var $logger;
var $bonusesDAO;
var $bonuses;
var $sorted;
var $accountsManager;
var $accountTypesManager;
var $timeService;
var $calculator;
var $loaded;

function setLoggerFactory($obj){
	$this->logger = $obj->getLogger('bonuses');
}

function setBonusesDAO($obj){
	$this->bonusesDAO = $obj;
}

function setTimeService($obj){
	$this->timeService = $obj;
}

function setCalculator($obj){
	$this->calculator = $obj;
}

function setAccountsManager($obj){
	$this->accountsManager = $obj;
}

function setAccountTypesManager($obj){
	$this->accountTypesManager = $obj;
}

function load(){
	if(!$this->loaded){
		$time = $this->timeService->getTime();
		$this->bonuses = array();
		$this->sorted = array();
		$it = $this->bonusesDAO->iterator();
		$counter = 0;
		while($arr = $it->next()){
			$arr['actual'] = (!$arr['actual_from'] || ($arr['actual_from'] <= $time)) && (!$arr['actual_to'] || ($arr['actual_to'] >= $time));
			$this->bonuses[$arr['client']][$arr['account_type']] = $arr;
			$counter++;
		}
		$this->logger->debug("%s bonuses loaded", $counter);
		$this->loaded = true;
	}
}

function checkExpiredBonuses(){
	//$this->logger->info("begin check expired");
	$time = $this->timeService->getTime();
	foreach($this->bonuses as $clid=>$bonuses){
		foreach($bonuses as $bonus){
			if($bonus['actual_to'] && ($bonus['actual_to'] < $time) && $bonus['lost_dst']){
				$rest = $this->accountsManager->getVirtualState($bonus['account']);
				if($this->calculator->cmp($rest, '0') > 0){
					$type = $this->accountTypesManager->getAccountType($bonus['account_type']);
					$this->logger->info("client[%s] bonus type '%s' expired, clearing", $clid, $type['name']);
					$this->accountsManager->actualMove($bonus['account'], $bonus['lost_dst'], $rest);
				}
			}
		}
	}
	//$this->logger->info("end check expired");
}

function getSortedBonuses($client){
	if(isset($this->bonuses[$client])){
		if(!isset($this->sorted[$client])){
			uasort($this->bonuses[$client], array($this, 'compareBonuses'));
			$this->sorted[$client] = 1;
		}
		return $this->bonuses[$client];
	}else{
		return array();
	}
}

function compareBonuses($b1, $b2){
	$t1 = $this->accountTypesManager->getAccountType($b1['account_type']);
	$t2 = $this->accountTypesManager->getAccountType($b2['account_type']);
	$p1 = $t1['priority'];
	$p2 = $t2['priority'];
	return $p2 - $p1;
}

function walkClientActualBonuses($clid, $callback){
	foreach($this->getSortedBonuses($clid) as $bonus){
		if($bonus['actual']){
			call_user_func($callback, $bonus);
		}
	}
}

function walkClientActualHighBonuses($clid, $callback){
	foreach($this->getSortedBonuses($clid) as $bonus){
		if($bonus['actual'] && ($bonus['priority'] >= 0)){
			call_user_func($callback, $bonus);
		}
	}
}

function walkClientActualLowBonuses($clid, $callback){
	foreach($this->getSortedBonuses($clid) as $bonus){
		if($bonus['actual'] && ($bonus['priority'] < 0)){
			call_user_func($callback, $bonus);
		}
	}
}

function getBonus($clid, $tid){
	if(isset($this->bonuses[$clid][$tid])){
		return $this->bonuses[$clid][$tid];
	}
}

function createBonus($account, $tid, $clid){
	$type = $this->accountTypesManager->getAccountType($tid);
	$this->bonusesDAO->insert($account, $tid, $clid);
	$this->bonuses[$clid][$tid] = array(
		'account'=>$account,
		'account_type'=>$tid,
		'client'=>$clid,
		'actual_from'=>'',
		'actual_to'=>'',
		'actual'=>1,
		'name'=>$type['name'],
	);
	$this->logger->debug("bonus created: [%s] (cl_id=%s, t_id=%s)", $account, $clid, $tid);
	unset($this->sorted[$clid]);
	return $this->bonuses[$clid][$tid];
}

function show(){
	print_r($this->bonuses);
}

}
?>