<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class Chainer {

var $logger;
var $cli;
var $tableOption;
var $chainOption;
var $locationOption;
var $maskOption;
var $targetOption;
var $keepFirstOption;
var $keepLastOption;
var $workDirOption;
var $ipSource;
var $workdir;

function setLogFactory($logFactory){
	$this->logger = $logFactory->getLogger('chainer');
}

function setCommandLineInterface($obj){
	$this->cli = $obj;
}

function setTableOption($obj){
	$this->tableOption = $obj;
}

function setChainOption($obj){
	$this->chainOption = $obj;
}

function setLocationOption($obj){
	$this->locationOption = $obj;
}

function setTargetOption($obj){
	$this->targetOption = $obj;
}

function setKeepFirstOption($obj){
	$this->keepFirstOption = $obj;
}

function setKeepLastOption($obj){
	$this->keepLastOption = $obj;
}

function setWorkDirOption($obj){
	$this->workDirOption = $obj;
}

function setIpSource($obj){
	$this->ipSource = $obj;
}

function execute(){
	$this->cli->parse();
	$table = $this->tableOption->getValue();
	$chain = $this->chainOption->getValue();
	$loc = $this->locationOption->getValue();
	$target = $this->targetOption->getValue();
	$keepFirst = $this->keepFirstOption->getValue();
	$keepLast = $this->keepLastOption->getValue();
	$this->workdir = $this->workDirOption->getValue();
	if(substr($this->workdir, -1, 1)!='/'){
		$this->workdir .= '/';
	}
	$list = $this->listChain($table, $chain);
	if($keepLast==0){
		$keepLast = count($list);
	}else{
		$keepLast = -$keepLast;
	}
	$list = array_slice($list, $keepFirst, $keepLast);
	$data = array();
	$it = $this->ipSource->iterator();
	while($ip = $it->next()){
		$data[$ip] = 1;
	}
	$diff = $this->makeDiff($table, $chain, $keepFirst + 1, $list, $data, $loc=='src', $target);
	$this->apply($table, $chain, $diff);
}

function listChain($table, $chain){
	$filename = $this->workdir.'list-'.$table.'-'.$chain;
	$this->logger->debug("listing '%s' chain", $chain);
	$cmd = sprintf("iptables -t %s -L %s -n | grep -vE 'target|Chain' > %s", $table, $chain, $filename);
	$this->logger->debug($cmd);
	$list = array();
	exec($cmd, $out, $exit_code);
	if(!$exit_code){
		if($f = fopen($filename, "r")){
			while (!feof ($f)) {
				$line = trim(fgets($f, 255));
				if($line){
					$split = preg_split("|\s+|",$line, -1, PREG_SPLIT_NO_EMPTY);
					list($target, $prot, $opt, $src, $dst) = $split;
					$list[] = array($src, $dst, $target);
				}
			}
			fclose($f);
		}else{
			$this->logger->error("can't open '%s' for reading", $filename);
		}
	}else{
		$this->logger->warning("can't list chain '%s', probably empty, exit-code is %d, output: %s", $chain, $exit_code, implode(", ", $out));
	}
	return $list;
}

function makeDiff($table, $chain, $offset, $list, $data, $is_src, $target){
	$script = array();
	$script[] = '*'.$table;
	foreach($list as $existen){
		list($src, $dst, $tar) = $existen;
		$ip = $is_src ? $src : $dst;
		$valid = isset($data[$ip]) && $tar==$target;
		if($valid){
			unset($data[$ip]);
		}else{
			$script[] = sprintf("[0:0] -D %s -s %s -d %s -j %s", $chain, $src, $dst, $tar);
		}
	}
	$location = $is_src ? '-s' : '-d';
	foreach($data as $ip=>$x){
		$script[] = sprintf("[0:0] -I %s %s %s %s -j %s", $chain, $offset, $location, $ip, $target);
	}
	$script[] = "COMMIT\n";
	return $script;
}

function apply($table, $chain, $script){
	$count = count($script) - 2;
	$filename = $this->workdir.'diff-'.$table.'-'.$chain;
	if($file = @fopen($filename, "w+")){
		$this->logger->debug("Writing diff to '%s'", $filename);
		fputs($file, implode("\n", $script));
		fclose($file);
		if($count > 0){
			$this->logger->debug("Applying %s difference rules", $count);
			$cmd = sprintf("iptables-restore --noflush 2>&1 < %s", $filename);
			$this->logger->debug($cmd);
			exec($cmd, $out, $exit_code);
			if($exit_code){
				$this->logger->error("restoring failed, exit code is %d, output: %s", $exit_code, implode(",", $out));
			}
		}else{
			$this->logger->debug("No differences");
		}
	}else{
		$this->logger->error("Can't write to '%s'", $filename);
	}
}

}
?>