<?
class RulesBuilder {

var $logger;
var $timeService;
var $clientsDAO;
var $clientsManager;
var $connectionTypesManager;

var $connectionPolicy;
var $blockPolicy;
var $hostsManager;
var $personalFirewallManager;
var $personalPeriodsManager;

var $rulesManager;
var $workdir;
var $fetchPeriod = 5;

var $interval;
var $personalPeriod;
var $client;
var $connectionType;
var $host;
var $ctRule;
var $ctResource;
var $rule;
var $tariff;
var $rest;
var $profiler;
var $ipSetBuildHelper;
var $spawner;
var $splitter;

function setSplitter($obj){
	$this->splitter = $obj;
}

function setSpawner($obj){
	$this->spawner = $obj;
}

function setIpSetBuildHelper($obj){
	$this->ipSetBuildHelper = $obj;
}

function setProfiler($obj){
	$this->profiler = $obj;
}

function setLogFactory($logFactory){
	$this->logger = $logFactory->getLogger('rules.builder');
}

function setTimeService($obj){
	$this->timeService = $obj;
}

function setClientsManager($obj){
	$this->clientsManager = $obj;
}

function setClientsDAO($obj){
	$this->clientsDAO = $obj;
}

function setConnectionTypesManager($obj){
	$this->connectionTypesManager = $obj;
}

function setConnectionPolicy($obj){
	$this->connectionPolicy = $obj;
}

function setBlockPolicy($obj){
	$this->blockPolicy = $obj;
}

function setHostsManager($obj){
	$this->hostsManager = $obj;
}

function setIpUseLogManager($obj){
	$this->ipUseLogManager = $obj;
}

function setPersonalPeriodsManager($obj){
	$this->personalPeriodsManager = $obj;
}

function setPersonalFirewallManager($obj){
	$this->personalFirewallManager = $obj;
}

function setRulesManager($obj){
	$this->rulesManager = $obj;
}

function setWorkdir($str){
	$this->workdir = $str;
}

function setFetchPeriod($i){
	$this->fetchPeriod = $i;
}

function writeTime(){
	$filename = $this->workdir.'/accounting-make-date';
	if($file = @fopen($filename, "w+")){
		$this->logger->debug("Writing time to '%s'", $filename);
		fputs($file, $this->timeService->getTime());
		fclose($file);
	}
}

function readTime(){
	$filename = $this->workdir.'/accounting-make-date';
	if($file = @fopen($filename, "r")){
		$this->logger->debug("Reading time from '%s'", $filename);
		$time = fgets($file);
		fclose($file);
		return $time;
	}
}

function make(){
	$opt = getopt('w:a:t:x:');
	if(isset($opt['t'])){
		$this->timeService->setTime($opt['t']);
	}
	$split = max(1, isset($opt['x']) ? intval($opt['x']) : 1);
	$this->writeTime();
	$this->ipSetBuildHelper->prepare();
	$this->profiler->enter('rules.builder');
	$this->makeSplit($split);
	$this->ipSetBuildHelper->process();
	$this->profiler->leave('rules.builder');
	$this->profiler->dump();
}

function makeSplit($split){
	$time = $this->timeService->getTime();
	$pids = array();
	$count = $this->clientsDAO->getCount();
	$size = intval($count / $split);
	$prev = 1;
	for($i = 1; $i <= $split; $i++){
		$from = $prev;
		if($i==$split){
			$to = $this->clientsDAO->getLastId() + 1;
		}else{
			$to = $this->clientsDAO->getShiftId($i * $size);
		}
		$cmd = $_SERVER['argv'][0].' -a make_slave -r '.$from.':'.$to.' -t '.$time;
		$pids[] = $this->spawner->spawn($cmd);
		$prev = $to;
	}
	$this->spawner->waitFor($pids);
}

function makeSlave(){
	$opt = getopt('w:a:r:t:');
	if(!isset($opt['r'])){
		$this->logger->error('set range by -r option');
		return;
	}
	if(isset($opt['t'])){
		$this->timeService->setTime($opt['t']);
	}
	$range = explode(":", $opt['r']);
	if(count($range)!=2){
		$this->logger->error('wrong range');
	}
	$range = array(intval($range[0]), intval($range[1]));
	$this->splitter->setClientsRange($range);
	
	$this->buildAccountingRules();
}

function buildAccountingRules(){
	$this->clientsManager->load();
	$this->hostsManager->load();
	$this->personalPeriodsManager->load();
	$this->blockPolicy->load();
	$this->connectionPolicy->load();
	$this->personalFirewallManager->load();
	$this->logger->debug("begin building");
	$this->rulesManager->beginSetup();
	$this->clientsManager->walk(array($this, 'buildClient'));
	$this->rulesManager->finishSetup();
	$this->logger->debug("end building");
}

function buildClient($client){
	$this->client = $client;
	if(!$this->connectionPolicy->isClientDisconnected($client['id'])){
		if(($this->personalPeriod = $this->personalPeriodsManager->getClientPersonalPeriod($client['id'])) && $this->personalPeriod['state']==1){
			$this->connectionType = $this->connectionTypesManager->getConnectionType($this->personalPeriod['connection_type']);
			$this->hostsManager->walkClientHosts($client['id'], array($this, 'buildHost'));
		}
	}
}

function buildHost($host){
	$this->host = $host;
	$this->connectionTypesManager->walkResources($this->connectionType['id'], array($this, 'buildCtResource'));
}

function buildCtResource($ctResource){
	$this->ctResource = $ctResource;
	$this->ctResource['blocked'] = $this->blockPolicy->isResourceBlocked($this->client['id'], $this->ctResource['resource_type']);
	$this->connectionTypesManager->walkRules($ctResource['id'], array($this, 'buildCtRule'));
}

function buildCtRule($ctRule){
	$rule = $this->rulesManager->getRule($ctRule['rule']);
	$closed = $this->personalFirewallManager->isClosed($this->host['id'], $rule['id']);
	$block = $ctRule['blockable'] && ($this->ctResource['blocked'] || $closed);
	$this->rulesManager->setCounter($rule['id'], $this->host['ip'], $block);
}

}
?>