<?/* author: Ponomarev Denis <ponomarev@gmail.com> */

class NamedService {

var $connection;
var $optionsManager;
var $logger;
var $scriptExecutor;

function setLoggerFactory($obj){
	$this->logger = $obj->getLogger('task.dns.service');
}

function setConnection($obj){
	$this->connection = $obj;
}

function setOptionsManager($obj){
	$this->optionsManager = $obj;
}

function setScriptExecutor($obj){
	$this->scriptExecutor = $obj;
}

function FillDomainTemplate($darr, $hosts){
	$tmpl = $darr['host_template'];
	$items = array();
	foreach($hosts as $host){
		$name = $host['name'] ? $host['name'] : 'host-'.$host['id'];
		$items[] = str_replace(
			array("%name", "%ip"),
			array($name, $host['ip']),
			$tmpl);
	}
	$rows = implode("\n", $items);
	$config = str_replace(
		array("%version", "%hosts"),
		array($darr['version'], $rows),
		$darr['template'])."\n";
	return $config;
}

function getDomainMemebers($id){
	$members = array();
	foreach(array("hosts", "modems") as $table){
		$sel = $this->connection->select("SELECT * FROM $table WHERE domain='$id' AND POSITION('/' IN ip)=0 ORDER BY name");
		while($arr = $sel->next()){
			$arr['type'] = $table;
			$members[] = $arr;
		}
	}
	return $members;
}

function SaveDomain($darr, $config){
	global $db;
	$dirName = $this->optionsManager->getDirOptionValue('DNS_DIR');
	if($dirName){
		$fileName = $dirName.$darr['name'];
		if($fd = @fopen ($fileName, "w")){
			$writen = fwrite($fd, $config);
			fclose($fd);
			if($remote = $this->optionsManager->getTrimOptionValue('DNS_REMOTE')){
				exec('scp -p '.$fileName.' '.$remote.':'.$dirName);
			}
		}else{
			$this->logger->error('cant save into %s', $fileName);
		}
	}
}

function NextDomainVersion($prev){
	$date = date("Ymd");
	$version_date = substr($prev,0,8);
	if($version_date != $date){
		$version = $date.'01';
	}else{
		$version_num = substr($prev,8);
		$version_num = ($version_num + 1) % 100;
		if($version_num==0){
			$version_num = 1;
		}
		$version = $version_date.substr('0'.$version_num, -2);
	}
	return $version;
}

function RemakeDomain($id=0){
	$dirName = $this->optionsManager->getDirOptionValue('DNS_DIR');
	if($dirName){
		$date = date("Ymd");
		$sel = $this->connection->select("SELECT * FROM domains ".($id ? " WHERE id='$id'" : ""));
		while($darr = $sel->next()){
			$darr['version'] = $this->NextDomainVersion($darr['version']);
			$config = $this->FillDomainTemplate($darr, $this->getDomainMemebers($darr['id']));
			$this->SaveDomain($darr, $config);
			$this->connection->update("UPDATE domains SET version='".$darr['version']."' WHERE id=".$darr['id']);
		}
	}
}

// --- NETWORKS ---

function FillNetworkTemplate($narr, $hosts){
	$domains = array();
	$sel = $this->connection->select("SELECT id, name FROM domains");
	while($arr = $sel->next()){
		$domains[$arr['id']] = $arr['name'];
	}
	$parts = explode("/", $narr['ip']);
	$net = $parts[0];
	$mask = $parts[1];
	if (!is_numeric($mask)){
		$m = ip2long($mask);
		$bin = decbin($m);
		$mask = strpos($bin,'0');
	}
	$c = 4 - ceil((32 - $mask)/8);
	$tmpl = $narr['host_template'];
	$items = array();
	foreach($hosts as $host){
		$ip = implode(".", array_reverse(array_slice(explode(".", $host['ip']), $c)));
		$name = $host['name'] ? $host['name'] : 'host-'.$host['id'];
		$domain = isset($domains[$host['domain']]) ? $domains[$host['domain']] : '';
		$domainName = $name;
		if($domain){
			$domainName .= '.'.$domain;
		}
		$items[] = str_replace(
			array("%ip", "%name", "%domainName", "%domain"),
			array($ip, $name, $domainName, $domain ),
			$tmpl);
	}
	$rows = implode("\n", $items);
	$version = date("Ymd").substr('0'.$narr['version'], -2);
	$config = str_replace(
		array("%version", "%hosts"),
		array($version, $rows),
		$narr['template'])."\n";
	return $config;
}

function getNetworkMemebers($ip){
	$parts = explode("/", $ip);
	$net = $parts[0];
	$mask = $parts[1];
	$members = array();
	foreach(array("hosts", "modems") as $table){
		$sel = $this->connection->select("SELECT * FROM $table WHERE POSITION('/' IN ip)=0 ORDER BY ip");
		while($arr = $sel->next()){
			if($this->IpMatch($net, $mask, $arr['ip'])){
				$arr['type'] = $table;
				$members[] = $arr;
			}
		}
	}
	return $members;
}

function SaveNetwork($narr, $config){
	$dirName = $this->optionsManager->getDirOptionValue('DNS_BACK_DIR');
	if($dirName){
		$fileName = $dirName.$narr['filename'];
		if($fd = @fopen ($fileName, "w")){
			$writen = fwrite($fd, $config);
			fclose($fd);
			if($remote = $this->optionsManager->getTrimOptionValue('DNS_REMOTE')){
				exec('scp -p '.$fileName.' '.$remote.':'.$dirName);
			}
			$this->connection->update("UPDATE networks SET version=version+1 WHERE id=".$narr['id']);
		}else{
			$this->logger->error('cant save into %s', $fileName);
		}
	}
}

function RemakeNetwork($id=0){
	$dirName = $this->optionsManager->getDirOptionValue('DNS_BACK_DIR');
	if($dirName){
		$sel = $this->connection->select("SELECT * FROM networks".($id ? " WHERE id='$id'" : ""));
		while($narr = $sel->next()){
			$config = $this->FillNetworkTemplate($narr, $this->getNetworkMemebers($narr['ip']));
			$this->SaveNetwork($narr, $config);
		}
	}
}

function RestartDNS(){
	$command = $this->optionsManager->getOptionValue('DNS_RESTARTER');
	if($command){
		$this->logger->info('restarting DNS');
		$err = $this->scriptExecutor->ExecuteScript($command, $this->logger);
		return $err;
	}
}

function IpMatch($network, $mask, $ip) {
   $n = ip2long($network);
   $i = ip2long($ip);
   if (is_numeric($mask)) {// 10.10.10.0/24
       $m = ~((1<< (32-$mask))-1);
   }else{// 10.10.10.0/255.255.255.0
       $m = ip2long($mask); 
   }
   return !(($n & $m)-($i & $m));
}

}
?>