<?
class DBImport {

var $src;
var $dst;
var $logger;
var $tables;
var $databases;
var $keyColumns;
var $implicitForeignKeys = array();
var $foreignKeys;
var $columns;
var $imported = array();
var $path = array();
var $inherit = array();
var $showTables;
var $stick = array();
var $beforeImport = array();
var $afterImport = array();

function setLoggerFactory($obj){
	$this->logger = $obj->getLogger('dbi');
}

function setImplicitForeignKeys($arr){
	$this->implicitForeignKeys = $arr;
}

function setSource($obj){
	$this->src = $obj;
}

function setDestination($obj){
	$this->dst = $obj;
}

function setDatabases($arr){
	$this->databases = $arr;
}

function setTables($arr){
	$this->tables = $arr;
}

function setInherit($arr){
	$this->inherit = $arr;
}

function setStick($arr){
	$this->stick = $arr;
}

function setBeforeImport($arr){
	$this->beforeImport = $arr;
}

function setAfterImport($arr){
	$this->afterImport = $arr;
}

function getShowTables(){
	if(!isset($this->showTables)){
		$this->showTables = array();
		foreach($this->databases as $dst_db=>$src_db){
			$this->dst->query("USE %s", $dst_db);
			$sel = $this->dst->select("SHOW TABLES");
			while($arr = $sel->next()){
				$table = array_pop($arr);
				$this->showTables[] = $dst_db.'.'.$table;
			}
		}
		$this->dst->query("USE %s", $this->dst->database);
	}
	return $this->showTables;
}

function expandTables(){
	$show_tables = $this->getShowTables();
	$tables = array();
	foreach($this->tables as $table){
		foreach($show_tables as $show){
			if((strcmp($table, $show)==0) || @preg_match($table, $show)){
				$tables[$show] = 1;
			}
		}
	}
	return array_keys($tables);
}

function walkTables($callback){
	foreach($this->tables as $table){
		call_user_func($callback, $this, $table);
	}
}

function process(){
//	$this->prepareQuick();
	$this->tables = $this->expandTables();
	$this->deepList();
	$this->importTables();
}

function prepareQuick(){
	$this->prepareQuickForeignKeys();
	$this->prepareQuickKeyColumns();
}

function deepList(){
	$this->deepList = array();
	$this->walkTables(function($me, $table){ $me->deepTable($table); });
	//ksort($this->deepList);
	print_r($this->deepList);
}

function importTables(){
	$this->dst->query("BEGIN");
	$this->walkTables(function($me, $table){ $me->importTable($table); });
	$this->dst->query("COMMIT");
}

function deepTable($table){
	if(!isset($this->deepList[$table])){
		$this->deepList[$table] = 1;
		$fks = $this->getTableForeignKeys($table);
		foreach($fks as $fk){
			list($dst, $src_col, $dst_col) = $fk;
			$this->deepTable($dst);
		}
	}
}

function beforeImport($table, $arr){
	if(isset($this->beforeImport[$table])){
		list($func, $args) = $this->beforeImport[$table];
		if(method_exists($this, $func)){
			array_unshift($args, $table, $arr);
			call_user_func_array(array($this, $func), $args);
		}else{
			$this->logger->error("method %s does not exists", $func);
		}
	}
}

function afterImport($table, $arr, $new){
	if(isset($this->afterImport[$table])){
		list($func, $args) = $this->afterImport[$table];
		if(method_exists($this, $func)){
			array_unshift($args, $table, $arr, $new);
			call_user_func_array(array($this, $func), $args);
		}else{
			$this->logger->error("method %s does not exists", $func);
		}
	}
}

function afterClient($table, $old, $new, $rest_acc, $debt_acc, $date){
	$rest = $this->getSrcAccountState($old['account']);
	//$this->logger->debug($old['account'].": ".$rest);
	if($rest > 0){
		$this->dstMove($rest_acc, $new['account'], $rest, $date);
	}elseif($rest < 0){
		$this->dstMove($new['account'], $debt_acc, -$rest, $date);
	}
}

function dstMove($src, $dst, $amount, $date){
	$cp = $this->getDstCalendarPeriod();
	$this->dst->insert("INSERT INTO movements (calendar_period, src, dst, amount, date) VALUES ('%s', '%s', '%s', '%s', '%s')",
		$cp, $src, $dst, $amount, $date);
}

function getSrcAccountState($acc){
	$cp = $this->getSrcCalendarPeriod();
	$sel = $this->src->select("SELECT state_end FROM calendar_account_states WHERE account='%s' AND calendar_period='%s'", $acc, $cp);
	if($arr = $sel->next()){
		return $arr['state_end'];
	}
}

function getSrcCalendarPeriod(){
	if(!isset($this->srcCalendarPeriod)){
		$sel = $this->src->select("SELECT id FROM calendar_periods WHERE marker=1");
		$arr = $sel->next();
		$this->srcCalendarPeriod = $arr['id'];
	}
	return $this->srcCalendarPeriod;
}

function getDstCalendarPeriod(){
	if(!isset($this->dstCalendarPeriod)){
		$sel = $this->dst->select("SELECT id FROM calendar_periods WHERE marker=1");
		$arr = $sel->next();
		$this->dstCalendarPeriod = $arr['id'];
	}
	return $this->dstCalendarPeriod;
}

function importTable($table){
	$this->logger->info("importing %s", $table);
	$keyColumns = $this->getTableKeyColumns($table);
	if($pk = $this->getTablePrimaryKey($table)){
		$order_by = ' ORDER BY '.implode(",", $pk);
	}

	list($dst_db, $dst_table) = explode(".", $table);
	$src = $this->databases[$dst_db].'.'.$dst_table;
	$sel = $this->src->select("SELECT * FROM %s $order_by", $src);
	while($arr = $sel->next()){
		$this->importRecord($table, $arr);
	}
}

function stick($table, $id){
	if(isset($this->stick[$table])){
		foreach($this->stick[$table] as $child=>$field){
			list($dst_db, $dst_table) = explode(".", $child);
			$src = $this->databases[$dst_db].'.'.$dst_table;
			$sel = $this->src->select("SELECT * FROM %s WHERE %s='%s'", $src, $field, $id);
			while($arr = $sel->next()){
				$this->importRecord($child, $arr);
			}
		}
	}
}

function importRecord($table, $arr){
	array_push($this->path, $table);
	
	$new = array();
	$auto = $this->getTableAutoKey($table);
	$id = '';
	if($auto && (($x = $this->getInheritId($table, $arr[$auto])) >=0 )){
		$id = $x;
	}else{
		$columns = $this->getTableColumns($table);
		foreach($columns as $cinfo){
			$c = $cinfo['Field'];
			if(($c!=$auto) && isset($arr[$c])){
				$new[$c] = $arr[$c];
			}
		}
		$fks = $this->getTableForeignKeys($table);
		foreach($fks as $fk){
			list($dst, $src_col, $dst_col) = $fk;
			$cinfo = $columns[$src_col];
			$fk_val = isset($arr[$src_col]) ? $arr[$src_col] : '';
			if(isset($this->imported[$dst][$fk_val])){
				$new[$src_col] = $this->imported[$dst][$fk_val];
			}else{
				if($cinfo['Null']!='YES'){
					$new[$src_col] = $fk_val ? $this->getImportedRecordId($dst, $fk_val) : '';
				}else{
					unset($new[$src_col]);
				}
			}
		}
		if($auto && isset($this->imported[$table][$arr[$auto]])){
			return $this->imported[$table][$arr[$auto]];
		}
		$this->beforeImport($table, $arr);
		//insert
		$names = array();
		$values = array();
		foreach($columns as $cinfo){
			$c = $cinfo['Field'];
			if(isset($new[$c])){
				$names[] = $c;
				if((strlen($new[$c])==0) && ($cinfo['Null']=='YES')){
					$values[] = 'NULL';
				}else{
					$values[] = "'".addslashes($new[$c])."'";
				}
			}
		}
		$query = "INSERT INTO ".$table." (".implode(", ", $names).") VALUES (".implode(", ", $values).")";
		if($auto){
			$new[$auto] = $this->dst->insert_auto($query);
			$this->imported[$table][$arr[$auto]] = $new[$auto];
		}else{
			$this->dst->insert($query);
		}
		//update
		$update = array();
		foreach($fks as $fk){
			list($dst, $src_col, $dst_col) = $fk;
			$cinfo = $columns[$src_col];
			$fk_val = isset($arr[$src_col]) ? $arr[$src_col] : '';
			if(($cinfo['Null']=='YES') && $fk_val && !isset($new[$src_col])){
				$val = $this->getImportedRecordId($dst, $fk_val);
				$new[$src_col] = $val;
				$update[] = $src_col."=".($val ?: 'NULL');
			}
		}
		if(!empty($update)){
			if($pk = $this->getTablePrimaryKey($table)){
				$where = array();
				foreach($pk as $k){
					$where[] = $k."='".addslashes($new[$k])."'";
				}
				$query = "UPDATE ".$table." SET ".implode(", ", $update)." WHERE ".implode(" AND ", $where);
				$this->dst->update($query);
			}
		}
		$this->afterImport($table, $arr, $new);
		$id = $auto ? $new[$auto] : '';
		if($auto){
			$this->stick($table, $arr[$auto]);
		}
	}
	array_pop($this->path);
	return $id;
}

function getTableAutoKey($table){
	if($pk = $this->getTablePrimaryKey($table)){
		$columns = $this->getTableColumns($table);
		foreach($pk as $k){
			$cinfo = $columns[$k];
			if($cinfo['Extra']=='auto_increment'){
				return $k;
			}
		}
	}
}

function importRecordById($table, $id){
	if($rec = $this->getRecordById($table, $id)){
		return $this->importRecord($table, $rec);
	}
}

function getRecordById($table, $id){
	list($pk) = $this->getTablePrimaryKey($table);
	list($dst_db, $dst_table) = explode(".", $table);
	$src = $this->databases[$dst_db].'.'.$dst_table;
	$sel = $this->src->select("SELECT * FROM %s WHERE %s='%s'", $src, $pk, $id);
	return $sel->next();
}

function getInheritId($table, $id){
	$new_id = -1;
	if(isset($this->inherit[$table])){
		$inherit = $this->inherit[$table];
		if(isset($inherit[$id])){
			$new_id = $inherit[$id];
		}else{
			if(isset($inherit['*'])){
				$new_id = $inherit['*']=='*' ? $id : $inherit['*'];
			}
		}
	}
	return $new_id;
}

function getImportedRecordId($table, $id){
	if(!isset($this->imported[$table][$id])){
		$inherit = $this->getInheritId($table, $id);
		$this->imported[$table][$id] = $inherit >= 0 ? $inherit : $this->importRecordById($table, $id);;
	}
	return $this->imported[$table][$id];
}

function getTablePrimaryKeyValue($table, $arr){
	if($pk = $this->getTablePrimaryKey($table)){
		$val = array();
		foreach($pk as $k){
			$val[$k] = $arr[$k];
		}
		return $val;
	}
}

function getTablePrimaryKey($table){
	$keyColumns = $this->getTableKeyColumns($table);
	if(isset($keyColumns['PRIMARY'])){
		return $keyColumns['PRIMARY'];
	}
}

function getTableForeignKeys($table){
	if(!isset($this->foreignKeys)){
		$this->foreignKeys = $this->readForeignKeys();
	}
	if(!isset($this->foreignKeys[$table])){
		$this->foreignKeys[$table] = array();
	}
	return $this->foreignKeys[$table];
}

function prepareQuickForeignKeys(){
	$this->src->query("DROP TABLE IF EXISTS quick.REFERENTIAL_CONSTRAINTS");
	$this->src->query("CREATE TABLE quick.REFERENTIAL_CONSTRAINTS (
		CONSTRAINT_SCHEMA varchar(255),
		TABLE_NAME varchar(255),
		REFERENCED_TABLE_NAME varchar(255),
		CONSTRAINT_NAME varchar(255)
	) ENGINE=InnoDB");
	foreach($this->databases as $dst_db=>$src_db){
		$this->src->query("
			INSERT INTO quick.REFERENTIAL_CONSTRAINTS
			SELECT '%s', TABLE_NAME, REFERENCED_TABLE_NAME, CONSTRAINT_NAME
			FROM INFORMATION_SCHEMA.REFERENTIAL_CONSTRAINTS
			WHERE CONSTRAINT_SCHEMA='%s'", $dst_db, $dst_db);
	}
}

function readForeignKeys(){
	$result = $this->implicitForeignKeys;
	foreach($this->databases as $dst_db=>$src_db){
		$this->logger->info("read foreign keys from %s", $dst_db);
		$sel = $this->dst->select("SELECT TABLE_NAME src, REFERENCED_TABLE_NAME dst, CONSTRAINT_NAME fk
			FROM quick.REFERENTIAL_CONSTRAINTS
			WHERE CONSTRAINT_SCHEMA='%s'", $dst_db);
		while($arr = $sel->next()){
			$keyColumns = $this->getTableKeyColumns($dst_db.'.'.$arr['src']);
			list($src_col, $dst, $dst_col) = $keyColumns[$arr['fk']];
			$result[$dst_db.'.'.$arr['src']][$arr['fk']] = array($dst, $src_col, $dst_col);
		}
	}
	return $result;
}

function getTableKeyColumns($table){
	if(!isset($this->keyColumns)){
		$this->keyColumns = $this->readKeyColumns();
	}
	if(!isset($this->keyColumns[$table])){
		$this->keyColumns[$table] = array();
	}
	return $this->keyColumns[$table];
}

function prepareQuickKeyColumns(){
	$this->src->query("DROP TABLE IF EXISTS quick.KEY_COLUMN_USAGE");
	$this->src->query("CREATE TABLE quick.KEY_COLUMN_USAGE (
		CONSTRAINT_SCHEMA varchar(255),
		TABLE_NAME varchar(255),
		CONSTRAINT_NAME varchar(255),
		COLUMN_NAME varchar(255),
		REFERENCED_TABLE_NAME varchar(255),
		REFERENCED_COLUMN_NAME varchar(255)
	) ENGINE=InnoDB");
	foreach($this->databases as $dst_db=>$src_db){
		$this->src->query("
			INSERT INTO quick.KEY_COLUMN_USAGE
			SELECT '%s', TABLE_NAME, CONSTRAINT_NAME, COLUMN_NAME, REFERENCED_TABLE_NAME, REFERENCED_COLUMN_NAME
			FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE
			WHERE CONSTRAINT_SCHEMA='%s'
			ORDER BY TABLE_NAME, CONSTRAINT_NAME, ORDINAL_POSITION, POSITION_IN_UNIQUE_CONSTRAINT", $dst_db, $dst_db);
	}
}

function readKeyColumns(){
	$result = array();
	foreach($this->databases as $dst_db=>$src_db){
		$this->logger->info("read key columns from %s", $dst_db);
		$sel = $this->dst->select("SELECT TABLE_NAME src, CONSTRAINT_NAME fk, COLUMN_NAME src_col, REFERENCED_TABLE_NAME dst, REFERENCED_COLUMN_NAME dst_col
			FROM quick.KEY_COLUMN_USAGE
			WHERE CONSTRAINT_SCHEMA='%s'
			ORDER BY TABLE_NAME, CONSTRAINT_NAME", $dst_db);
		while($arr = $sel->next()){
			if($arr['dst']){
				$result[$dst_db.'.'.$arr['src']][$arr['fk']] = array($arr['src_col'], $dst_db.'.'.$arr['dst'], $arr['dst_col']) ;
			}elseif($arr['fk']=='PRIMARY'){
				$result[$dst_db.'.'.$arr['src']][$arr['fk']][] = $arr['src_col'];
			}
		}
	}
	return $result;
}

function getTableColumns($table){
	if(!isset($this->columns[$table])){
		$this->columns[$table] = $this->readTableColumns($table);
	}
	return $this->columns[$table];
}

function readTableColumns($table){
	$this->logger->info("read columns from %s", $table);
	$result = array();
	$sel = $this->dst->select("SHOW COLUMNS FROM %s", $table);
	while($arr = $sel->next($sel)){
		$result[$arr['Field']] = $arr;
	}
	return $result;
}

}
?>