<?
include_once "includes/environment.php";
include_once "includes/core.php";
include_once "functions/accounts.php";
include_once "classes/xml/StringXmlParser.php";
include_once "classes/xml/XmlNodeDelegate.php";
include_once "classes/tools/PhpTemplate.php";
include_once "classes/log/FileAppender.php";
include_once "classes/log/DefaultLogger.php";

class WrongXmlException extends Exception {}

class WrongSignException extends Exception {}

class EasyHandler {

var $pub_key;
var $priv_key;
var $token;
var $datetime;
var $account = '';
var $codes = array(
	'wrong_xml' => -1,
	'wrong_token' => -2,
	'wrong_sign' => -99,
	'wrong_operation' => -3,
	'account_not_found' => -4,
	'pay_not_found' => -5,
	'cant_pay' => -6,
	'cant_confirm' => -7,
	'cant_cancel' => -8,
);
var $logger;

function EasyHandler($logger){
	$this->logger = $logger;
	$this->token =  getTrimOption('EASYSOFT_TOKEN');
	$this->pub_key = getTrimOption('EASYSOFT_EASY_PUBLIC');
	$this->priv_key = getTrimOption('EASYSOFT_PROVIDER_PRIVATE');
	if($arr = loadRecord("SELECT account FROM system_accounts WHERE code='PAY_EASY'")){
		$this->account = $arr['account'];
	}
}

function uniName($name){
	$name = strtolower($name);
	$arr = explode(':', $name);
	return count($arr)==2 ? $arr[1] : $arr[0];
}

function isTag($node, $name){
	return $this->uniName($node->name) == $this->uniName($name);
}

function getInnerSafe($node, $name){
	foreach($node->nodes as $inner){
		if($this->isTag($inner, $name)){
			return $inner;
		}
	}
	throw new WrongXmlException($name.' expected');
}

function getInnerSafeValue($node, $name){
	$node = $this->getInnerSafe($node, $name);
	return $node->body;
}

function mapNodes($nodes){
	$map = array();
	foreach($nodes as $n){
		$map[$this->uniName($n->name)] = $n;
	}
	return $map;
}

function mapValues($nodes){
	$map = array();
	foreach($nodes as $n){
		$map[$this->uniName($n->name)] = $n->body;
	}
	return $map;
}

function handle($xml){
	global $beans;
	if(!$this->account || !$this->pub_key || !$this->priv_key || !$this->token){
		$this->logger->warning('unconfigured (account:PAY_EASY, config: EASYSOFT_EASY_PUBLIC, EASYSOFT_PROVIDER_PRIVATE, EASYSOFT_TOKEN)');
		return;
	}
	try{
		$parser = new StringXmlParser();
		$parser->setEncoding("UTF-8");
		$parser->setOptions(array(
			XML_OPTION_CASE_FOLDING => 0,
			XML_OPTION_SKIP_WHITE => 1,
		));
		$parser->setLogFactory($beans->getBean('loggerFactory'));
		$delegate = new XmlNodeDelegate();
		if(!$parser->parse($delegate, $xml)){
			throw new WrongXmlException('cant parse xml');
		}
		$root = $delegate->node;
		$request = $this->getInnerSafe($root, 'request');
		$token = $this->getInnerSafeValue($request, 'token');
		$this->datetime = $this->getInnerSafeValue($request, 'datetime');
		$sign = $this->getInnerSafe($request, 'sign');
		$this->verify_sign($xml, $sign->body);
/*
		if($this->token != $token){
			$this->logger->error('wrong token: '.$token);
			$this->response($this->codes['wrong_token'], ' Token');
			return;
		}
*/
		$map = $this->mapNodes($request->nodes);
		if(isset($map['check'])){
			$this->handleCheck($map['check']);
		}elseif(isset($map['payment'])){
			$this->handlePayment($map['payment']);
		}elseif(isset($map['confirm'])){
			$this->handleConfirm($map['confirm']);
		}elseif(isset($map['clearing'])){
			$this->handleClearing($map['clearing']);
		}else{
			throw new WrongXmlException('operation not defined');
		}
	}catch(WrongXmlException $ex){
		$this->logger->error('wrong xml: '. $ex->getMessage());
		$this->logger->error($xml);
		$this->response($this->codes['wrong_xml'], ' XML');
	}catch(WrongSignException $ex){
		$this->logger->error('wrong sign');
		$this->response($this->codes['wrong_sign'], '  ');
	}
}

function verify_sign($xml, $sign){
	$xml = str_replace($sign, '', $xml);
	$bin_sign = pack("H*", $sign);
	$result = openssl_verify($xml, $bin_sign, $this->pub_key);
	switch($result){
		case 0:
			throw new WrongSignException();
			break;
		case -1:
			$this->logger->error('cant verify sign');
			throw new WrongSignException();
			break;
	}
}

function _verify_sign($xml, $sign){
	$xml = str_replace($sign, '', $xml);
	$sha1_calc = sha1($xml);
	$bin_sign = pack("H*", $sign);
	openssl_public_decrypt($bin_sign, $sha1_rcvd, $this->pub_key);
	$sha1_rcvd = bin2hex($sha1_rcvd);
	if(strcasecmp($sha1_calc, $sha1_rcvd)!=0){
		$this->logger->info('calc: '.$sha1_calc);
		$this->logger->info('rcvd: '.$sha1_rcvd);
		throw new WrongSignException();
	}
}

function handleCheck($operation){
	$service_id = $this->getInnerSafeValue($operation, 'serviceid');
	$account = $this->getInnerSafeValue($operation, 'account');
	$this->logger->info('check: sid=%s, acc=%s', $service_id, $account);
	$client = loadRecord(sprintf("SELECT id, first_name, last_name FROM clients WHERE contract='%s'", addslashes($account)));
	if($client){
		$body = sprintf("<AccountInfo><Name>%s</Name></AccountInfo>", $this->wrap(trim($client['last_name'].' '.$client['first_name'])));
		$this->response(0, 'OK', $body);
	}else{
		$this->logger->warning('acc not found: '.$account);
		$this->response($this->codes['account_not_found'], '  ');
	}
}

function handlePayment($operation){
	global $db;
	$service_id = $this->getInnerSafeValue($operation, 'serviceid');
	$order_id =  $this->getInnerSafeValue($operation, 'orderid');
	$account =  $this->getInnerSafeValue($operation, 'account');
	$amount =  intval(strval($this->getInnerSafeValue($operation, 'amount') * 100));
	$this->logger->info('pay: sid=%s, oid=%s, acc=%s, amount=%s', $service_id, $order_id, $account, $amount);
	$client = loadRecord(sprintf("SELECT id FROM clients WHERE contract='%s'", addslashes($account)));
	if($client){
		$db->Execute(sprintf("INSERT IGNORE INTO easy_pays (client, service_id, order_id, amount, date) VALUES ('%s', '%s', '%s', '%s', now())",
			$client['id'], addslashes($service_id), addslashes($order_id), $amount));
		$pay = loadRecord(sprintf("SELECT id FROM easy_pays WHERE service_id='%s' AND order_id='%s'", addslashes($service_id), addslashes($order_id)));
		if($pay){
			$this->logger->info('pid: '.$pay['id']);
			$this->response(0, ' ', sprintf("<PaymentId>%s</PaymentId>", $pay['id']));
		}else{
			$this->logger->error('cant pay');
			$this->response($this->codes['cant_pay'], '  ');
		}
	}else{
		$this->logger->warning('acc not found: '.$account);
		$this->response($this->codes['account_not_found'], '  ');
	}
}

function handleConfirm($operation){
	global $db;
	$service_id = $this->getInnerSafeValue($operation, 'serviceid');
	$payment_id = $this->getInnerSafeValue($operation, 'paymentid');
	$this->logger->info('confirm: sid=%s, pid=%s', $service_id, $payment_id);
	$pay = loadRecord(sprintf("SELECT * FROM easy_pays WHERE id='%s'", addslashes($payment_id)));
	if($pay){
		if($pay['movement']){
			$this->logger->warning('already confirmed: '.$pay['id']);
			$mov = loadRecord(sprintf("SELECT * FROM movements WHERE id='%s'", $pay['movement']));
		}else{
			$client = loadRecord(sprintf("SELECT id, account FROM clients WHERE id='%s'", $pay['client']));
			$db->Begin();
			$mid = move($this->account, $client['account'], $pay['amount']);
			$db->Execute(sprintf("UPDATE easy_pays SET movement=%s WHERE id=%s", $mid, $pay['id']));
			if($db->hasErrors()){
				$db->Rollback();
				$mov = '';
			}else{
				$db->Commit();
				$mov = loadRecord(sprintf("SELECT * FROM movements WHERE id='%s'", $mid));
			}
		}
		if($mov){
			$this->logger->info('confirm ok: '.$pay['id']);
			$this->response(0, ' ', sprintf("<OrderDate>%s</OrderDate>", str_replace(" ", "T", $mov['date'])));
		}else{
			$this->logger->error('cant confirm');
			$this->response($this->codes['cant_confirm'], '  ');
		}
	}else{
		$this->logger->warning('pay not found: '.$payment_id);
		$this->response($this->codes['pay_not_found'], '  ');
	}
}

function handleCancel($operation){
	global $db;
	$service_id = $this->getInnerSafeValue($operation, 'serviceid');
	$payment_id = $this->getInnerSafeValue($operation, 'paymentid');
	$canceled = date("Y-m-d H:i:s");
	$this->logger->info('cancel: sid=%s, pid=%s', $service_id, $payment_id);
	$pay = loadRecord(sprintf("SELECT * FROM easy_pays WHERE id='%s'", addslashes($payment_id)));
	if($pay){
		$success = false;
		if($pay['movement']){
			$mov = loadRecord(sprintf("SELECT * FROM movements WHERE id='%s'", $pay['movement']));
			$db->Begin();
			$db->Execute("DELETE FROM movements WHERE id=".$mov['id']);
			moveState($mov['calendar_period'], $mov['dst'], $mov['src'], $mov['amount']);
			$db->Execute("UPDATE easy_pays SET canceled='".$canceled."' WHERE id=".$pay['id']);
			if($db->hasErrors()){
				$db->Rollback();
			}else{
				$db->Commit();
				$success = true;
			}
		}else{
			if($pay['canceled']){
				$canceled = $pay['canceled'];
				$this->logger->warning('already canceled: '.$pay['id'].', '.$canceled);
			}else{
				$db->Execute("UPDATE easy_pays SET canceled='".$canceled."' WHERE id=".$pay['id']);
			}
			$success = true;
		}
		if($success){
			$this->logger->info('canceled: '.$canceled);
			$this->response(0, ' ', sprintf("<CancelDate>%s</CancelDate>", str_replace(" ", "T", $canceled)));
		}else{
			$this->logger->error('cant cancel');
			$this->response($this->codes['cant_cancel'], '  ');
		}
	}else{
		$this->logger->warning('pay not found: '.$payment_id);
		$this->response($this->codes['pay_not_found'], '  ');
	}
}

function handleClearing($operation){
	global $db;
	$order_date = $this->getInnerSafeValue($operation, 'orderdate');
	$this->logger->info('clearing: date=%s', $order_date);
	$pays = loadRecords(sprintf("SELECT p.*, c.contract FROM easy_pays p LEFT JOIN clients c ON c.id=p.client WHERE p.movement is not null and p.date like '%s%%'", addslashes($order_date)));
	$this->logger->info('pays found: '.count($pays));
	$body = "<Payments>";
	foreach($pays as $pay){
		$body .= '<Payment OrderId="'.$pay['order_id'].'" PaymentId="'.$pay['id'].'" ServiceId="'.$pay['service_id'].'" Account="'.$pay['contract'].'" Amount="'.($pay['amount']/100).'" OrderDate="'.str_replace(" ", "T", $pay['date']).'"/>';
	}
	$body .= "</Payments>";
	$this->response(0, 'OK', $body);
}

function wrap($str){
	return iconv('windows-1251', 'utf-8', htmlspecialchars($str));
}

function response($code, $message, $body = ''){
	$date = str_replace(" ", "T", date("Y-m-d H:i:s"));
	$body = sprintf("<Response><StatusCode>%s</StatusCode><StatusDetail>%s</StatusDetail><DateTime>%s</DateTime><Sign></Sign>%s</Response>", $code, $this->wrap($message), $date, $body);
	openssl_sign($body, $sign, $this->priv_key);
	$hexsign = bin2hex($sign);
	$body = str_replace("<Sign></Sign>", "<Sign>".$hexsign."</Sign>", $body);
	$this->logger->debug(urlencode($body));
	header('Content-Type: text/xml; charset=utf-8');
	header('Content-Length: '.strlen($body));
	echo $body;
}

}

$appender = new FileAppender('/var/log/easy-soft.log');
$logger = new DefaultLogger();
$logger->setAppender($appender);
$logger->setName('easy');
//$logger->setLevel(2);

$handler = new EasyHandler($logger);

if(isset($_GET['test'])){
	$clearing = "<Request>
		<Token>Lanet</Token>
		<DateTime>2010-02-26T11:00:00</DateTime>
		<Sign></Sign>
		<Clearing>
		<OrderDate>2010-02-26</OrderDate>
		</Clearing>
		</Request>";
	$cancel = "<Request>
		<Token>Lanet</Token>
		<DateTime>2010-02-24T11:00:00</DateTime>
		<Sign></Sign>
		<Cancel>
		<ServiceId>1</ServiceId>
		<PaymentId>2</PaymentId>
		</Cancel>
		</Request>";
	$confirm = "<Request>
		<Token>Lanet</Token>
		<DateTime>2010-02-24T11:00:00</DateTime>
		<Sign></Sign>
		<Confirm>
		<ServiceId>1</ServiceId>
		<PaymentId>2</PaymentId>
		</Confirm>
		</Request>";
	$payment = "<Request>
		<Token>Lanet</Token>
		<DateTime>2010-02-24T11:00:00</DateTime>
		<Sign></Sign>
		<Payment>
		<ServiceId>1</ServiceId>
		<OrderId>123</OrderId>
		<Account>44443333</Account>
		<Amount>10.00</Amount>
		</Payment>
		</Request>";
	$check = "<Request>
		<Token>Lanet</Token>
		<DateTime>2010-02-24T11:00:00</DateTime>
		<Sign></Sign>
		<Check>
		<ServiceId>1</ServiceId>
		<Account>44443333</Account>
		</Check>
		</Request>";
	$xml = $check;
	//$sha1 = sha1($xml, true);
	//openssl_private_encrypt($sha1, $sign, $handler->priv_key);
	openssl_sign($xml, $sign, $handler->priv_key);
	$hexsign = bin2hex($sign);
	$xml = str_replace("<Sign></Sign>", "<Sign>".$hexsign."</Sign>", $xml);

	$xml = urldecode("%3CRequest%3E%0D%0A++%3CDateTime%3E2010-09-23T11%3A47%3A11%3C%2FDateTime%3E%0D%0A++%3CSign%3E1FDB6B43D84F23CDC5E2A861F6080762FF446B18F2A20834E9A5BB7D66A5040A34413BB4DA8C7B78CD47B334FCE20A923190CE4B6642E3C9E71050794F45F664AB1D0E9A83BE665F948D8ECE2A507A98C0A87CD4A2AB34760567FBC81FCC27700D80A6E4667AE318952C6E8D11710F1230D64054730645C209F5E216DAD8232D%3C%2FSign%3E%0D%0A++%3CCheck%3E%0D%0A++++%3CServiceId%3E401%3C%2FServiceId%3E%0D%0A++++%3CAccount%3E10092252%3C%2FAccount%3E%0D%0A++%3C%2FCheck%3E%0D%0A++%3CToken%3ELanet%3C%2FToken%3E%0D%0A%3C%2FRequest%3E");
	$ch = curl_init();
	curl_setopt($ch, CURLOPT_URL,            "https://stat.lanet.kiev.ua/easy-pay.php" );
	curl_setopt($ch, CURLOPT_POST,           1 );
	curl_setopt($ch, CURLOPT_POSTFIELDS,     $xml);
	curl_setopt($ch, CURLOPT_HTTPHEADER,     array('Content-Type: text/xml; charset=utf-8')); 
	header('Content-Type: text/xml; charset=utf-8');
	$result = curl_exec ($ch);
	//echo $xml;
}else{
	$logger->info('request from: '.$_SERVER['REMOTE_ADDR']);
	$xml = file_get_contents("php://input");
	$decoded = urldecode($xml);
	$encoded = $xml==$decoded ? urlencode($decoded) : $xml;
	$logger->debug($encoded);
	$handler->handle($decoded);
}
?>
