<?
include_once "includes/environment.php";
include_once "includes/core.php";
include_once "functions/accounts.php";
include_once "classes/xml/StringXmlParser.php";
include_once "classes/xml/XmlNodeDelegate.php";
include_once "classes/tools/PhpTemplate.php";
include_once "classes/tools/IpAddress.php";
include_once "classes/log/FileAppender.php";
include_once "classes/log/DefaultLogger.php";

class WrongXmlException extends Exception {}

class WrongSignException extends Exception {}

class PayGateHandler {

var $pub_key;
var $priv_key;
var $token;
var $datetime;
var $account = '';
var $codes = array(
	'wrong_xml' => -1,
	'wrong_token' => -2,
	'wrong_sign' => -3,
	'wrong_operation' => -4,
	'account_not_found' => -5,
	'cant_pay' => -6,
	'access_denied' => -7,
);
var $logger;

function PayGateHandler($logger){
	$this->logger = $logger;
}

function uniName($name){
	$name = strtolower($name);
	$arr = explode(':', $name);
	return count($arr)==2 ? $arr[1] : $arr[0];
}

function isTag($node, $name){
	return $this->uniName($node->name) == $this->uniName($name);
}

function getInnerSafe($node, $name){
	foreach($node->nodes as $inner){
		if($this->isTag($inner, $name)){
			return $inner;
		}
	}
	throw new WrongXmlException($name.' expected');
}

function getInnerSafeValue($node, $name){
	$node = $this->getInnerSafe($node, $name);
	return $node->body;
}

function mapNodes($nodes){
	$map = array();
	foreach($nodes as $n){
		$map[$this->uniName($n->name)] = $n;
	}
	return $map;
}

function mapValues($nodes){
	$map = array();
	foreach($nodes as $n){
		$map[$this->uniName($n->name)] = $n->body;
	}
	return $map;
}

function handle($xml){
	global $beans;
	$this->priv_key = getTrimOption('PAYGATE_OUR_PRIVATE');
	try{
		$parser = new StringXmlParser();
		$parser->setEncoding("UTF-8");
		$parser->setOptions(array(
			XML_OPTION_CASE_FOLDING => 0,
			XML_OPTION_SKIP_WHITE => 1,
		));
		$parser->setLogFactory($beans->getBean('loggerFactory'));
		$delegate = new XmlNodeDelegate();
		if(!$parser->parse($delegate, $xml)){
			throw new WrongXmlException('cant parse xml');
		}
		$root = $delegate->node;
		$request = $this->getInnerSafe($root, 'request');
		$this->token = addslashes($this->getInnerSafeValue($request, 'token'));
		$this->datetime = $this->getInnerSafeValue($request, 'datetime');
		$sign = $this->getInnerSafe($request, 'sign');
		$this->account = getTrimOption($this->token.'_ACCOUNT');
		$this->pub_key = getTrimOption($this->token.'_THEIRS_PUBLIC');
		
		if(!$this->account || !$this->pub_key || !$this->priv_key){
			$this->logger->warning('unconfigured (%s_ACCOUNT, %_THEIRS_PUBLIC, PAYGATE_OUR_PRIVATE)',
			 $this->token, $this->token);
			$this->response($this->codes['wrong_token'], ' Token');
			return;
		}
		
		$valid_zone = false;
		$access_zone = getTrimOption($this->token.'_ALLOW');
		$ip = new SlashedIpAddress($_SERVER["REMOTE_ADDR"]);
		foreach(preg_split("|[\s,;]+|", $access_zone, -1, PREG_SPLIT_NO_EMPTY) as $ip_str){
			$zone = new SlashedIpAddress($ip_str);
			if($zone->contains($ip)){
				$valid_zone = true;
				break;
			}
		}
		if(!$valid_zone){
			$this->response($this->codes['access_denied'], '  ');
			return;
		}

		$this->verify_sign($xml, $sign->body);
		
		if(!isset($request->nodes[3])){
			throw new WrongXmlException('operation not defined');
		}
		$operation = $request->nodes[3];
		$operationName = $this->uniName($operation->name);
		switch($operationName){
			case 'check':
				$this->handleCheck($operation);
				break;
			case 'payment':
				$this->handlePayment($operation);
				break;
			default:
				$this->logger->error('wrong operation: '. $operationName);
				$this->response($this->codes['wrong_operation'], '  ');
		}
	}catch(WrongXmlException $ex){
		$this->logger->error('wrong xml: '. $ex->getMessage());
		$this->logger->error($xml);
		$this->response($this->codes['wrong_xml'], ' XML');
	}catch(WrongSignException $ex){
		$this->logger->error('wrong sign');
		$this->response($this->codes['wrong_sign'], '  ');
	}
}

function verify_sign($xml, $sign){
	$xml = str_replace($sign, '', $xml);
	$bin_sign = pack("H*", $sign);
	$result = openssl_verify($xml, $bin_sign, $this->pub_key);
	switch($result){
		case 0:
			throw new WrongSignException();
			break;
		case -1:
			$this->logger->error('cant verify sign');
			throw new WrongSignException();
			break;
	}
}

function _verify_sign($xml, $sign){
	$xml = str_replace($sign, '', $xml);
	$sha1_calc = sha1($xml);
	$bin_sign = pack("H*", $sign);
	openssl_public_decrypt($bin_sign, $sha1_rcvd, $this->pub_key);
	$sha1_rcvd = bin2hex($sha1_rcvd);
	if(strcasecmp($sha1_calc, $sha1_rcvd)!=0){
		$this->logger->info('calc: '.$sha1_calc);
		$this->logger->info('rcvd: '.$sha1_rcvd);
		throw new WrongSignException();
	}
}

function handleCheck($operation){
	$account = $this->getInnerSafeValue($operation, 'account');
	$this->logger->info('check:  acc=%s', $account);
	$client = loadRecord(sprintf("SELECT id, first_name, last_name FROM clients WHERE contract='%s'", addslashes($account)));
	if($client){
		$body = sprintf("<AccountInfo><Name>%s</Name></AccountInfo>", $this->wrap(trim($client['last_name'].' '.$client['first_name'])));
		$this->response(0, 'OK', $body);
	}else{
		$this->logger->warning('acc not found: '.$account);
		$this->response($this->codes['account_not_found'], '  ');
	}
}

function handlePayment($operation){
	global $db;
	$order_id =  $this->getInnerSafeValue($operation, 'orderid');
	$account =  $this->getInnerSafeValue($operation, 'account');
	$amount =  intval($this->getInnerSafeValue($operation, 'amount'));
	$this->logger->info('pay: oid=%s, acc=%s, amount=%s', $order_id, $account, $amount);
	$client = loadRecord(sprintf("SELECT id, account FROM clients WHERE contract='%s'", addslashes($account)));
	if($client){
		$pay = loadRecord(sprintf("SELECT * FROM gate_pays WHERE order_id='%s'", addslashes($order_id)));
		if(!$pay){
			$db->Begin();
			$mid = move($this->account, $client['account'], $amount);
			$db->Execute(sprintf("INSERT IGNORE INTO gate_pays (token, client, order_id, amount, date, movement) VALUES ('%s', '%s', '%s', '%s', now(), '%s')",
				$this->token, $client['id'], addslashes($order_id), $amount, $mid));
			$pid = $db->getJustInserted('gate_pays', 'id');
			if($db->hasErrors()){
				$db->Rollback();
				$pid = '';
			}else{
				$db->Commit();
			}
		}else{
			$pid = $pay['id'];
		}
		if($pid){
			$this->logger->info('pid: '.$pid);
			$this->response(0, ' ', sprintf("<PaymentId>%s</PaymentId>", $pid));
		}else{
			$this->logger->error('cant pay');
			$this->response($this->codes['cant_pay'], '  ');
		}
	}else{
		$this->logger->warning('acc not found: '.$account);
		$this->response($this->codes['account_not_found'], '  ');
	}
}

function wrap($str){
	return iconv('windows-1251', 'utf-8', htmlspecialchars($str));
}

function response($code, $message, $body = ''){
	$date = str_replace(" ", "T", date("Y-m-d H:i:s"));
	$body = sprintf("<Response><StatusCode>%s</StatusCode><StatusDetail>%s</StatusDetail><DateTime>%s</DateTime><Sign></Sign>%s</Response>", $code, $this->wrap($message), $date, $body);
	openssl_sign($body, $sign, $this->priv_key);
	$hexsign = bin2hex($sign);
	$body = str_replace("<Sign></Sign>", "<Sign>".$hexsign."</Sign>", $body);
	$this->logger->debug(urlencode($body));
	header('Content-Type: text/xml; charset=utf-8');
	header('Content-Length: '.strlen($body));
	echo $body;
}

}

$appender = new FileAppender('/var/log/pay-gate.log');
$logger = new DefaultLogger();
$logger->setAppender($appender);
$logger->setName('paygate');
//$logger->setLevel(2);

$handler = new PayGateHandler($logger);

if(isset($_GET['test'])){
	$payment = "<Request>
		<Token>PLATEZHKA</Token>
		<DateTime>2010-02-24T11:00:00</DateTime>
		<Sign></Sign>
		<Payment>
		<OrderId>124</OrderId>
		<Account>33334444</Account>
		<Amount>1000</Amount>
		</Payment>
		</Request>";
	$check = "<Request>
		<Token>PLATEZHKA</Token>
		<DateTime>2010-02-24T11:00:00</DateTime>
		<Sign></Sign>
		<Check>
		<Account>33334444</Account>
		</Check>
		</Request>";
	$xml = $check;
	//$sha1 = sha1($xml, true);
	//openssl_private_encrypt($sha1, $sign, $handler->priv_key);
	openssl_sign($xml, $sign, getTrimOption('PAYGATE_OUR_PRIVATE'));
	$hexsign = bin2hex($sign);
	$xml = str_replace("<Sign></Sign>", "<Sign>".$hexsign."</Sign>", $xml);

	//$xml = urldecode("%3cRequest%3e%0d%0a++%3cToken%3eLanet%3c%2fToken%3e%0d%0a++%3cDateTime%3e2010-03-01T17%3a12%3a21%3c%2fDateTime%3e%0d%0a++%3cSign%3e670731F0E3D3A63EE03AE1DB34052CC5B1EEBFED77CCBB7D2FBB6C18E46FBB7DA053ECA271679420DA906B00794E16014902CFFFAAFC2EF93C99EEEB45963F3F670517004EE1B132A61EDDF28DFE7CED54D2E26A6ECD1DF6AF07DFE63BF07D79004DD42200C8E5DE5EFA53CC62B1B4132B79F05CDB28732904D89619A91108F1%3c%2fSign%3e%0d%0a++%3cCheck%3e%0d%0a++++%3cServiceId%3e100%3c%2fServiceId%3e%0d%0a++++%3cAccount%3e44443333%3c%2fAccount%3e%0d%0a++%3c%2fCheck%3e%0d%0a%3c%2fRequest%3e");
	$ch = curl_init();
	curl_setopt($ch, CURLOPT_URL,            "http://devel.lanet.kiev.ua/oz/stat/pay-gate.php" );
	curl_setopt($ch, CURLOPT_POST,           1 );
	curl_setopt($ch, CURLOPT_POSTFIELDS,     $xml);
	curl_setopt($ch, CURLOPT_HTTPHEADER,     array('Content-Type: text/xml; charset=utf-8')); 
	header('Content-Type: text/xml; charset=utf-8');
	$result = curl_exec ($ch);

}else{
	$logger->info('request from: '.$_SERVER['REMOTE_ADDR']);
	$xml = file_get_contents("php://input");
	$decoded = urldecode($xml);
	$encoded = $xml==$decoded ? urlencode($decoded) : $xml;
	$logger->debug($encoded);
	$handler->handle($decoded);
}
?>
