function ModemNode(pid, id, name, model, street, house, entrance, floor, state, port, audit, off_all, off_month){
	this.pid = pid;
	this.id = id;
	this.name = name;
	this.model = model;
	this.street = street;
	this.house = house;
	this.entrance = entrance;
	this.floor = floor;
	this.state = state;
	this.port = port;
	this.off_all = off_all;
	this.off_month = off_month;
	this.audit = audit;
	this.isLast = false;
	this.opened = false;
	this.kids = new Array();

	this.addKid = function (node){
		if(this.kids.length > 0){
			this.kids[this.kids.length - 1].isLast = false;
		}
		node.isLast = true;
		this.kids.push(node);
	}
}

function ModemTree(name, icons){
	this.name = name;
	this.icons = icons;
	this.indent = new Array();
	this.result = "";
	this.nodes = new Object();
	this.root = new Array();
	this.opened = false;

	this.add = function(node){
		this.nodes[node.id] = node;
		if(node.pid > 0){
			this.nodes[node.pid].addKid(node);
		}else{
			if(this.root.length > 0){
				this.nodes[this.root[this.root.length - 1]].isLast = false;
			}
			node.isLast = true;
			this.root.push(node.id);
		}
	}

	this.draw = function(){
		this.result = "";
		for (var i=0; i<this.root.length; i++){
			this.render(this.nodes[this.root[i]]);
		}
		return this.result;
	}

	this.render = function(node){
		this.result += '<table id="' + this.name + '" class="' + 'au' + node.audit + ' st' + node.state + '"><tr>';
		for (var i=0; i<this.indent.length; i++){
			this.result += '<td class="indent"><img src="' + ( (this.indent[i] == 1) ? this.icons.line : this.icons.empty ) + '" height=18 width=18 /></td>';
		}
		if (node.kids.length > 0) {
			this.result += '<td class="indent"><a href="javascript: ' + this.name + '.toggle(' + node.id + ');"><img id="toggle' + node.id + '" src="';
			if(node.opened){
				this.result += node.isLast ? this.icons.minusBottom : this.icons.minus;
			}else{
				this.result += node.isLast ? this.icons.plusBottom : this.icons.plus;
			}
			this.result += '" height=18 width=18 /></a></td>';
		} else {
			this.result += '<td class="indent"><img src="' + (node.isLast ? this.icons.joinBottom : this.icons.join) + '" height=18 width=18 /></td>';
		}
		this.result += '<td class="icon"><img id="ni' + node.id + '" src="' + (node.state=='0' ? this.icons.nodeBad : this.icons.node) + '" height=18 width=18 /></td>'
		this.result += '<td class="name">';

		if (node.kids.length > 0) {
			this.result += '<a href="javascript: ' + this.name + '.toggle(' + node.id + ');">' + node.name + '</a>';
		}else{
			this.result += node.name;
		}

		//this.result += '<a href="modems.php?id=' + node.id + '">' + node.name + '</a>';
		this.result += "</td>";

		this.result += '<td class="street">' + node.street + " " + node.house + "</td>";
		this.result += '<td class="entrance">' + node.entrance + "</td>";
		this.result += '<td class="floor">' + node.floor + "</td>";
		this.result += '<td class="model">' + node.model + "</td>";
		this.result += '<td class="off">' + node.off_all + "</td>";
		this.result += '<td class="off">' + node.off_month + "</td>";
		this.result += '<td class="port">' + node.port + "</td>";
		this.result += '<td class="toCatalog"><a href="modems.php?id=' + node.id + '"><b>&raquo;</b></a></td>';

		this.result += '</tr></table>';
		this.indent.push(node.isLast ? 0 : 1);
		this.result += '<div class="kids"><div class="kids_in" id="kids' + node.id +
			'" style="display:' + (node.opened ? 'block' : 'none') + ';">';
		for (var j=0; j<node.kids.length; j++){
			this.render(node.kids[j]);
		}
		this.result += '</div></div>';
		this.indent.pop();
	}

	this.toggle = function(id){
		var node = this.nodes[id];
		this.setNode(node, !node.opened);
	}

	this.setNode = function(node, state){
		if(node.kids.length > 0){
			node.opened = state;
			var img = document.getElementById('toggle' + node.id);
			var div = document.getElementById('kids' + node.id);
			div.style.display = node.opened ? 'block' : 'none';
			if(node.opened){
				img.src = node.isLast ? this.icons.minusBottom : this.icons.minus;
			}else{
				img.src = node.isLast ? this.icons.plusBottom : this.icons.plus;
			}
		}
	}

	this.setRecursive = function(node, state) {
		this.setNode(node, state)
		for (var j=0; j<node.kids.length; j++){
			this.setRecursive(node.kids[j], state);
		}
	}

	this.setAll = function(state){
		for (var i=0; i<this.root.length; i++){
			this.setRecursive(this.nodes[this.root[i]], state);
		}
	}

	this.toggleAll = function(){
		this.opened = !this.opened;
		this.setAll(this.opened);
	}


	this.registerHighLight = function (className){
		for (var i=0; i<this.root.length; i++){
			this.registerHighLightNode(this.nodes[this.root[i]], className);
		}
	}

	this.registerHighLightNode = function (node, className) {
		var img = document.getElementById("ni" + node.id);
		var table = img.parentNode.parentNode.parentNode.parentNode;
		table.setAttribute("highlight", table.className + className);
		table.setAttribute("oldClassName", table.className);
		table.onmouseover = function() {
			this.className = this.getAttribute("highlight");

		};
		table.onmouseout = function() {
			this.className = this.getAttribute("oldClassName");

		};
		for (var j=0; j<node.kids.length; j++){
			this.registerHighLightNode(node.kids[j], className);
		}
	}

	this.openTo = function(id) {
		var node = this.nodes[id];
		if(node){
			this.setNode(node, 1)
			if(node.pid > 0){
				this.openTo(node.pid);
			}
		}
	}

	this.select = function (id) {
		this.openTo(id);
		var img = document.getElementById("ni" + id);
		var table = img.parentNode.parentNode;
		table.className = table.className + " selected";
	}

}

