#!/usr/bin/php -q
<?php
/*
author: Ponomarev Denis <ponomarev@gmail.com>
$Date: 2004/04/27 08:37:36 $
$Revision: 1.19 $
*/

$servers = array(
	'localhost'=>'',
	'nas01'=>"root@nas01",
	'nas02'=>"root@nas02"
);

$ipsets = array(
	'10.2.0.0/16'=>array(
		'ipset'=>'host_10_2',
		'servers'=>array('localhost','nas01','nas02'),
	),
	'10.4.0.0/16'=>array(
		'ipset'=>'host_10_4',
		'servers'=>array('localhost','nas01','nas02'),
	),
	'10.10.0.0/16'=>array(
		'ipset'=>'host_10_10',
		'servers'=>array('localhost','nas01','nas02'),
	),
	'10.50.0.0/16'=>array(
		'ipset'=>'host_10_50',
		'servers'=>array('localhost','nas01','nas02'),
	),
	'10.252.0.0/16'=>array(
		'ipset'=>'host_10_252',
		'servers'=>array('localhost','nas01','nas02'),
	),
	'192.168.2.0/24'=>array(
		'ipset'=>'host_192_168',
		'servers'=>array('localhost','nas01','nas02'),
	),
	'195.189.196.0/23'=>array(
		'ipset'=>'host_195_189',
		'servers'=>array('localhost','nas01','nas02'),
	),
);


!isset($_SERVER['SERVER_NAME']) || exit;
error_reporting(E_ALL);
ini_set('include_path', DIRNAME(__FILE__). '/../');

include("includes/options.php");
include("classes/context/include.php");

$beanFactory = new CompositBeanFactory(array(
	new ArrayBeanFactory('boot/caban/log.php'),
	new ArrayBeanFactory('boot/caban/tools.php'),
	new ArrayBeanFactory('boot/caban/dao.php'),
));

$loggerFactory = $beanFactory->getBean('loggerFactory');
$logger = $loggerFactory->getLogger('make.access');
if(!isset($_SERVER['argv'][3])){
	$locker = $beanFactory->getBean('locker');
	if(!$locker->capture('make_access', 0)){
		$logger->warning("locked");
		exit(0);
	}
	$logger->info("BEGIN");
}

$processor = $beanFactory->getBean('processorLogged');
//$processor->out_call = 'info';



$no_ua = array('localhost','nas01','nas02');

$d = dirname(__FILE__);

$make_time = time();

if(isset($_SERVER['argv'][3])){
	list($me, $filename, $server, $ipset) = $_SERVER['argv'];
	$access = $servers[$server];
	if($access){
		$copy_cmd = "scp -o StrictHostKeyChecking=no -o BatchMode=yes -o ConnectTimeout=3 -q ".TMP_DIR."/caban/".$filename." ".$access.":".TMP_DIR."/caban/".$filename;
		if($processor->process($copy_cmd, $logger)){
			$logger->error("failed scp to %s command: %s", $server, $copy_cmd);
			exit(3);
		}
		$build_cmd = 'ssh -o StrictHostKeyChecking=no -o BatchMode=yes -o ConnectTimeout=3 '.$access.' "cat '.TMP_DIR.'/caban/'.$filename.' | /var/www/html/billing-slave/tools/ipset-builder.php '.$ipset.'"';
		if($processor->process($build_cmd, $logger)){
			$logger->error("failed ipset-builder command at %s: %s", $server, $build_cmd);
			exit(4);
		}
	}else{
		$build_cmd = "cat ".TMP_DIR."/caban/".$filename." | $d/ipset-builder.php ".$ipset;
		if($processor->process($build_cmd, $logger)){
			$logger->error("failed ipset-builder command: %s", $build_cmd);
			exit(4);
		}
	}
	exit(0);
}

foreach($servers as $server=>$access){
	if(!in_array($server, $no_ua)){
		if($access){
			$uaset_cmd = 'ssh -o StrictHostKeyChecking=no -o BatchMode=yes -o ConnectTimeout=3 '.$access.' "/var/www/html/billing-slave/tools/uaset-make"';
		}else{
			$uaset_cmd = $d.'/uaset-make';
		}
		//echo $uaset_cmd."\n";
		if($processor->process($uaset_cmd, $logger)){
			$logger->error("failed uaset command at %s: %s", $server, $uaset_cmd);
		}
	}
}

$list_cmd = "$d/hosts-ip-list.php -c connected > ".TMP_DIR."/caban/allow";
if($processor->process($list_cmd, $logger)){
	$logger->error("failed host list command: %s", $list_cmd);
	exit(1);
}

$make_date = date("Y-m-d/H-i-s", $make_time);
$history_dir = TMP_DIR."/caban/".$make_date;
//mkdir($history_dir, 0777, true);

foreach($ipsets as $net=>$data){
	$filename = 'allow-'.str_replace("/", "-", $net);
	$split_cmd = "cat ".TMP_DIR."/caban/allow | $d/masked-ip-list.php ".$net." | grep -v / > ".TMP_DIR."/caban/".$filename;
	if($processor->process($split_cmd, $logger)){
//		$logger->error("failed list split command: %s", $split_cmd);
//		break;
	}
//	copy(TMP_DIR."/caban/".$filename, $history_dir.'/'.$filename);
	foreach($data['servers'] as $server){
		$target_cmd = $_SERVER['argv'][0].' '.$filename.' '.$server.' '.$data['ipset'].' ';//&
		//if(proc_close(proc_open($target_cmd, array(), $foo))){
		if($processor->process($target_cmd, $logger)){
			$logger->error("failed server %s command: %s", $server, $target_cmd);
		}
	}
}

foreach($servers as $server=>$access){
	//; mkdir -p /etc/sysconfig/ipset-dump-log/'.date("Y-m-d", $make_time).'; cp /etc/sysconfig/ipset-dump /etc/sysconfig/ipset-dump-log/'.date("Y-m-d/H-i-s", $make_time);
	$dump_cmd = 'ipset -S > /etc/sysconfig/ipset-dump';
	if($access){
		$dump_cmd = 'ssh -o StrictHostKeyChecking=no -o BatchMode=yes -o ConnectTimeout=3 '.$access.' "'.$dump_cmd.'"';
	}
	if($processor->process($dump_cmd, $logger)){
		$logger->error("failed ipset dump command at %s: %s", $server, $dump_cmd);
	}
}

if(!isset($_SERVER['argv'][3])){
	$logger->info("END");
}

?>
