#!/usr/bin/php -q
<?php
/*
author: Ponomarev Denis <ponomarev@gmail.com>
$Date: 2004/04/27 08:37:36 $
$Revision: 1.19 $
*/
!isset($_SERVER['SERVER_NAME']) || exit;
error_reporting(E_ALL);

ini_set('include_path', DIRNAME(__FILE__). '/../');
include("includes/options.php");
include("classes/context/include.php");

$beanFactory = new CompositBeanFactory(array(
	new ArrayBeanFactory('boot/caban/log.php'),
	new ArrayBeanFactory('boot/caban/dao.php'),
	new ArrayBeanFactory('boot/caban/tools.php'),
));
$loggerFactory = $beanFactory->getBean('loggerFactory');
$logger = $loggerFactory->getLogger('switch.fix.cable');

$logger->info("BEGIN");

$edgecore_models = array(
	'.1.3.6.1.4.1.259.6.10.94'=>'ES3552M',
);

$options = array(
	'snmp-timeout' => 2000000,
	'snmp-retries' => 2,
	'snmp-reader' => 'public',
	'snmp-writer' => 'security',
);

snmp_set_oid_numeric_print(1);
snmp_set_quick_print(1);
snmp_set_valueretrieval(SNMP_VALUE_PLAIN);

function _snmpset($oid, $type, $value){
	global $options, $modem;
	return snmpset($modem['ip'], $options['snmp-writer'], $oid, $type, $value, $options['snmp-timeout'], $options['snmp-retries']);
}

function countLevel($id){
	global $modems;
	$modem = $modems[$id];
	if($parent = $modem['modem']){
		return countLevel($parent) + 1;
	}else{
		return 0;
	}
}

function compareByLevelDesc($m1, $m2){
	return $m2['level'] - $m1['level'];
}

$dao = $beanFactory->getBean('modemsDAO');
$db = $beanFactory->getBean('connection');
$hostsDao = $beanFactory->getBean('hostsDAO');

$it = $dao->iterator();

$modems = array();
while($modem = $it->next()){
	$modems[$modem['id']] = $modem;
}
foreach($modems as $id=>$modem){
	$modems[$id]['level'] = countLevel($id);
}
usort($modems, 'compareByLevelDesc');

foreach($modems as $modem){
	if($modem['model']=='ES3528M' || $modem['model']=='ES3552M'){
		$model_oid = @snmpget($modem['ip'], $options['snmp-reader'], ".1.3.6.1.2.1.1.2.0", $options['snmp-timeout'], $options['snmp-retries']);
		if($model_oid){
			if(isset($edgecore_models[$model_oid])){
				$sel = $hostsDao->listByModem($modem['id']);
				while($host = $sel->next()){
					if($host['port'] > 0){
						$status = @snmpget($modem['ip'], $options['snmp-reader'], ".1.3.6.1.2.1.2.2.1.8.".$host['port'], $options['snmp-timeout'], $options['snmp-retries']);					// get port $port operstatus
						$tested = @snmpget($modem['ip'], $options['snmp-reader'], "SNMPv2-SMI::enterprises.259.6.10.94.1.2.3.2.1.2.".$host['port'], $options['snmp-timeout'], $options['snmp-retries']);	// get port $port teststatus
						if($status != 1 && ($tested == 1 || $tested == 2 || $tested == 9)){
							$success = true;
							$success &= _snmpset('SNMPv2-SMI::enterprises.259.6.10.94.1.2.3.1.0', 'i', $host['port']);											// execute cable test on port $port
							if($success){
								$a_status   = @snmpget($modem['ip'], $options['snmp-reader'], "SNMPv2-SMI::enterprises.259.6.10.94.1.2.3.2.1.2.".$host['port'], $options['snmp-timeout'], $options['snmp-retries']);
								$b_status   = @snmpget($modem['ip'], $options['snmp-reader'], "SNMPv2-SMI::enterprises.259.6.10.94.1.2.3.2.1.3.".$host['port'], $options['snmp-timeout'], $options['snmp-retries']);
								$a_distance = @snmpget($modem['ip'], $options['snmp-reader'], "SNMPv2-SMI::enterprises.259.6.10.94.1.2.3.2.1.6.".$host['port'], $options['snmp-timeout'], $options['snmp-retries']);
								$b_distance = @snmpget($modem['ip'], $options['snmp-reader'], "SNMPv2-SMI::enterprises.259.6.10.94.1.2.3.2.1.7.".$host['port'], $options['snmp-timeout'], $options['snmp-retries']);
								if( $a_status === false ||
									$b_status === false ||
									$a_distance === false ||
									$b_distance === false
								){
									$logger->error("edge %s: port %d cable read failed", $modem['name'], $host['port']);
								}else{
									$a_distance = $a_distance > 0 ? $a_distance : $host['pair_a_distance'];
									$b_distance = $b_distance > 0 ? $b_distance : $host['pair_b_distance'];
									$logger->info("edge %s: port %d cable read ok", $modem['name'], $host['port']);
									$hostsDao->updatePairsInfo($host['id'], $a_status, $b_status, $a_distance, $b_distance);
								}
								$logger->info("edge %s: port %d cable test ok", $modem['name'], $host['port']);
							}else{
								$logger->error("edge %s: port %d cable test failed", $modem['name'], $host['port']);
							}
						}
					}
				}
			}
		}else{
			$logger->error("edge %s: no answer", $modem['name']);
		}
	}
}
$logger->info("END");
?>
