#!/usr/bin/php -q
<?php
/*
author: Ponomarev Denis <ponomarev@gmail.com>
$Date: 2004/04/27 08:37:36 $
$Revision: 1.19 $
*/
!isset($_SERVER['SERVER_NAME']) || exit;
error_reporting(E_ALL);

ini_set('include_path', DIRNAME(__FILE__). '/../');
include("includes/options.php");
include("classes/context/include.php");

$beanFactory = new CompositBeanFactory(array(
	new ArrayBeanFactory('boot/caban/log.php'),
	new ArrayBeanFactory('boot/caban/dao.php'),
	new ArrayBeanFactory('boot/caban/tools.php'),
));
$loggerFactory = $beanFactory->getBean('loggerFactory');
$logger = $loggerFactory->getLogger('switch.fix.mvr');

$logger->info("BEGIN");

$edgecore_models = array(
	'.1.3.6.1.4.1.259.6.10.57'=>'ES4612',
	'.1.3.6.1.4.1.259.6.10.94'=>'ES3552M',
	'.1.3.6.1.4.1.259.8.1.5'=>'ES3510',
	'.1.3.6.1.4.1.259.8.1.6'=>'ES3510',
);

$options = array(
	'snmp-timeout' => 2000000,
	'snmp-retries' => 2,
	'snmp-reader' => 'public',
	'snmp-writer' => 'security',
);

snmp_set_oid_numeric_print(1);
snmp_set_quick_print(1);
snmp_set_valueretrieval(SNMP_VALUE_PLAIN);

function _snmpset($oid, $type, $value){
	global $options, $modem;
	return snmpset($modem['ip'], $options['snmp-writer'], $oid, $type, $value, $options['snmp-timeout'], $options['snmp-retries']);
}

function countLevel($id){
	global $modems;
	$modem = $modems[$id];
	if($parent = $modem['modem']){
		return countLevel($parent) + 1;
	}else{
		return 0;
	}
}

function compareByLevelDesc($m1, $m2){
	return $m2['level'] - $m1['level'];
}

$dao = $beanFactory->getBean('modemsDAO');
$db = $beanFactory->getBean('connection');

$it = $dao->iterator();

$modems = array();
while($modem = $it->next()){
	$modems[$modem['id']] = $modem;
}
foreach($modems as $id=>$modem){
	$modems[$id]['level'] = countLevel($id);
}
usort($modems, 'compareByLevelDesc');

foreach($modems as $modem){
	if($modem['model']=='ES3528M' || $modem['model']=='ES3552M'){
		$model_oid = @snmpget($modem['ip'], $options['snmp-reader'], ".1.3.6.1.2.1.1.2.0", $options['snmp-timeout'], $options['snmp-retries']);
		if($model_oid){
			if(isset($edgecore_models[$model_oid])){
//					'set'=>'SNMPv2-SMI::enterprises.259.6.10.94.1.9.15.0',		// IGMP leave proxy Integer (1) enabled (2) disabled
//					'set'=>'SNMPv2-SMI::enterprises.259.6.10.94.1.44.1.0',		// MVR Integer (1) enabled (2) disabled
//					'set'=>'SNMPv2-SMI::enterprises.259.6.10.94.1.44.2.0',		// MVR VLAN Integer (number)
//					'set'=>'SNMPv2-SMI::enterprises.259.6.10.94.1.44.5.1.0',	// start of MVR group (address)
//					'set'=>'SNMPv2-SMI::enterprises.259.6.10.94.1.44.5.2.0',	// range count for MVR group addresses Integer (number)
//					'set'=>'SNMPv2-SMI::enterprises.259.6.10.94.1.44.5.3.0',	// action for MVR group Integer (1) create (2) destroy
//					'set'=>'SNMPv2-SMI::enterprises.259.6.10.94.1.44.9.1.2.x',	// MVR port (x) type Integer (1) source (2) reciever
//					'set'=>'SNMPv2-SMI::enterprises.259.6.10.94.1.44.9.1.3.x',	// Immediate leave on MVR port (x) type Integer (1) enabled (2) disabled
				$oid = array(
					'set'=>'SNMPv2-SMI::enterprises.259.6.10.94.1.44.1.0',
					'config_src'=>$model_oid.".1.24.1.1.0",
					'config_dst'=>$model_oid.".1.24.1.3.0",
					'config_dst_name'=>$model_oid.".1.24.1.4.0",
					'config_action'=>$model_oid.".1.24.1.8.0",
					'iptv'=>'.1.3.6.1.4.1.259.6.10.94.1.44.9.1.2',
				);
				$cfg_name = $modem['name'].'-confg';
				$success = true;
/*
				$success &= _snmpset('SNMPv2-SMI::enterprises.259.6.10.94.1.9.15.0', 'i', 1);
				$success &= _snmpset('SNMPv2-SMI::enterprises.259.6.10.94.1.44.1.0', 'i', 1);
				$success &= _snmpset('SNMPv2-SMI::enterprises.259.6.10.94.1.44.2.0', 'i', 33);
				$success &= _snmpset('SNMPv2-SMI::enterprises.259.6.10.94.1.44.5.1.0', 'a', '239.0.0.1');
				$success &= _snmpset('SNMPv2-SMI::enterprises.259.6.10.94.1.44.5.2.0', 'i', 250);
				$success &= _snmpset('SNMPv2-SMI::enterprises.259.6.10.94.1.44.5.3.0', 'i', 2);
				$success &= _snmpset('SNMPv2-SMI::enterprises.259.6.10.94.1.44.5.1.0', 'a', '239.0.1.1');
				$success &= _snmpset('SNMPv2-SMI::enterprises.259.6.10.94.1.44.5.2.0', 'i', 253);
				$success &= _snmpset('SNMPv2-SMI::enterprises.259.6.10.94.1.44.5.3.0', 'i', 1);

				$success &= _snmpset('SNMPv2-SMI::enterprises.259.6.10.94.1.9.17.0', 'i', 1); // Filter

				$success &= _snmpset('SNMPv2-SMI::enterprises.259.6.10.94.1.9.18.1.3.1', 'i', 1);
				$success &= _snmpset('SNMPv2-SMI::enterprises.259.6.10.94.1.9.18.1.3.2', 'i', 1);
				$success &= _snmpset('SNMPv2-SMI::enterprises.259.6.10.94.1.9.18.1.3.3', 'i', 1);
				$success &= _snmpset('SNMPv2-SMI::enterprises.259.6.10.94.1.9.18.1.2.1', 'i', 1);
				$success &= _snmpset('SNMPv2-SMI::enterprises.259.6.10.94.1.9.18.1.2.2', 'i', 1);
				$success &= _snmpset('SNMPv2-SMI::enterprises.259.6.10.94.1.9.18.1.2.3', 'i', 1);

				$success &= _snmpset('SNMPv2-SMI::enterprises.259.6.10.94.1.9.19.1.0', 'u', 1);
				$success &= _snmpset('SNMPv2-SMI::enterprises.259.6.10.94.1.9.19.2.0', 'i', 1);
				$success &= _snmpset('SNMPv2-SMI::enterprises.259.6.10.94.1.9.19.3.0', 'x', 'EF000101');
				$success &= _snmpset('SNMPv2-SMI::enterprises.259.6.10.94.1.9.19.4.0', 'x', 'EF000116');
				$success &= _snmpset('SNMPv2-SMI::enterprises.259.6.10.94.1.9.19.5.0', 'i', 2);
				$success &= _snmpset('SNMPv2-SMI::enterprises.259.6.10.94.1.9.19.1.0', 'u', 2);
				$success &= _snmpset('SNMPv2-SMI::enterprises.259.6.10.94.1.9.19.4.0', 'x', 'EF00012E');
				$success &= _snmpset('SNMPv2-SMI::enterprises.259.6.10.94.1.9.19.5.0', 'i', 2);
				$success &= _snmpset('SNMPv2-SMI::enterprises.259.6.10.94.1.9.19.1.0', 'u', 3);
				$success &= _snmpset('SNMPv2-SMI::enterprises.259.6.10.94.1.9.19.4.0', 'x', 'EF0001FF');
				$success &= _snmpset('SNMPv2-SMI::enterprises.259.6.10.94.1.9.19.5.0', 'i', 2);
*/
				if($modem['uplink']){
					$success &= _snmpset($oid['iptv'].'.'.$modem['uplink'], 'i', 1);					// Uplink Source port
				}
				if($success){

					$sel = $db->select("SELECT port FROM hosts WHERE modem='%s' and port>0", $modem['id']);
					while($success && ($arr = $sel->next())){
						$success &= _snmpset($oid['iptv'].'.'.$arr['port'], 'i', 2); 					// Client Reciever port
						$success &= _snmpset('SNMPv2-SMI::enterprises.259.6.10.94.1.44.9.1.3.'.$arr['port'], 'i', 1);	// Client Immediate leave
					}

					$sel = $db->select("SELECT port FROM modems WHERE modem='%s' and port>0", $modem['id']);
					while($success && ($arr = $sel->next())){
						$success &= _snmpset($oid['iptv'].'.'.$arr['port'], 'i', 1);					// Downlink Source port
					}
				}
				if(
					$success
					&& _snmpset($oid['config_src'], 'i', 2)
					&& _snmpset($oid['config_dst'], 'i', 3)
					&& _snmpset($oid['config_dst_name'], 's', $cfg_name)
					&& _snmpset($oid['config_action'], 'i', 2)
				){
					$logger->info("edge %s: success", $modem['name']);
				}else{
					$logger->error("edge %s: failed", $modem['name']);
				}
			}
		}else{
			$logger->error("edge %s: no answer", $modem['name']);
		}
	}
}
$logger->info("END");
?>
