#!/usr/bin/php -q
<?
set_time_limit (0);
ini_set('include_path', DIRNAME(__FILE__). '/../');

include_once "classes/log/StdoutAppender.php";
include_once "classes/log/FileAppender.php";
include_once "classes/log/StreamAppender.php";
include_once "classes/log/DefaultLogger.php";
include("includes/environment.php");
include("functions/accounts.php");

class OSMPPayer {

var $account;
var $logger;

function OSMPPayer($logger){
	global $db;
	$this->logger = $logger;
	if($arr = $db->Next($db->Execute("SELECT account FROM system_accounts WHERE code='PAY_OSMP'"))){
		$this->account = $arr['account'];
	}
}

function getClientByContract($contract){
	return loadRecord(sprintf("SELECT c.id, c.login, c.contract, c.account, c.connection_type, c.first_name, c.last_name, 
		CONCAT(s.name, ' ', h.house, ', ',c.apartment) as address
		FROM clients c
		LEFT JOIN houses h ON h.id=c.house_id
		LEFT JOIN streets s ON s.id=h.street
		WHERE c.contract='%s'", addslashes($contract)));
}

function getUseLack($id){
	if($plans = loadRecord(sprintf("
		SELECT sum(
			if((pp.state=1) && (ppp.use_actual < ppp.use_plan), ppp.use_plan - ppp.use_actual, 0) +
			if((pp.state=2) && (ctr.penalty_type=0) && (ppp.use_actual < ppp.use_plan), ppp.use_plan - ppp.use_actual, 0) +
			if((pp.state=2) && (ctr.penalty_type=2), if(ppp.use_plan > ppp.use_actual, if((ppp.use_plan - ppp.use_actual) > ctr.use_plan, ctr.use_plan, ppp.use_plan - ppp.use_actual), 0), 0)
		) as use_lack
		  FROM personal_period_plans ppp
		  LEFT JOIN personal_periods pp ON pp.id=ppp.personal_period
		  left join connection_type_resources ctr on ctr.connection_type=pp.connection_type and ctr.resource_type=ppp.resource_type
		  WHERE pp.marker=1 AND pp.client=%s", $id))){
		return $plans['use_lack'];
	}else{
		return 0;
	}
}

function process($input, $output){
	global $db;
	while(!feof($input)){
		if($line = trim(fgets($input, 1024))){
			$request = explode(";", $line);
			if(!$this->account){
				$this->logger->error("payer is not configured: system acccount PAY_OSMP not found");
				fputs($output, $request[0].";9;\n");
				return;
			}
			switch($request[0]){
				case '01':
					$contract = $request[1];
					if($contract){
						if($client = $this->getClientByContract($contract)){
							$state = getAccountState(getCurrentCalendarPeriod(), $client['account']);
							$deposit = $state['state_end'] - $this->getUseLack($client['id']);
							$show_deposit = number_format($deposit/100, 2, '.','');
							$this->logger->debug("check contract %s: found", $contract);
							fputs($output, "02;".($deposit < 0  ? "0" : "2").";".$contract.";".$show_deposit.";".$client['address'].";\n");
						}else{
							$this->logger->debug("check contract %s: not found", $contract);
							fputs($output, "02;5;".$contract.";\n");
						}
					}else{
						$this->logger->warning('contract not specified in check request');
						fputs($output, "02;9;\n");
					}
					break;
				case '03':
					$contract = $request[1];
					$amount = $request[2];
					$iamount = intval(strval($amount * 100));
					$order_id = $request[3];
					$date = $request[4];
					$this->logger->info("pay request: contract=%s, amount=%s, oder_id=%s", $contract, $amount, $order_id);
					if($client = $this->getClientByContract($contract)){
						if($pay = loadRecord(sprintf("SELECT * FROM osmp_pays WHERE order_id='%s'", addslashes($order_id)))){
							if($iamount==$pay['amount'] && $client['id']==$pay['client']){
								$this->logger->warning("pay %s already exists, ok", $order_id);
								fputs($output, "04;1;".$contract.";".$amount.";".$order_id.";\n");
							}else{
								$this->logger->warning("pay %s already exists, diff: contract=%s, amount=%s", $order_id, $client['contract'], $pay['amount']/100);
								fputs($output, "04;5;".$contract.";".$amount.";".$order_id.";\n");
							}
						}else{
							$success = false;
							for($i=0; $i<3; $i++){
								$db->Begin();
								$mid = move($this->account, $client['account'], $iamount);
								$db->Execute(sprintf("INSERT INTO osmp_pays (client, movement, order_id, amount, date)
										VALUES ('%s', '%s', '%s', '%s', '%s')",
										$client['id'], $mid,
										addslashes($order_id),
										addslashes($iamount),
										addslashes($date)));
								if($db->hasErrors()){
									$this->logger->warning(implode("\n", $db->errors));
									$db->Rollback();
									sleep(3);
								}else{
									$db->Commit();
									$success = true;
									break;
								}
							}
							if($success){
								$this->logger->info('pay %s is ok', $order_id);
								fputs($output, "04;0;".$contract.";".$amount.";".$order_id.";\n");
							}else{
								$this->logger->error('pay %s cant be moved', $order_id);
								fputs($output, "04;9;\n");
							}
						}
					}else{
						$this->logger->error("cant pay %s: contract %s not found", $order_id, $contract);
						fputs($output, "04;9;\n");
					}
					break;
				default:
					$this->logger->error("unexpected request type: %s", $request[0]);
					fputs($output, $request[0].";9;\n");
					break;
			}
		}
	}
}

}

$opt = getopt('v:l:i:o:h');

if(isset($opt['h'])){
?>

-i <stream>      file descriptor where command will access data, default 0 - stdin
-o <stream>      file descriptor where command will return data, default 1 - stdout
-l <stream>      file descriptor for log, default 2 - stderr
-v <level>       logging level 1-5 (DEBUG - NONE)
-n               name in log, default 'osmp.payer'
-h               this help

<?
exit;
}

$in_from = isset($opt['i']) ? $opt['i'] : 0;
$out_to = isset($opt['o']) ? $opt['o'] : 1;
$log_to = isset($opt['l']) ? $opt['l'] : 2;
$log_level = isset($opt['v']) ? intval($opt['v']) : 2;
$log_name = isset($opt['n']) ? $opt['n'] : 'osmp.payer';

$logger = new DefaultLogger();
$logger->setName($log_name);
$logger->setLevel($log_level);
if($log_stream = fopen('/proc/self/fd/'.$log_to, "a+")){
	$logger->setAppender(new StreamAppender($log_stream));
}else{
	$logger->setAppender(new StdoutAppender());
	$logger->error('cant open %s for logging', $log_to);
}

$payer = new OSMPPayer($logger);

if($input = fopen('/proc/self/fd/'.$in_from, "r")){
	$logger->debug('input stream %s opened', $in_from);
	if($output = fopen('/proc/self/fd/'.$out_to, "w")){
		$logger->debug('output stream %s opened', $out_to);
		$payer->process($input, $output);
		fclose($output);
	}else{
		$logger->error('cant open output stream: %s', $out_to);
		exit(1);
	}
	fclose($input);
}else{
	$logger->error('cant open input stream: %s', $in_from);
	exit(1);
}
?>