<?
include_once "includes/environment.php";
include_once "includes/core.php";
include_once "functions/accounts.php";
include_once "classes/xml/StringXmlParser.php";
include_once "classes/xml/XmlNodeDelegate.php";
include_once "classes/tools/PhpTemplate.php";

class WrongXmlException extends Exception {}

class WrongSerialException extends Exception {}

class WrongKeywordException extends Exception {}

class USMPHandler {

function uniName($name){
	$name = strtolower($name);
	$arr = explode(':', $name);
	return count($arr)==2 ? $arr[1] : $arr[0];
}

function isTag($node, $name){
	return $this->uniName($node->name) == $this->uniName($name);
}

function getInnerSafe($node, $name){
	if(empty($node->nodes)){
		throw new WrongXmlException($name.' expected');
	}
	$inner = $node->nodes[0];
	if(!$this->isTag($inner, $name)){
		throw new WrongXmlException($name.' expected');
	}
	return $inner;
}

function mapNodes($nodes){
	$map = array();
	foreach($nodes as $n){
		$map[$this->uniName($n->name)] = $n;
	}
	return $map;
}

function mapValues($nodes){
	$map = array();
	foreach($nodes as $n){
		$map[$this->uniName($n->name)] = $n->body;
	}
	return $map;
}

function listValues($nodes){
	$list = array();
	foreach($nodes as $n){
		$list[$this->uniName($n->name)] = $n->body;
	}
	return $map;
}

function handle($xml){
	global $beans;
	$tagName = 'GetStatus';
	try{
		$parser = new StringXmlParser();
		$parser->setEncoding("UTF-8");
		$parser->setOptions(array(
			XML_OPTION_CASE_FOLDING => 0,
			XML_OPTION_SKIP_WHITE => 1,
		));
		$parser->setLogFactory($beans->getBean('loggerFactory'));
		$delegate = new XmlNodeDelegate();
		if(!$parser->parse($delegate, $xml)){
			throw new WrongXmlException('cant parse xml');
		}
		$root = $delegate->node;
		$env = $this->getInnerSafe($root, 'envelope');
		$body = $this->getInnerSafe($env, 'body');
		if(empty($body->nodes)){
			throw new WrongXmlException('method not found');
		}
		$method = $body->nodes[0];
		$request = $this->getInnerSafe($method, 'request');
		$map = $this->mapNodes($request->nodes);
		$methodName = $this->uniName($method->name);
		$response = '';
		switch($methodName){
			case 'processpayment':
				$tagName = 'ProcessPayment';
				$response = $this->handleProcessPayment($map);
				break;
			case 'getstatus':
				$tagName = 'GetStatus';
				$response = $this->handleGetStatus($map);
				break;
			case 'validatephone':
				$tagName = 'ValidatePhone';
				$response = $this->handleValidatePhone($map);
				break;
			default:
				throw new WrongXmlException($method->name.' is unsupported');
		}
	}catch(WrongXmlException $ex){
		$response = $this->error(212, 'Incorrect XML', $tagName);
	}catch(WrongSerialException $ex){
		$response = $this->error(211, 'Incorrect Serial', $tagName);
	}catch(WrongKeywordException $ex){
		$response = $this->error(210, 'Incorrect KeyWord', $tagName);
	}
	header('Content-Type: text/xml; charset=utf-8');
	header('Content-Length: '.strlen($response));
	echo $response;
}

function checkSerialKeyword($serial, $keyword){
	$opt_serial = getTrimOption('USMP_SERIAL');
	if(!$opt_serial || ($opt_serial!=$serial)){
		throw new WrongSerialException();
	}
	$opt_keyword = getTrimOption('USMP_KEYWORD');
	if(!$opt_keyword || ($opt_keyword!=$keyword)){
		throw new WrongKeywordException();
	}
}

function handleProcessPayment($map){
	global $db;
	$this->checkSerialKeyword($map['serial']->body, $map['keyword']->body);
	if($arr = loadRecord("SELECT account FROM system_accounts WHERE code='PAY_USMP'")){
		$usmp_account = $arr['account'];
	}else{
		$usmp_account = '';
	}
	$opt_pid = getTrimOption('USMP_PAY_ELEMENT_ID');
	$statuses = array();
	foreach($map['payments']->nodes as $pay){
		$status = 0;
		$details = $this->mapValues($pay->nodes);
		$client = loadRecord(sprintf("SELECT id, account FROM clients WHERE contract='%s'", addslashes($details['account'])));
		$amount = intval($details['amount']);
		$cheque = intval($details['chequenumber']);
		$pid = intval($details['payelementid']);
		$time = strtotime($details['date']);
		if(!$usmp_account){
			$status = 4;
		}elseif("$amount"!=$details['amount']
			|| $amount <= 0
			|| "$pid"!=$details['payelementid']
			|| $pid <=0
			|| "$cheque"!=$details['chequenumber']
			|| $cheque<=0
			|| $time < 0
			|| $time===false){
				$status = 120;
		}elseif($pid!=$opt_pid){
			$status = 121;
		}elseif(!$client){
			$status = 113;
		}else{
			$prev = loadRecord(sprintf("SELECT id FROM usmp_pays WHERE cheque='%s'", $cheque));
			if($prev){
				$status = 108;
			}else{
				$db->Begin();
				$mid = move($usmp_account, $client['account'], $amount);
				$db->Execute(sprintf("INSERT INTO usmp_pays (client, movement, cheque, amount, date) VALUES (%s, %s, '%s', %s, FROM_UNIXTIME('%s'))",
					$client['id'], $mid, $cheque, $amount, $time));
				if($db->hasErrors()){
					$db->Rollback();
					$status = 3;
				}else{
					$db->Commit();
					$status = 9;
				}
			}
		}
		$statuses[] = array($details['chequenumber'], $status);
	}
	$template = new PhpTemplate();
	$template->setFileName('usmp/process-payment-response.php');
	return $template->render(array(
		'statuses'=>$statuses,
	));
}

function handleGetStatus($map){
	$this->checkSerialKeyword($map['serial']->body, $map['keyword']->body);
	$statuses = array();
	foreach($map['chequenumbers']->nodes as $numNode){
		$status = 0;
		$num = $numNode->body;
		$cheque = intval($num);
		if("$cheque"!=$num || $cheque<=0){
			$status = 120;
		}elseif($pay = loadRecord(sprintf("SELECT u.id, m.id as mid FROM usmp_pays u LEFT JOIN movements m ON m.id=u.movement WHERE u.cheque='%s'", addslashes($num)))){
			$status = $pay['mid'] ? 9 : 4;
		}else{
			$status = 123;
		}
		$statuses[] = array($num, $status);
	}
	$template = new PhpTemplate();
	$template->setFileName('usmp/get-status-response.php');
	return $template->render(array(
		'statuses'=>$statuses,
	));
}

function handleValidatePhone($map){
	$this->checkSerialKeyword($map['serial']->body, $map['keyword']->body);
	$opt_pid = getTrimOption('USMP_PAY_ELEMENT_ID');
	$contract = $map['account']->body;
	$pid = $map['payelementid']->body;
	if($pid!=$opt_pid){
		$result = 'false';
		$message = '';
	}elseif($client = loadRecord(sprintf("SELECT id, first_name, last_name FROM clients WHERE contract='%s'", addslashes($contract)))){
		$result = 'true';
		$message = iconv('windows-1251', 'utf-8', htmlspecialchars($client['last_name'].' '.$client['first_name']));
	}else{
		$result = 'false';
		$message = '';
	}
	$template = new PhpTemplate();
	$template->setFileName('usmp/validate-phone-response.php');
	return $template->render(array(
		'result'=>$result,
		'account'=>$contract,
		'message'=>$message,
	));
}

function error($code, $message, $tagName){
	$template = new PhpTemplate();
	$template->setFileName('usmp/error.php');
	return $template->render(array(
		'code'=>$code,
		'message'=>$message,
		'tagName'=>$tagName,
	));
}

}

$xml = '
';

$xml = file_get_contents("php://input");

$handler = new USMPHandler();
$handler->handle($xml);
?>
