set names cp1251;

ALTER TABLE connection_type_resources ADD COLUMN penalty_type TINYINT NOT NULL after penalty_dst;
UPDATE connection_type_resources SET penalty_type=1 WHERE penalty_dst IS NOT NULL;

CREATE TABLE client_shaper_patches (
	id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	client INT UNSIGNED NOT NULL,
	from_time datetime,
	to_time datetime,
	resource_use INT UNSIGNED,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;

ALTER TABLE client_shaper_patches ADD FOREIGN KEY (resource_use) REFERENCES resource_uses (id) ON DELETE SET NULL;

ALTER TABLE connection_types ADD COLUMN unshape_resource_type INT UNSIGNED AFTER abon_resource_type;
ALTER TABLE connection_types ADD COLUMN unshape_dst INT UNSIGNED AFTER unshape_resource_type;
ALTER TABLE connection_types ADD COLUMN unshape_amount BIGINT UNSIGNED NOT NULL AFTER unshape_dst;
ALTER TABLE connection_types ADD FOREIGN KEY (unshape_resource_type) REFERENCES resource_types(id);
ALTER TABLE connection_types ADD FOREIGN KEY (unshape_dst) REFERENCES accounts(id);

alter table client_notes add column type tinyint not null after client;
alter table client_notes add column active tinyint not null after type;

CREATE TABLE jobs (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	street INT UNSIGNED,
	house VARCHAR(64),
	entrance TINYINT,
	floor TINYINT,
	apartment INT,
	client VARCHAR(255) NOT NULL,
	phone VARCHAR(64),
	type TINYINT NOT NULL,
	alarm TINYINT NOT NULL,
	worker INT UNSIGNED,
	date DATETIME,
	notes VARCHAR(255),
	state TINYINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;

CREATE TABLE workers (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;

ALTER TABLE jobs ADD FOREIGN KEY (street) REFERENCES streets (id);
ALTER TABLE jobs ADD FOREIGN KEY (worker) REFERENCES workers (id);
ALTER TABLE jobs ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE workers ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

insert into system_entities (id, key_name, name) values (33, "jobs", "");
insert into system_entities (id, key_name, name) values (34, "workers", "");

alter table connection_types add column allow_enter tinyint not null;
alter table connection_types add column allow_leave tinyint not null;

CREATE TABLE portmone_pays (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	client INT UNSIGNED,
	code VARCHAR(64) NOT NULL UNIQUE,
	amount BIGINT UNSIGNED NOT NULL,
	date datetime not null,
	movement BIGINT UNSIGNED,
	KEY movement (movement)
) ENGINE=InnoDB;

ALTER TABLE portmone_pays ADD FOREIGN KEY (movement) REFERENCES movements (id) ON DELETE SET NULL;
ALTER TABLE portmone_pays ADD FOREIGN KEY (client) REFERENCES clients (id) ON DELETE SET NULL;

alter table config add column control tinyint not null after value;

insert into config (id, value, actual) values ('PORTMONE_PAYEE', '2754', 1);
insert into config (id, value, actual) values ('PORTMONE_LOGIN', 'SHP_LANET', 1);
insert into config (id, value, actual) values ('PORTMONE_PASSWORD', 'lan0228', 1);

CREATE TABLE ibox_pays (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	client INT UNSIGNED,
	terminal BIGINT UNSIGNED NOT NULL,
	receipt BIGINT UNSIGNED NOT NULL,
	amount BIGINT UNSIGNED NOT NULL,
	date datetime not null,
	movement BIGINT UNSIGNED,
	KEY movement (movement),
	UNIQUE(terminal, receipt)
) ENGINE=InnoDB;

ALTER TABLE ibox_pays ADD FOREIGN KEY (movement) REFERENCES movements (id) ON DELETE SET NULL;
ALTER TABLE ibox_pays ADD FOREIGN KEY (client) REFERENCES clients (id) ON DELETE SET NULL;

insert into config (id, value, control, actual) values ('WM_ID', '', 1, 0);
insert into config (id, value, actual) values ('WM_KWM', '', 0);
insert into config (id, value, control, actual) values ('WM_KWM_PASSWORD', '', 3, 0);

ALTER TABLE modems ADD COLUMN up_since DATETIME AFTER check_address;
ALTER TABLE modems ADD COLUMN model VARCHAR(16) NOT NULL AFTER up_since;
ALTER TABLE modems ADD COLUMN firmware VARCHAR(16) NOT NULL AFTER model;

CREATE TABLE modems_states (
	id BIGINT NOT NULL AUTO_INCREMENT PRIMARY KEY,
	modem INT UNSIGNED NOT NULL,
	state TINYINT NOT NULL,
	date DATETIME NOT NULL,
	key (modem),
	key (date)
) ENGINE=InnoDB;
ALTER TABLE modems_states ADD FOREIGN KEY (modem) REFERENCES modems (id) ON DELETE CASCADE;

ALTER TABLE modems ADD COLUMN state_id BIGINT AFTER firmware;

alter table modems ADD COLUMN audit tinyint NOT NULL;

CREATE TABLE staff (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	duties VARCHAR(255) NOT NULL,
	phone VARCHAR(64) NOT NULL,
	mobile VARCHAR(64) NOT NULL,
	notes TEXT,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;

ALTER TABLE staff ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

alter table connection_types add column max_hosts int not null;

alter table blocks add column client_note varchar(255) after note;

alter table departments add column description varchar(255) not null after name;

create table pays_import_head (
	id int not null primary key auto_increment,
	head text not null,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;

ALTER TABLE pays_import_head ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
set names cp1251;
INSERT INTO pays_import_head (id, head) VALUES (1, ' ');
ALTER TABLE pays_import ADD column head INT NOT NULL AFTER id;
ALTER TABLE pays_import ADD key(head);
UPDATE pays_import SET head=1;
ALTER TABLE pays_import ADD FOREIGN KEY (head) REFERENCES pays_import_head (id) ON DELETE CASCADE;

ALTER TABLE accounts ADD COLUMN type char(1) NOT NULL, ADD COLUMN name VARCHAR(255) NOT NULL, ADD KEY(name);
UPDATE system_accounts sa LEFT JOIN accounts a ON a.id=sa.account SET a.type='S', a.name=sa.name;
UPDATE clients c LEFT JOIN accounts a ON a.id=c.account SET a.type='D', a.name=CONCAT(c.last_name, ' ', c.first_name, ' (', c.login, ')');
UPDATE bonus_accounts ba LEFT JOIN accounts a ON a.id=ba.account LEFT JOIN clients c ON c.id=ba.client SET a.type='B', a.name=CONCAT(c.last_name, ' ', c.first_name, ' (', c.login, ') - ', ba.name);

set names cp1251;
INSERT INTO system_entities (id, key_name, name) VALUES (35, 'drweb', 'Dr.WEB');

alter table hosts add column vlan INT NOT NULL after port;

alter table modems add column uplink int not null after port;

alter table personal_periods add column note varchar(255) after marker;

alter table charge_types add column driver VARCHAR(255) after account;

INSERT INTO config (id, value, control, actual) VALUES ('DRWEB_CHARGE_TYPE', '0', 2, 0);
INSERT INTO config (id, value, control, actual) VALUES ('DRWEB_API_URL', 'http://beast.la.net.ua:9080/avdesk/api', 1, 0);
INSERT INTO config (id, value, control, actual) VALUES ('DRWEB_GROUP', 'Lanet', 1, 0);
INSERT INTO config (id, value, control, actual) VALUES ('DRWEB_USER', 'admin', 1, 0);
INSERT INTO config (id, value, control, actual) VALUES ('DRWEB_PASSWORD', '', 1, 0);

alter table modems add column address_space varchar(255) after check_address;

CREATE TABLE mac_vendors (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	code VARCHAR(32) NOT NULL UNIQUE,
	name VARCHAR(32) NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;

ALTER TABLE mac_vendors ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

ALTER TABLE connection_types ADD COLUMN start_always TINYINT NOT NULL after max_hosts;

CREATE TABLE ads (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	period_begin DATETIME,
	period_end DATETIME,
	number INT NOT NULL,
	body TEXT,
	query INT UNSIGNED,
	stamp BIGINT UNSIGNED UNIQUE,
	key(period_begin),
	key(period_end),
	key(query)
) type=InnoDB;

ALTER TABLE ads ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE ads ADD FOREIGN KEY (query) REFERENCES queries (id) ON DELETE SET NULL;

set names cp1251;
insert into system_entities (id, key_name, name) values (36, 'ads', '');
alter table modems_states add column loss tinyint after state;


alter table clients add column mobile varchar(64) after phone;
alter table clients add column passport varchar(32) after contract;
alter table clients add column passport_info varchar(255) after passport;

CREATE TABLE districts (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;
ALTER TABLE districts ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

CREATE TABLE kpreos (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	number VARCHAR(64) NOT NULL UNIQUE,
	street INT UNSIGNED,
	house VARCHAR(16) NOT NULL,
	type TINYINT NOT NULL,
	dispatcher_phone VARCHAR(64),
	commander VARCHAR(255),
	commander_phone VARCHAR(64),
	chief_engineer VARCHAR(255),
	chief_engineer_phone VARCHAR(64),
	xman VARCHAR(255),
	xman_phone VARCHAR(64),
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	key(street)
) ENGINE=InnoDB;

ALTER TABLE kpreos ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE kpreos ADD FOREIGN KEY (street) REFERENCES streets (id);

CREATE TABLE koops (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	street INT UNSIGNED,
	house VARCHAR(16) NOT NULL,
	contract VARCHAR(32),
	type TINYINT NOT NULL,
	commander VARCHAR(255),
	commander_phone VARCHAR(64),
	commander_address VARCHAR(255),
	accounter VARCHAR(255),
	accounter_phone VARCHAR(64),
	props VARCHAR(255),
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	key(street)
) ENGINE=InnoDB;

ALTER TABLE koops ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE koops ADD FOREIGN KEY (street) REFERENCES streets (id);

CREATE TABLE houses (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	street INT UNSIGNED,
	house VARCHAR(16) NOT NULL,
	district INT UNSIGNED,
	state TINYINT,
	queries INT NOT NULL,
	floors TINYINT,
	entrances TINYINT,
	appartments INT,
	served_by TINYINT,
	kpreo INT UNSIGNED,
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	key(street),
	key(district),
	key(kpreo)
) ENGINE=InnoDB;

ALTER TABLE houses ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE houses ADD FOREIGN KEY (street) REFERENCES streets (id);
ALTER TABLE houses ADD FOREIGN KEY (district) REFERENCES districts (id);
ALTER TABLE houses ADD FOREIGN KEY (kpreo) REFERENCES kpreos (id) ON DELETE SET NULL;

set names cp1251;
REPLACE into system_entities (id, key_name, name) values (37, "districts", ": ");
REPLACE into system_entities (id, key_name, name) values (38, "koops", ": ");
REPLACE into system_entities (id, key_name, name) values (39, "kpreos", ":  ");
REPLACE into system_entities (id, key_name, name) values (40, "houses", ": ");

alter table connection_type_shapers add column use_case tinyint not null;

CREATE TABLE troubles (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	client INT UNSIGNED NOT NULL,
	code VARCHAR(16) UNIQUE,
	type TINYINT NOT NULL,
	title VARCHAR(255) NOT NULL,
	body TEXT,
	state TINYINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	key (client)
) ENGINE=InnoDB;
ALTER TABLE troubles ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE troubles ADD FOREIGN KEY (client) REFERENCES clients (id) ON DELETE CASCADE;

CREATE TABLE trouble_files (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	trouble INT UNSIGNED NOT NULL,
	file INT UNSIGNED NOT NULL,
	key(trouble),
	key(file)
) ENGINE=InnoDB;
ALTER TABLE trouble_files ADD FOREIGN KEY (trouble) REFERENCES troubles (id) ON DELETE CASCADE;

CREATE TABLE trouble_answers (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	trouble INT UNSIGNED NOT NULL,
	body TEXT,
	role TINYINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;
ALTER TABLE trouble_answers ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE trouble_answers ADD FOREIGN KEY (trouble) REFERENCES troubles (id) ON DELETE CASCADE;

CREATE TABLE trouble_answer_files (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	answer INT UNSIGNED NOT NULL,
	file INT UNSIGNED NOT NULL,
	key(answer),
	key(file)
) ENGINE=InnoDB;
ALTER TABLE trouble_answer_files ADD FOREIGN KEY (answer) REFERENCES trouble_answers (id) ON DELETE CASCADE;

CREATE DATABASE billing_storage character set cp1251;
use billing_storage;

CREATE TABLE files (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name varchar(255),
	size bigint unsigned NOT NULL,
	type varchar(60),
	filedate datetime,
	key(name)
) ENGINE=InnoDB;

CREATE TABLE file_chunks (
	id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	file_id INT UNSIGNED NOT NULL,
	chunk blob NOT NULL,
	key(file_id)
) ENGINE=InnoDB;
ALTER TABLE file_chunks ADD FOREIGN KEY (file_id) REFERENCES files (id) ON DELETE CASCADE;

use billing;
set names cp1251;
insert into system_entities (id, key_name, name) values (41, "support", "");

alter table operators add column role int unsigned after pass;

CREATE TABLE client_expenses (
	client INT UNSIGNED NOT NULL UNIQUE,
	cable INT NOT NULL,
	cable_in INT NOT NULL,
	connectors INT NOT NULL,
	ties INT NOT NULL,
	clips INT NOT NULL,
	connection INT NOT NULL,
	connection_port INT,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;
ALTER TABLE client_expenses ADD FOREIGN KEY (client) REFERENCES clients (id) ON DELETE CASCADE;
ALTER TABLE client_expenses ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

set names cp1251;
insert into system_entities (id, key_name, name) values (42, "client_expenses", ":  ");

alter table clients add column state tinyint not null after blockable;

insert into config (id, value, control, actual) values ('UKRPAYS_KEY', '71A3b23o811QbM121J23', 1, 1);

drop table if exists ukr_pays;
CREATE TABLE ukr_pays (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	client INT UNSIGNED,
	movement BIGINT UNSIGNED,
	id_ups VARCHAR(64) NOT NULL UNIQUE,
	amount BIGINT UNSIGNED,
	date datetime not null,
	KEY movement (movement)
) ENGINE=InnoDB;
ALTER TABLE ukr_pays ADD FOREIGN KEY (movement) REFERENCES movements (id) ON DELETE SET NULL;
ALTER TABLE ukr_pays ADD FOREIGN KEY (client) REFERENCES clients (id) ON DELETE SET NULL;

insert into system_entities (id, key_name, name) values (43, "ukrpays", "UkrPays");

CREATE TABLE mail_messages (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	subject VARCHAR(255) NOT NULL,
	addr_from VARCHAR(255) NOT NULL,
	addr_to VARCHAR(255) NOT NULL,
	reply_to VARCHAR(255) NOT NULL,
	date datetime,
	msg_id VARCHAR(255) NOT NULL,
	in_reply_to VARCHAR(255) NOT NULL,
	body TEXT,
	type tinyint not null,
	answer_id VARCHAR(64) NOT NULL,
	attachments tinyint not null,
	stamp BIGINT UNSIGNED UNIQUE,
	key (addr_from),
	key (addr_to),
	key (date),
	key (msg_id)
) ENGINE=InnoDB;
ALTER TABLE mail_messages ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

CREATE TABLE mail_attachments (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	message INT UNSIGNED NOT NULL,
	file INT UNSIGNED NOT NULL,
	key(message),
	key(file)
) ENGINE=InnoDB;
ALTER TABLE mail_attachments ADD FOREIGN KEY (message) REFERENCES mail_messages (id) ON DELETE CASCADE;

CREATE TABLE con_requests (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	client VARCHAR(64),
	email VARCHAR(64),
	street_name VARCHAR(64),
	house VARCHAR(16),
	entrance TINYINT,
	floor TINYINT,
	appartment INT,
	phone VARCHAR(64),
	notes VARCHAR(255),
	state tinyint not null,
	stamp BIGINT UNSIGNED UNIQUE
)ENGINE=InnoDB;
ALTER TABLE con_requests ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

alter table operators add column photo_file INT UNSIGNED after role;
alter table operators add index (photo_file);

alter table workers add column photo_file INT UNSIGNED after name;
alter table workers add index (photo_file);

alter table staff add column photo_file INT UNSIGNED after name;
alter table staff add index (photo_file);

alter table con_requests add column inzone tinyint not null after notes;

alter table koops add column pay_type tinyint not null after type;
alter table koops add column pay_amount BIGINT UNSIGNED not null after pay_type;
alter table koops add column kuda varchar(255) after pay_amount;

alter table access_tracker add column connection_type int unsigned after state;
alter table connection_types add column port_speed int unsigned;

CREATE TABLE polls (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	title VARCHAR(255),
	body TEXT, 
	period_begin DATETIME,
	period_end DATETIME,
	priority INT NOT NULL,
	query INT UNSIGNED,
	stamp BIGINT UNSIGNED UNIQUE,
	key(period_begin),
	key(period_end),
	key(query)
) ENGINE=InnoDB;
ALTER TABLE polls ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE polls ADD FOREIGN KEY (query) REFERENCES queries (id) ON DELETE SET NULL;

CREATE table poll_options (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	poll INT UNSIGNED NOT NULL,
	body VARCHAR(255) not null,
	number INT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	key (poll)
) ENGINE=InnoDB;
ALTER TABLE poll_options ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE poll_options ADD FOREIGN KEY (poll) REFERENCES polls (id) ON DELETE CASCADE;

CREATE table poll_votes (
	poll INT UNSIGNED NOT NULL,
	client INT UNSIGNED NOT NULL,
	poll_option INT UNSIGNED NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	primary key(poll, client),
	key(poll),
	key(client)
) ENGINE=InnoDB;
ALTER TABLE poll_votes ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE poll_votes ADD FOREIGN KEY (poll) REFERENCES polls (id) ON DELETE CASCADE;
ALTER TABLE poll_votes ADD FOREIGN KEY (client) REFERENCES clients (id) ON DELETE CASCADE;
ALTER TABLE poll_votes ADD FOREIGN KEY (poll_option) REFERENCES poll_options (id) ON DELETE CASCADE;

ALTER TABLE clients ADD COLUMN house_id INT UNSIGNED NOT NULL DEFAULT 1 AFTER house;
UPDATE clients c LEFT JOIN houses h ON h.street=c.street and CONCAT(c.house, c.letter)=h.house SET c.house_id=h.id;

ALTER TABLE jobs ADD COLUMN found_by TINYINT UNSIGNED NOT NULL AFTER state;
ALTER TABLE jobs ADD COLUMN found_by_client INT UNSIGNED AFTER found_by;
ALTER TABLE jobs ADD COLUMN found_by_memo VARCHAR(255) AFTER found_by_client;
ALTER TABLE jobs ADD KEY(found_by_client);
ALTER TABLE jobs ADD FOREIGN KEY(found_by_client) REFERENCES clients (id) ON DELETE SET NULL;

CREATE TABLE partners (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	photo_file INT UNSIGNED,
	duties VARCHAR(255) NOT NULL,
	phone VARCHAR(64) NOT NULL,
	mobile VARCHAR(64) NOT NULL,
	email VARCHAR(64) NOT NULL,
	company VARCHAR(64) NOT NULL,
	site VARCHAR(64) NOT NULL,
	status tinyint NOT NULL,
	notes TEXT,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;

ALTER TABLE partners ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

insert into config (id, value, control, actual) values ('NONSTOP24_KEY', '', 1, 1);

drop table if exists nonstop24_pays;
CREATE TABLE nonstop24_pays (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	client INT UNSIGNED,
	movement BIGINT UNSIGNED,
	pay_id VARCHAR(64) NOT NULL UNIQUE,
	receipt_num VARCHAR(64),
	trade_point VARCHAR(64),
	amount BIGINT UNSIGNED,
	date datetime not null,
	KEY movement (movement)
) ENGINE=InnoDB;
ALTER TABLE nonstop24_pays ADD FOREIGN KEY (movement) REFERENCES movements (id) ON DELETE SET NULL;
ALTER TABLE nonstop24_pays ADD FOREIGN KEY (client) REFERENCES clients (id) ON DELETE SET NULL;

insert into system_entities (id, key_name, name) values (44, "nonstop24", "24 non Stop");

ALTER TABLE modems ADD COLUMN loader VARCHAR (16) NOT NULL AFTER firmware;
ALTER TABLE modems ADD COLUMN diag VARCHAR (16)  NOT NULL AFTER loader;

drop table if exists usmp_pays;
CREATE TABLE usmp_pays (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	client INT UNSIGNED,
	movement BIGINT UNSIGNED,
	cheque BIGINT UNSIGNED NOT NULL UNIQUE,
	amount BIGINT UNSIGNED,
	date datetime not null,
	KEY movement (movement)
) ENGINE=InnoDB;
ALTER TABLE usmp_pays ADD FOREIGN KEY (movement) REFERENCES movements (id) ON DELETE SET NULL;
ALTER TABLE usmp_pays ADD FOREIGN KEY (client) REFERENCES clients (id) ON DELETE SET NULL;

set names cp1251;
insert into system_entities (id, key_name, name) values (45, "usmp", ": USMP");
update system_entities SET name=": 24 non Stop" WHERE key_name="nonstop24";
update system_entities SET name=": Webmoney" WHERE key_name="webmoney";
update system_entities SET name=": UkrPays" WHERE key_name="ukrpays";
update system_entities SET name=": USMP" WHERE key_name="usmp";

insert into config (id, value, control, actual) values ('USMP_SERIAL', '', 1, 1);
insert into config (id, value, control, actual) values ('USMP_KEYWORD', '', 1, 1);
insert into config (id, value, control, actual) values ('USMP_PAY_ELEMENT_ID', '', 1, 1);

alter table clients add column acl_pref int not null default 2 after notes;

CREATE TABLE snapshots (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	date datetime not null,
	clients INT ,
	clients_new INT ,
	clients_inc INT ,
	payed BIGINT UNSIGNED,
	KEY date(date)
) ENGINE=InnoDB;

alter table modems_states change column loss loss int unsigned;
alter table modems_states add column loss_prev int unsigned after loss;

alter table modems_states change column loss loss_in int unsigned;
alter table modems_states change column loss_prev loss_in_prev int unsigned;

alter table modems_states add column loss_out int unsigned after loss_in_prev;
alter table modems_states add column loss_out_prev int unsigned after loss_out;

alter table hosts add column pair_a_status tinyint not null after countable;
alter table hosts add column pair_b_status tinyint not null after pair_a_status;
alter table hosts add column pair_a_distance smallint not null after pair_b_status;
alter table hosts add column pair_b_distance smallint not null after pair_a_distance;

create table pair_status_names (
	id tinyint not null primary key,
	name varchar(32)
) ENGINE=InnoDB;
insert into pair_status_names (id, name) values (1,  'not tested');
insert into pair_status_names (id, name) values (2,  'ok');
insert into pair_status_names (id, name) values (3,  'open');
insert into pair_status_names (id, name) values (4,  'short');
insert into pair_status_names (id, name) values (5,  'open short');
insert into pair_status_names (id, name) values (6,  'crosstalk');
insert into pair_status_names (id, name) values (7,  'unknown');
insert into pair_status_names (id, name) values (8,  'impedance mismatch');
insert into pair_status_names (id, name) values (9,  'fail');
insert into pair_status_names (id, name) values (10, 'not support');

create table modems_clients (
	modem INT UNSIGNED NOT NULL PRIMARY KEY,
	amount INT NOT NULL
) ENGINE=InnoDB;
alter table modems_clients add foreign key(modem) references modems(id) on delete cascade;

alter table staff add column section int not null after name;
alter table staff add column status int not null after duties;

create table exp_materials (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	unit TINYINT NOT NULL,
	code VARCHAR(64) NOT NULL,
	description TEXT,
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;
ALTER TABLE exp_materials ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

create table exp_points (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	house INT UNSIGNED, 
	entrance TINYINT,
	floor TINYINT,
	type TINYINT,
	place VARCHAR(255),
	notes VARCHAR(255),
	photo_file1 INT UNSIGNED,
	photo_file2 INT UNSIGNED,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;
ALTER TABLE exp_points ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE exp_points ADD FOREIGN KEY (house) REFERENCES houses (id);

create table exp_executors (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) not null UNIQUE,
	type TINYINT,
	func TINYINT,
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;
ALTER TABLE exp_executors ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

create table exp_trunks (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	begin_point INT UNSIGNED,
	end_point INT UNSIGNED,
	type TINYINT NOT NULL,
	material INT UNSIGNED,
	length INT,
	notes VARCHAR (255),
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;
ALTER TABLE exp_trunks ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE exp_trunks ADD FOREIGN KEY (begin_point) REFERENCES exp_points (id);
ALTER TABLE exp_trunks ADD FOREIGN KEY (end_point) REFERENCES exp_points (id);
ALTER TABLE exp_trunks ADD FOREIGN KEY (material) REFERENCES exp_materials (id);

create table hosts_actions (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	action_pattern TEXT,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;
ALTER TABLE hosts_actions ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

set names cp1251;
insert into system_entities (id, key_name, name) values (46, "hosts_actions", ": ");

create table exp_storage_places (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE, 
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;
ALTER TABLE exp_storage_places ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

create table exp_storages (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	material INT UNSIGNED NOT NULL,
	amount BIGINT NOT NULL,
	place INT UNSIGNED NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	UNIQUE(place,material),
	KEY (material),
	KEY(place)
) ENGINE=InnoDB;
ALTER TABLE exp_storages ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE exp_storages ADD FOREIGN KEY (material) REFERENCES exp_materials (id);
ALTER table exp_storages add FOREIGN KEY (place) REFERENCES exp_storage_places (id);

create table exp_storages_in (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	material INT UNSIGNED NOT NULL,
	place TINYINT NOT NULL,
	amount BIGINT NOT NULL,
	date DATETIME,
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	KEY(place,material),
	KEY (material)
) ENGINE=InnoDB;
ALTER TABLE exp_storages_in ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE exp_storages_in ADD FOREIGN KEY (material) REFERENCES exp_materials (id);

create table exp_tasks (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) not null UNIQUE,
	state TINYINT,
	priority TINYINT,
	begin_date DATETIME,
	end_date DATETIME,
	description TEXT,
	executor INT UNSIGNED,
	current_state VARCHAR(255),
	discharge VARCHAR(255),
	file INT UNSIGNED,
	stamp BIGINT UNSIGNED UNIQUE,
	key (executor)
) ENGINE=InnoDB;
ALTER TABLE exp_tasks ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE exp_tasks ADD FOREIGN KEY (executor) REFERENCES exp_executors (id) ON DELETE SET NULL;

create table exp_storages_out (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	material INT UNSIGNED NOT NULL,
	place TINYINT NOT NULL,
	amount BIGINT NOT NULL,
	date DATETIME,
	executor INT UNSIGNED NOT NULL,
	task INT UNSIGNED NOT NULL,
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	KEY(place,material),
	KEY (material),
	KEY (executor),
	KEY (task)
) ENGINE=InnoDB;
ALTER TABLE exp_storages_out ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE exp_storages_out ADD FOREIGN KEY (material) REFERENCES exp_materials (id);
ALTER TABLE exp_storages_out ADD FOREIGN KEY (executor) REFERENCES exp_executors (id);
ALTER TABLE exp_storages_out ADD FOREIGN KEY (task) REFERENCES exp_tasks (id);

create table exp_points_materials (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	point INT UNSIGNED NOT NULL,
	material INT UNSIGNED NOT NULL,
	amount BIGINT NOT NULL,
	KEY(point),
	KEY(material),
	UNIQUE(point, material),
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;
ALTER TABLE exp_points_materials ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE exp_points_materials ADD FOREIGN KEY (point) REFERENCES exp_points (id);
ALTER TABLE exp_points_materials ADD FOREIGN KEY (material) REFERENCES exp_materials (id);

create table exp_tools (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	amount BIGINT NOT NULL,
	code VARCHAR(64) NOT NULL,
	description TEXT,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;
ALTER TABLE exp_tools ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

create table exp_executor_tools (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	executor INT UNSIGNED NOT NULL,
	tool INT UNSIGNED NOT NULL,
	amount BIGINT NOT NULL,
	issue_date DATETIME,
	state VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	KEY(executor),
	KEY(tool)
) ENGINE=InnoDB;
ALTER TABLE exp_executor_tools ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE exp_executor_tools ADD FOREIGN KEY (executor) REFERENCES exp_executors (id);
ALTER TABLE exp_executor_tools ADD FOREIGN KEY (tool) REFERENCES exp_tools (id);

create table exp_storage_tools (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	place INT UNSIGNED NOT NULL,
	tool INT UNSIGNED NOT NULL,
	amount BIGINT NOT NULL,
	state VARCHAR(255),
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	KEY(place),
	KEY(tool)
) ENGINE=InnoDB;
ALTER TABLE exp_storage_tools ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE exp_storage_tools ADD FOREIGN KEY (place) REFERENCES exp_storage_places (id);
ALTER TABLE exp_storage_tools ADD FOREIGN KEY (tool) REFERENCES exp_tools (id);

create table exp_awaiting_tools (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	source VARCHAR(255) NOT NULL,
	tool INT UNSIGNED NOT NULL,
	amount BIGINT NOT NULL,
	date DATETIME,
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	KEY(tool)
) ENGINE=InnoDB;
ALTER TABLE exp_awaiting_tools ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE exp_awaiting_tools ADD FOREIGN KEY (tool) REFERENCES exp_tools (id);

ALTER TABLE personal_period_plans ADD COLUMN use_value BIGINT UNSIGNED NOT NULL after use_actual;

ALTER TABLE connection_type_shapers ADD COLUMN use_resource_type INT UNSIGNED AFTER use_case;
ALTER TABLE connection_type_shapers ADD COLUMN use_value BIGINT UNSIGNED NOT NULL AFTER use_resource_type;
ALTER TABLE connection_type_shapers ADD KEY (use_resource_type);
ALTER TABLE connection_type_shapers ADD FOREIGN KEY (use_resource_type) REFERENCES resource_types(id) ON DELETE CASCADE; 

create table exp_in_use_materials (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	point INT UNSIGNED NOT NULL,
	material INT UNSIGNED NOT NULL,
	amount BIGINT NOT NULL,
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	KEY(point),
	KEY(material)
) ENGINE=InnoDB;
ALTER TABLE exp_in_use_materials ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE exp_in_use_materials ADD FOREIGN KEY (point) REFERENCES exp_points (id);
ALTER TABLE exp_in_use_materials ADD FOREIGN KEY (material) REFERENCES exp_materials (id);

create table exp_off_materials (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	place INT UNSIGNED NOT NULL,
	material INT UNSIGNED NOT NULL,
	amount BIGINT NOT NULL,
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	KEY(place),
	KEY(material)
) ENGINE=InnoDB;
ALTER TABLE exp_off_materials ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE exp_off_materials ADD FOREIGN KEY (place) REFERENCES exp_storage_places (id);
ALTER TABLE exp_off_materials ADD FOREIGN KEY (material) REFERENCES exp_materials (id);

create table exp_awaiting_materials (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	source VARCHAR(255) NOT NULL,
	material INT UNSIGNED NOT NULL,
	amount BIGINT NOT NULL,
	date DATETIME,
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	KEY(material)
) ENGINE=InnoDB;
ALTER TABLE exp_awaiting_materials ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE exp_awaiting_materials ADD FOREIGN KEY (material) REFERENCES exp_materials (id);

CREATE TABLE exp_executors_staff (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	executor INT UNSIGNED NOT NULL,
	member INT UNSIGNED NOT NULL,
	role  VARCHAR(255),
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	KEY (executor),
	KEY(member)
) ENGINE=InnoDB;
ALTER TABLE exp_executors_staff ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE exp_executors_staff ADD FOREIGN KEY (executor) REFERENCES exp_executors (id) ON DELETE CASCADE;
ALTER TABLE exp_executors_staff ADD FOREIGN KEY (member) REFERENCES staff (id) ON DELETE CASCADE;

-- ALTER TABLE koops ADD COLUMN house_id INT UNSIGNED AFTER house;
-- UPDATE koops k LEFT JOIN houses h ON h.street=k.street and k.house=h.house SET k.house_id=h.id;

CREATE TABLE koop_pays (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	koop INT UNSIGNED NOT NULL,
	date datetime NOT NULL,
	status TINYINT NOT NULL,
	act TINYINT NOT NULL,
	amount BIGINT NOT NULL,
	notes VARCHAR(255),
	UNIQUE(koop, date),
	stamp BIGINT UNSIGNED UNIQUE,
	KEY(koop)
) ENGINE=InnoDB;
ALTER TABLE koop_pays ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE koop_pays ADD FOREIGN KEY (koop) REFERENCES koops (id) ON DELETE CASCADE;


set names cp1251;
insert into system_entities (id, key_name, name) values (47, "exp_materials", ": ");
insert into system_entities (id, key_name, name) values (48, "exp_points", ": ");
insert into system_entities (id, key_name, name) values (49, "exp_executors", ": ");
insert into system_entities (id, key_name, name) values (50, "exp_trunks", ": ");
insert into system_entities (id, key_name, name) values (51, "exp_storages", ": ");
insert into system_entities (id, key_name, name) values (52, "exp_tasks", ": ");
insert into system_entities (id, key_name, name) values (53, "exp_tools", ": ");
delete from system_entities  where key_name='games';

alter table connection_types ADD COLUMN allow_credits TINYINT NOT NULL DEFAULT 1 AFTER allow_leave;

alter table houses add column contract tinyint not null after notes;

create table admin_districts (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	stamp BIGINT UNSIGNED UNIQUE
)ENGINE=InnoDB;
ALTER TABLE admin_districts ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

alter table districts add column admin INT UNSIGNED after name;
alter table districts add KEY (admin);
alter table districts add FOREIGN KEY (admin) REFERENCES admin_districts(id) ON DELETE SET NULL;

alter table exp_materials ADD column photo_file int unsigned after notes;
alter table exp_tools ADD column photo_file int unsigned after description;

create table vlans (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	number INT NOT NULL UNIQUE,
	name VARCHAR(255) NOT NULL UNIQUE,
	addresses VARCHAR(255) NOT NULL,
	description VARCHAR(255),
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;
ALTER TABLE vlans ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

insert into system_entities (id, key_name, name) values (54, "vlans", "Vlans");

create table drweb_types (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	group_name VARCHAR(255) NOT NULL,
	charge_type INT UNSIGNED NOT NULL,
	actual TINYINT NOT NULL,
	description VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	KEY(charge_type)
) ENGINE=InnoDB;
ALTER TABLE drweb_types ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE drweb_types ADD FOREIGN KEY (charge_type) REFERENCES charge_types (id);

create table drweb_subscriptions (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	subscription INT UNSIGNED UNIQUE,
	drweb_type INT UNSIGNED NOT NULL,
	drweb_user VARCHAR(255) NOT NULL UNIQUE,
	key(drweb_type)
) ENGINE=InnoDB;
ALTER TABLE drweb_subscriptions ADD FOREIGN KEY (subscription) REFERENCES charge_subscriptions (id) ON DELETE CASCADE;
ALTER TABLE drweb_subscriptions ADD FOREIGN KEY (drweb_type) REFERENCES drweb_types (id);

ALTER TABLE browsers ADD column user_agent VARCHAR(255) NOT NULL;

alter table clients add column last_browser int unsigned after acl_pref;
alter table clients add key(last_browser);

create table browser_to_client (
	browser int unsigned not null,
	client int unsigned not null,
	primary key(browser, client),
	key (client),
	key (browser)
);
alter table browser_to_client add foreign key(browser) references borwsers on delete cascade;
alter table browser_to_client add foreign key(client) references clients on delete cascade;

/*
alter table hosts add column model varchar(255);
alter table hosts add column department int unsigned, add key(department);
alter table hosts add column street int unsigned, add key(street);
alter table hosts add column house varchar(32) not null;
alter table hosts add column entrance int;
alter table hosts add column floor int;

alter table hosts add foreign key (department) references departments(id);
alter table hosts add foreign key (street) references streets(id);

alter table hosts add column type tinyint not null;

create table hardwares (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	description VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE
)ENGINE=InnoDB;
ALTER TABLE hardwares ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

create table hosts_hardwares (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	host int unsigned not null,
	hardware int unsigned not null,
	serial VARCHAR(255),
	amount INT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	KEY(host),
	KEY(hardware),
	UNIQUE(hardware, serial)
)ENGINE=InnoDB;
ALTER TABLE hosts_hardwares ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE hosts_hardwares ADD FOREIGN KEY (host) REFERENCES hosts (id) ON DELETE CASCADE;
ALTER TABLE hosts_hardwares ADD FOREIGN KEY (hardware) REFERENCES hardwares (id);

*/

alter table connection_types add column public TINYINT NOT NULL DEFAULT 1;

CREATE TABLE client_calls (
	id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	client INT UNSIGNED NOT NULL,
	type tinyint NOT NULL,
	start_status tinyint NOT NULL,
	off_rating tinyint NOT NULL,
	off_reason tinyint NOT NULL,
	off_return tinyint NOT NULL,
	note TEXT,
	stamp BIGINT UNSIGNED UNIQUE,
	KEY client (client)
) ENGINE=InnoDB;
ALTER TABLE client_calls ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
ALTER TABLE client_calls ADD FOREIGN KEY (client) REFERENCES clients (id) ON DELETE CASCADE;

ALTER TABLE credits ADD column type TINYINT NOT NULL after amount;

ALTER TABLE jobs ADD COLUMN client_id INT UNSIGNED AFTER id;
ALTER TABLE jobs ADD KEY(client_id);
ALTER TABLE jobs ADD FOREIGN KEY(client_id) REFERENCES clients (id) ON DELETE SET NULL; 

alter table jobs add column worker_op int unsigned after worker;
alter table jobs add key (worker_op), add foreign key (worker_op) references operators (id) on delete set null;
update jobs j left join workers w on w.id=j.worker left join operators o on o.name=w.name set j.worker_op=o.id;

set names cp1251;
insert into system_entities (id, key_name, name) values (55, "jobs2", ":  ");

alter table networks add column domain int unsigned after version, add key(domain), add foreign key (domain) references domains (id) on delete set null;

CREATE TABLE modems_models (
	id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	model VARCHAR(255) NOT NULL UNIQUE,
	firmware varchar(16) NOT NULL,
	loader varchar(16) NOT NULL,
	diag varchar(16) NOT NULL,
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;
ALTER TABLE modems_models ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

alter table koops ADD column contract_file int unsigned after contract;

alter table koop_pays ADD column act_file int unsigned after act;

alter table exp_points add column executor int unsigned after type, add key(executor), add foreign key(executor) references exp_executors (id) on delete set null;
alter table exp_points add column date datetime after notes, add key(date);

alter table exp_trunks add column executor int unsigned after type, add key(executor), add foreign key(executor) references exp_executors (id) on delete set null; 
alter table exp_trunks add column date datetime after notes, add key(date);

alter table houses add column koop int unsigned after kpreo, add index (koop), add foreign key(koop) references koops (id) on delete set null;

CREATE TABLE offline_ports (
	modem INT UNSIGNED NOT NULL,
	port INT NOT NULL,
	client INT UNSIGNED,
	date DATETIME,
	host INT UNSIGNED,
	PRIMARY KEY(modem, port),
	INDEX (modem),
	INDEX (client),
	INDEX (date),
	INDEX (host)
) ENGINE=InnoDB;
ALTER TABLE offline_ports 
	ADD FOREIGN KEY (modem) REFERENCES modems (id) ON DELETE CASCADE,
	ADD FOREIGN KEY (client) REFERENCES clients (id) ON DELETE SET NULL;
	
ALTER TABLE operators ADD COLUMN access_zone VARCHAR(255) AFTER role;

ALTER TABLE operators ADD column sip_id VARCHAR(255) UNIQUE after access_zone;

set names cp1251;
insert into system_entities (id, key_name, name) values (56, "support_asterisk", ": ");

CREATE TABLE delayed_sequences (
	id BIGINT NOT NULL AUTO_INCREMENT PRIMARY KEY,
	operator INT UNSIGNED NOT NULL
) ENGINE=InnoDB;

CREATE TABLE delayed_tasks (
	id BIGINT NOT NULL AUTO_INCREMENT PRIMARY KEY,
	proc_type TINYINT NOT NULL,
	data VARCHAR(255) NOT NULL,
	sequence BIGINT NOT NULL,
	max_attempts TINYINT NOT NULL,
	try_count TINYINT NOT NULL,
	status TINYINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;
ALTER TABLE delayed_tasks ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

ALTER TABLE modems_clients ADD column amount_2 INT NOT NULL AFTER amount;

ALTER TABLE modems ADD COLUMN cacti_id INT;

drop table if exists easy_pays;
CREATE TABLE easy_pays (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	client INT UNSIGNED,
	movement BIGINT UNSIGNED,
	service_id BIGINT,
	order_id BIGINT,
	amount BIGINT UNSIGNED,
	date datetime not null,
	KEY movement (movement),
	UNIQUE (service_id, order_id)
) ENGINE=InnoDB;
ALTER TABLE easy_pays ADD FOREIGN KEY (movement) REFERENCES movements (id) ON DELETE SET NULL;
ALTER TABLE easy_pays ADD FOREIGN KEY (client) REFERENCES clients (id) ON DELETE SET NULL;
ALTER TABLE easy_pays ADD COLUMN canceled DATETIME;

insert into config (id, value, control, actual) values ('EASYSOFT_TOKEN', 'easy', 1, 1);
insert into config (id, value, control, actual) values ('EASYSOFT_PROVIDER_PRIVATE', '', 0, 0);
insert into config (id, value, control, actual) values ('EASYSOFT_EASY_PUBLIC', '', 0, 0);

set names cp1251;
insert into system_entities (id, key_name, name) values (57, "easy", ": EasySoft");

-- create table ecall_uploads (
--	id int not null primary key auto_increment,
--	filename varchar(255) not null,
--	stamp BIGINT UNSIGNED UNIQUE
-- ) ENGINE=InnoDB;
-- ALTER TABLE ecall_uploads ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

create table ecalls (
	id int not null primary key auto_increment,
	filename varchar(255),
	ecall_id int,
	street varchar(64),
	house varchar(32),
	apartment varchar(32),
	contact varchar(255),
	phone varchar(64),
	mobile varchar(64),
	tariff varchar(128),
	notes varchar(255),
	date datetime,
	state tinyint not null,
	recall_date datetime,
	connection_type int unsigned,
	assumed_house int unsigned,
	stamp BIGINT UNSIGNED UNIQUE,
	index(connection_type)
) ENGINE=InnoDB;
ALTER TABLE ecalls 
	ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	ADD FOREIGN KEY (connection_type) REFERENCES connection_types (id) ON DELETE SET NULL,
	ADD FOREIGN KEY (assumed_house) REFERENCES houses (id) ON DELETE SET NULL;

set names cp1251;
insert into system_entities (id, key_name, name) values (58, "ecalls", "eCall");

CREATE TABLE ecalls_back (
	id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	ecall INT NOT NULL,
	state tinyint not null,
	recall_date datetime,
	note TEXT,
	stamp BIGINT UNSIGNED UNIQUE,
	KEY ecall (ecall)
) ENGINE=InnoDB;
ALTER TABLE ecalls_back 
	ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	ADD FOREIGN KEY (ecall) REFERENCES ecalls (id) ON DELETE CASCADE;

ALTER TABLE houses ADD column post_office VARCHAR(16) after koop;

ALTER TABLE jobs 
	ADD column used_before TINYINT NOT NULL after state,
	ADD column used_before_memo VARCHAR(128) after used_before;
	
ALTER TABLE modems ADD column type TINYINT NOT NULL default 1;

CREATE TABLE trunk_modems (
	id INT UNSIGNED NOT NULL PRIMARY KEY,
	ip_core_route VARCHAR(32),
	real_ip_vlan INT
) ENGINE=InnoDB;
ALTER TABLE trunk_modems ADD FOREIGN KEY(id) REFERENCES modems(id) ON DELETE CASCADE;

ALTER TABLE modems
	ADD column exp_point INT UNSIGNED,
	ADD KEY (exp_point),
	ADD FOREIGN KEY (exp_point) REFERENCES exp_points(id) ON DELETE SET NULL; 
	
ALTER TABLE modems
	ADD column power TINYINT NOT NULL after diag;

ALTER TABLE streets ADD column abbr VARCHAR(32) AFTER name;

ALTER TABLE exp_points 
	ADD column coord_1 VARCHAR(32),
	ADD column coord_2 VARCHAR(32);
	
CREATE TABLE modems_power_history (
	id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	modem INT UNSIGNED NOT NULL,
	power TINYINT NOT NULL,
	date datetime not null,
	key (date),
	key(modem)
) ENGINE=InnoDB;
ALTER TABLE modems_power_history
	ADD FOREIGN KEY (modem) REFERENCES modems(id) ON DELETE CASCADE;
	
ALTER TABLE
	client_expenses ADD COLUMN cable_type TINYINT NOT NULL DEFAULT 2;

-- Add tregub
CREATE TABLE `snmp_status` (
  `id` int(11) NOT NULL auto_increment,
  `client_id` int(11) default NULL,
  `open_status` int(11) default NULL,
  `admin_status` int(11) default NULL,
  `duplex_status` int(11) default NULL,
  `duplex_mode` int(11) default NULL,
  `security_status` int(11) default NULL,
  `static_address_table` int(11) default NULL,
  `sip_mac` int(11) default NULL,
  `ip_source_guard_binding` int(11) default NULL,
  `dynamic_adresses_table` text,
  `time` int(11) default NULL,
  PRIMARY KEY  (`id`)
) ENGINE=InnoDB;
-- End 

ALTER TABLE client_expenses ADD COLUMN id INT NOT NULL AUTO_INCREMENT PRIMARY KEY;
alter table client_expenses drop key client, add key (client);
ALTER TABLE client_expenses ADD COLUMN work_type TINYINT NOT NULL;

ALTER TABLE delayed_tasks ADD COLUMN fork TINYINT NOT NULL;
ALTER TABLE delayed_tasks ADD COLUMN exec_time INT NOT NULL;

drop table if exists tech_zones;
CREATE TABLE tech_zones (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(64) NOT NULL UNIQUE,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;
ALTER TABLE tech_zones 
	ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

drop table if exists tech_zone_districts;
CREATE TABLE tech_zone_districts (
	tech_zone INT UNSIGNED NOT NULL,
	district INT UNSIGNED NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	PRIMARY KEY(tech_zone, district),
	KEY (district)
) ENGINE=InnoDB;
ALTER TABLE tech_zone_districts
	ADD FOREIGN KEY (tech_zone) REFERENCES tech_zones (id) ON DELETE CASCADE,
	ADD FOREIGN KEY (district) REFERENCES districts (id) ON DELETE CASCADE,
	ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
	
drop table if exists tech_zone_times;
CREATE TABLE tech_zone_times (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	tech_zone INT UNSIGNED NOT NULL,
	time_name VARCHAR(32),
	from_time TINYINT NOT NULL,
	to_time TINYINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	KEY (tech_zone)
) ENGINE=InnoDB;
ALTER TABLE tech_zone_times
	ADD FOREIGN KEY (tech_zone) REFERENCES tech_zones (id) ON DELETE CASCADE,
	ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

drop table if exists tech_zones_graph;
CREATE TABLE tech_zones_graph (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	operator int unsigned NOT NULL,
	date datetime not null,
	tech_zone_time INT UNSIGNED NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	KEY(operator),
	KEY(tech_zone_time)
) ENGINE=InnoDB;
ALTER TABLE tech_zones_graph 
	ADD FOREIGN KEY (operator) REFERENCES operators (id) ON DELETE CASCADE,
	ADD FOREIGN KEY (tech_zone_time) REFERENCES tech_zone_times (id) ON DELETE CASCADE,
	ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;
	
alter table tech_zones_graph add unique(operator,date,tech_zone_time);
alter table tech_zone_districts add unique(district);

alter table ecalls add column concurent varchar(64), add column ecall_operator varchar(32);

alter table clients add key(house_id, apartment);

alter table operators add column person int unsigned, add key(person), add foreign key (person) references staff (id) on delete cascade;

CREATE TABLE `request_emails` (
  `id` int(11) NOT NULL auto_increment,
  `order_id` int(11) default NULL,
  `m_subject` text,
  `m_from` text,
  `m_to` text,
  `message` text,
  `stamp` int(11) default NULL,
  PRIMARY KEY  (`id`)
) ENGINE=InnoDB;


ALTER TABLE staff ADD COLUMN sip_id VARCHAR(255) AFTER mobile;
UPDATE staff s LEFT JOIN operators o ON o.person=s.id SET s.sip_id = o.sip_id WHERE o.sip_id > '';

ALTER TABLE staff ADD COLUMN email VARCHAR(64) AFTER sip_id;

CREATE TABLE ranks (
	id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(32) NOT NULL UNIQUE,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;
ALTER TABLE ranks ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

ALTER TABLE staff ADD COLUMN rank INT AFTER duties, ADD KEY(rank), ADD FOREIGN KEY(rank) REFERENCES ranks(id) ON DELETE SET NULL;

INSERT INTO ranks (name) select duties from staff where duties>'' group by duties order by count(*) desc;
UPDATE staff s LEFT JOIN ranks r ON r.name=s.duties SET s.rank=r.id WHERE r.id IS NOT NULL;

ALTER TABLE ranks ADD COLUMN accept_jobs TINYINT NOT NULL AFTER name;

alter table exp_points add column inv_number int after coord_2;
alter table exp_tasks add column complete datetime after stamp;

alter table jobs drop foreign key `jobs_ibfk_2`;

update jobs set worker=null;
alter table jobs add foreign key (worker) references staff(id) on delete set null;

UPDATE jobs j LEFT JOIN operators o ON o.id=j.worker_op SET j.worker=o.person;

ALTER TABLE tech_zones_graph ADD COLUMN person INT UNSIGNED, ADD KEY(person), ADD FOREIGN KEY (person) REFERENCES staff (id) ON DELETE CASCADE;
UPDATE tech_zones_graph tzg LEFT JOIN operators o ON o.id=tzg.operator SET tzg.person=o.person; 

ALTER TABLE tech_zones_graph CHANGE COLUMN operator operator INT UNSIGNED; 

alter table jobs change column state state TINYINT NOT NULL;

ALTER table tech_zones_graph DROP KEY `operator_2`, ADD UNIQUE(person,date,tech_zone_time);

CREATE TABLE groups (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(32) NOT NULL UNIQUE,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;
ALTER TABLE groups ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

CREATE TABLE groups_access (
	group_id INT UNSIGNED NOT NULL,
	entity INT UNSIGNED NOT NULL,
	access TINYINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	PRIMARY KEY(group_id, entity),
	KEY (entity)
) ENGINE=InnoDB;
ALTER TABLE groups_access 
	ADD FOREIGN KEY (group_id) REFERENCES groups (id) ON DELETE CASCADE,
	ADD FOREIGN KEY (entity) REFERENCES system_entities (id) ON DELETE CASCADE,
	ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

ALTER TABLE operators 
	ADD COLUMN group_id INT UNSIGNED AFTER department, 
	ADD KEY(group_id), 
	ADD FOREIGN KEY (group_id) REFERENCES groups (id) ON DELETE SET NULL;

CREATE TABLE `dispatching` (
  `id` int(11) NOT NULL auto_increment,
  `kp_reo` int(11) unsigned default NULL,
  `street` int(11) unsigned default NULL,
  `house` varchar(16) default NULL,
  `phone` varchar(64) default NULL,
  `notes` varchar(255) default NULL,
  `stamp` bigint(20) unsigned default NULL,
  PRIMARY KEY  (`id`),
  KEY `reos_ibfk_1` (`kp_reo`),
  KEY `reos_ibfk_2` (`stamp`),
  KEY `reos_ibfk_3` (`street`),
  CONSTRAINT `reos_ibfk_1` FOREIGN KEY (`kp_reo`) REFERENCES `kpreos` (`id`) ON DELETE SET NULL,
  CONSTRAINT `reos_ibfk_2` FOREIGN KEY (`stamp`) REFERENCES `stamps` (`id`) ON DELETE SET NULL,
  CONSTRAINT `reos_ibfk_3` FOREIGN KEY (`street`) REFERENCES `streets` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=32 DEFAULT CHARSET=cp1251;

ALTER TABLE houses 
	ADD COLUMN dispatching INT AFTER contract, 
	ADD FOREIGN KEY (dispatching) REFERENCES dispatching (id) ON DELETE SET NULL;
	
ALTER TABLE exp_points CHANGE column inv_number inv_number varchar (64);

ALTER TABLE ecalls ADD COLUMN sale_date DATETIME;

update system_entities set name='' where key_name='workers';

INSERT INTO system_entities (id, key_name, name) VALUES (59, 'workers_private', ' ');

alter table staff add column private tinyint not null after notes;

set names cp1251;
INSERT INTO system_entities (id, key_name, name) VALUES (60, 'partners', '');
insert into operators_access (operator, entity, access)  select operator, 60, access from operators_access where entity = 34;

set names cp1251;
INSERT INTO system_entities (id, key_name, name) VALUES (61, 'worker_features', '  ');
INSERT INTO operators_access (operator, entity, access) SELECT id, 61, 1 FROM operators WHERE role=1;

set names cp1251;
INSERT INTO system_entities (id, key_name, name) VALUES (62, 'ranks', '');
INSERT INTO operators_access (operator, entity, access) SELECT o.id, 62, oa.access FROM operators_access oa LEFT JOIN operators o ON o.id=oa.operator WHERE o.role!=1 AND oa.entity=34;

CREATE TABLE tech_zones_graph_template (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	person INT UNSIGNED NOT NULL,
	day TINYINT NOT NULL,
	tech_zone_time INT UNSIGNED NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	KEY(person),
	KEY(tech_zone_time)
) ENGINE=InnoDB;
ALTER TABLE tech_zones_graph_template 
	ADD FOREIGN KEY (person) REFERENCES staff (id) ON DELETE CASCADE,
	ADD FOREIGN KEY (tech_zone_time) REFERENCES tech_zone_times (id) ON DELETE CASCADE,
	ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	ADD UNIQUE(person,day,tech_zone_time);

	
ALTER TABLE ranks ADD column group_id INT UNSIGNED AFTER accept_jobs; 

ALTER TABLE staff ADD COLUMN work_phone VARCHAR(64) AFTER phone;

UPDATE operators o LEFT JOIN staff s ON s.id=o.person SET o.group_id=5 WHERE s.status=4;

alter table ranks change column name name varchar(255);

ALTER TABLE staff ADD COLUMN birthday DATETIME AFTER email;

CREATE TABLE menu_items (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	title VARCHAR(255) NOT NULL UNIQUE,
	link VARCHAR(255) NOT NULL,
	access_by INT UNSIGNED,
	access_level INT NOT NULL
) ENGINE=InnoDB;
ALTER TABLE menu_items ADD FOREIGN KEY (access_by) REFERENCES system_entities (id);

CREATE TABLE menu_operator_boxes (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	operator INT UNSIGNED NULL,
	title VARCHAR(255) NOT NULL,
	order_num INT NOT NULL,
	KEY (operator)
) ENGINE=InnoDB;
ALTER TABLE menu_operator_boxes ADD FOREIGN KEY (operator) REFERENCES operators(id) ON DELETE CASCADE ON UPDATE CASCADE;

CREATE TABLE menu_operator_items (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	operator INT UNSIGNED NULL,
	menu_item INT UNSIGNED NOT NULL,
	menu_box INT UNSIGNED NOT NULL,
	order_num INT NOT NULL,
	custom_title VARCHAR(128),
	UNIQUE (operator, menu_item)
) ENGINE=InnoDB;
ALTER TABLE menu_operator_items 
	ADD FOREIGN KEY (operator) REFERENCES operators(id) ON DELETE CASCADE,
	ADD FOREIGN KEY (menu_box) REFERENCES menu_operator_boxes(id) ON DELETE CASCADE ON UPDATE CASCADE,
	ADD FOREIGN KEY (menu_item) REFERENCES menu_items (id) ON DELETE CASCADE;

set names cp1251;
insert into menu_items (id, title, link, access_by, access_level) values (1, '', 'clients.php', 1, 0);
insert into menu_items (id, title, link, access_by, access_level) values (2, '', 'movements.php', 3, 0);
insert into menu_items (id, title, link, access_by, access_level) values (3, '', 'blocks.php', 1, 0);
insert into menu_items (id, title, link, access_by, access_level) values (4, '', 'notes.php', 1, 0);
insert into menu_items (id, title, link, access_by, access_level) values (5, '', 'calls.php', 1, 0);
insert into menu_items (id, title, link, access_by, access_level) values (6, ' ', 'client_shaper_patches.php', 1, 0);
insert into menu_items (id, title, link, access_by, access_level) values (7, ' ', 'connection_types.php', 5, 0);
insert into menu_items (id, title, link, access_by, access_level) values (8, '', 'bonuses.php', 22, 0);
insert into menu_items (id, title, link, access_by, access_level) values (9, ' ', 'calendar_periods.php', 30, 0);
insert into menu_items (id, title, link, access_by, access_level) values (10, ' ', 'personal_periods.php', 12, 0);
insert into menu_items (id, title, link, access_by, access_level) values (11, '', 'queries.php', 27, 0);
insert into menu_items (id, title, link, access_by, access_level) values (12, '', 'cards.php', 18, 0);
insert into menu_items (id, title, link, access_by, access_level) values (13, 'web money', 'webmoney.php', 19, 0);
insert into menu_items (id, title, link, access_by, access_level) values (14, 'UkrPays', 'ukrpays.php', 43, 0);
insert into menu_items (id, title, link, access_by, access_level) values (15, '24 non Stop', 'nonstop24.php', 44, 0);
insert into menu_items (id, title, link, access_by, access_level) values (16, 'usmp', 'usmp.php', 45, 0);
insert into menu_items (id, title, link, access_by, access_level) values (17, 'EasySoft', 'easy.php', 57, 0);
insert into menu_items (id, title, link, access_by, access_level) values (18, 'portmone', 'portmone.php', 19, 0);
insert into menu_items (id, title, link, access_by, access_level) values (19, 'ibox', 'ibox.php', 19, 0);
insert into menu_items (id, title, link, access_by, access_level) values (20, ' ', 'pays_import.php', 3, 1);
insert into menu_items (id, title, link, access_by, access_level) values (21, 'Dr.Web', 'drweb.php', 35, 0);
insert into menu_items (id, title, link, access_by, access_level) values (22, '', 'ads.php', 36, 0);
insert into menu_items (id, title, link, access_by, access_level) values (23, '  ', 'mailbox_types.php', 26, 0);
insert into menu_items (id, title, link, access_by, access_level) values (24, ' ', 'mailboxes.php', 20, 0);
insert into menu_items (id, title, link, access_by, access_level) values (25, '  ', 'homedir_types.php', 25, 0);
insert into menu_items (id, title, link, access_by, access_level) values (26, ' ', 'homedirs.php', 21, 0);
insert into menu_items (id, title, link, access_by, access_level) values (27, ' ', 'charge_types.php', 11, 0);
insert into menu_items (id, title, link, access_by, access_level) values (28, '', 'charge_subscriptions.php', 13, 0);
insert into menu_items (id, title, link, access_by, access_level) values (29, '', 'troubles.php', 41, 0);
insert into menu_items (id, title, link, access_by, access_level) values (30, '', 'answers.php', 41, 0);
insert into menu_items (id, title, link, access_by, access_level) values (31, '', 'mail_messages.php', 41, 0);
insert into menu_items (id, title, link, access_by, access_level) values (32, '', 'con_requests.php', 41, 0);
insert into menu_items (id, title, link, access_by, access_level) values (33, ' asterisk', 'stats_asterisk.php', 56, 0);
insert into menu_items (id, title, link, access_by, access_level) values (34, '', 'streets.php', 2, 0);
insert into menu_items (id, title, link, access_by, access_level) values (35, '', 'houses.php', 40, 0);
insert into menu_items (id, title, link, access_by, access_level) values (36, '', 'districts.php', 37, 0);
insert into menu_items (id, title, link, access_by, access_level) values (37, '. ', 'admin_districts.php', 37, 0);
insert into menu_items (id, title, link, access_by, access_level) values (38, ' ', 'kpreos.php', 39, 0);
insert into menu_items (id, title, link, access_by, access_level) values (39, '', 'koops.php', 38, 0);
insert into menu_items (id, title, link, access_by, access_level) values (40, '', 'dispatching.php', 39, 0);
insert into menu_items (id, title, link, access_by, access_level) values (41, '', 'calendar.php', 33, 0);
insert into menu_items (id, title, link, access_by, access_level) values (42, ' ', 'calendar2.php', 55, 0);
insert into menu_items (id, title, link, access_by, access_level) values (43, ' ', 'client_expenses.php', 42, 0);
insert into menu_items (id, title, link, access_by, access_level) values (44, '', 'staff.php', 34, 0);
insert into menu_items (id, title, link, access_by, access_level) values (45, '', 'ranks.php', 62, 0);
insert into menu_items (id, title, link, access_by, access_level) values (46, '', 'partners.php', 60, 0);
insert into menu_items (id, title, link, access_by, access_level) values (47, '  ', 'tech_zones.php', 33, 0);
insert into menu_items (id, title, link, access_by, access_level) values (48, '  ', 'tech_zones_graph_day.php', 33, 0);
insert into menu_items (id, title, link, access_by, access_level) values (49, '', 'exp_materials.php', 47, 0);
insert into menu_items (id, title, link, access_by, access_level) values (50, '', 'exp_points.php', 48, 0);
insert into menu_items (id, title, link, access_by, access_level) values (51, '', 'exp_storages.php', 51, 0);
insert into menu_items (id, title, link, access_by, access_level) values (52, ' ', 'exp_storage_places.php', 51, 0);
insert into menu_items (id, title, link, access_by, access_level) values (53, '', 'exp_tools.php', 53, 0);
insert into menu_items (id, title, link, access_by, access_level) values (54, '', 'exp_tasks.php', 52, 0);
insert into menu_items (id, title, link, access_by, access_level) values (55, '', 'exp_trunks.php', 50, 0);
insert into menu_items (id, title, link, access_by, access_level) values (56, '', 'exp_executors.php', 49, 0);
insert into menu_items (id, title, link, access_by, access_level) values (57, '', 'hosts.php', 4, 0);
insert into menu_items (id, title, link, access_by, access_level) values (58, 'firewall', 'hosts_firewall.php', 4, 0);
insert into menu_items (id, title, link, access_by, access_level) values (59, ' ()', 'hosts_actions.php', 46, 0);
insert into menu_items (id, title, link, access_by, access_level) values (60, '', 'rules.php', 7, 0);
insert into menu_items (id, title, link, access_by, access_level) values (61, '', 'tariffs.php', 6, 0);
insert into menu_items (id, title, link, access_by, access_level) values (62, '', 'shapers.php', 5, 0);
insert into menu_items (id, title, link, access_by, access_level) values (63, '', 'modems.php', 8, 0);
insert into menu_items (id, title, link, access_by, access_level) values (64, ' ', 'modems_struct.php', 8, 0);
insert into menu_items (id, title, link, access_by, access_level) values (65, '', 'modem_profiles.php', 9, 0);
insert into menu_items (id, title, link, access_by, access_level) values (66, ' ()', 'modems_actions.php', 16, 0);
insert into menu_items (id, title, link, access_by, access_level) values (67, 'vlans', 'vlans.php', 54, 0);
insert into menu_items (id, title, link, access_by, access_level) values (68, ' ', 'modems_models.php', 8, 0);
insert into menu_items (id, title, link, access_by, access_level) values (69, '', 'departments.php', 17, 0);
insert into menu_items (id, title, link, access_by, access_level) values (70, ' ', 'system_accounts.php', 29, 0);
insert into menu_items (id, title, link, access_by, access_level) values (71, ' ', 'resource_types.php', 32, 0);
insert into menu_items (id, title, link, access_by, access_level) values (72, '', 'operators.php', 10, 0);
insert into menu_items (id, title, link, access_by, access_level) values (73, '', 'groups.php', 10, 0);
insert into menu_items (id, title, link, access_by, access_level) values (74, '', 'action_log.php', 28, 0);
insert into menu_items (id, title, link, access_by, access_level) values (75, '', 'port_services.php', 15, 0);
insert into menu_items (id, title, link, access_by, access_level) values (76, 'mac vendors', 'mac_vendors.php', 15, 0);
insert into menu_items (id, title, link, access_by, access_level) values (77, '', 'config.php', 15, 0);
insert into menu_items (id, title, link, access_by, access_level) values (78, ' ', 'lists.php', 15, 0);
insert into menu_items (id, title, link, access_by, access_level) values (79, '', 'stats_all.php', 4, 0);
insert into menu_items (id, title, link, access_by, access_level) values (80, ': ', 'stats_streets.php', 2, 0);
insert into menu_items (id, title, link, access_by, access_level) values (81, ': ', 'stats_departments.php', 17, 0);
insert into menu_items (id, title, link, access_by, access_level) values (82, ': ', 'stats_clients.php', 1, 0);
insert into menu_items (id, title, link, access_by, access_level) values (83, ':  ', 'stats_connection_types.php', 5, 0);
insert into menu_items (id, title, link, access_by, access_level) values (84, ': arpu', 'stats_arpu.php', 5, 0);
insert into menu_items (id, title, link, access_by, access_level) values (85, '', 'snapshots.php', 1, 0);
insert into menu_items (id, title, link, access_by, access_level) values (86, ': ', 'stats_modems.php', 8, 0);
insert into menu_items (id, title, link, access_by, access_level) values (87, ': ', 'stats_hosts.php', 4, 0);
insert into menu_items (id, title, link, access_by, access_level) values (88, ': ', 'stats_rules.php', 7, 0);
insert into menu_items (id, title, link, access_by, access_level) values (89, ': ', 'stats_resources.php', 5, 0);
insert into menu_items (id, title, link, access_by, access_level) values (90, '', 'logoff.php', null, 0);
insert into menu_items (id, title, link, access_by, access_level) values (91, '', 'personal.php', null, 0);
insert into menu_items (id, title, link, access_by, access_level) values (92, '', 'menu_items.php', null, 0);
insert into menu_items (id, title, link, access_by, access_level) values (93, 'eCall', 'ecalls.php', 58, 0);
insert into menu_items (id, title, link, access_by, access_level) values (94, ' ', 'delayed_tasks.php', 15, 0);
insert into menu_items (id, title, link, access_by, access_level) values (95, '', 'networks.php', 15, 0);
insert into menu_items (id, title, link, access_by, access_level) values (96, '', 'domains.php', 15, 0);
insert into menu_items (id, title, link, access_by, access_level) values (97, '', 'protocols.php', 15, 0);
	
insert into menu_operator_boxes (id, title, order_num) values (1, '', 10);
insert into menu_operator_boxes (id, title, order_num) values (2, '', 20);
insert into menu_operator_boxes (id, title, order_num) values (3, '', 30);
insert into menu_operator_boxes (id, title, order_num) values (4, '', 40);
insert into menu_operator_boxes (id, title, order_num) values (5, ' ', 50);
insert into menu_operator_boxes (id, title, order_num) values (6, '', 60);
insert into menu_operator_boxes (id, title, order_num) values (7, '', 70);
insert into menu_operator_boxes (id, title, order_num) values (8, '', 80);
insert into menu_operator_boxes (id, title, order_num) values (9, '', 90);
insert into menu_operator_boxes (id, title, order_num) values (10, '', 100);
insert into menu_operator_boxes (id, title, order_num) values (11, '', 110);

insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (1, 1, 10, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (2, 1, 20, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (3, 1, 30, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (4, 1, 40, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (5, 1, 50, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (6, 1, 60, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (7, 1, 70, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (8, 1, 80, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (9, 1, 90, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (10, 1, 100, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (11, 1, 110, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (12, 1, 120, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (13, 1, 130, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (14, 1, 160, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (15, 1, 140, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (16, 1, 150, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (17, 1, 170, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (18, 1, 180, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (19, 1, 190, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (20, 1, 200, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (21, 1, 210, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (22, 1, 220, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (23, 6, 10, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (24, 6, 20, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (25, 6, 30, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (26, 6, 40, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (27, 6, 50, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (28, 6, 60, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (29, 2, 10, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (30, 2, 20, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (31, 2, 30, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (32, 2, 40, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (33, 2, 50, '');
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (34, 3, 10, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (35, 3, 20, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (36, 3, 30, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (37, 3, 40, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (38, 3, 50, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (39, 3, 60, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (40, 3, 70, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (41, 4, 10, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (42, 4, 20, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (43, 4, 30, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (44, 4, 40, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (45, 4, 50, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (46, 4, 60, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (47, 4, 70, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (48, 4, 80, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (49, 5, 10, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (50, 5, 20, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (51, 5, 30, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (52, 5, 40, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (53, 5, 50, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (54, 5, 60, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (55, 5, 70, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (56, 5, 80, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (57, 7, 10, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (58, 7, 20, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (59, 7, 30, '');
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (60, 7, 50, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (61, 7, 60, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (62, 7, 70, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (63, 8, 10, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (64, 8, 20, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (65, 8, 30, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (66, 8, 40, '');
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (67, 8, 50, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (68, 8, 60, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (69, 9, 10, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (70, 9, 20, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (71, 9, 30, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (72, 9, 40, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (73, 9, 50, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (74, 9, 60, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (75, 9, 120, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (76, 9, 130, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (77, 9, 70, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (78, 9, 140, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (79, 10, 10, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (80, 10, 20, '');
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (81, 10, 30, '');
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (82, 10, 40, '');
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (83, 10, 50, ' ');
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (84, 10, 60, 'arpu');
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (85, 10, 70, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (86, 10, 80, '');
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (87, 10, 90, '');
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (88, 10, 100, '');
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (89, 10, 110, '');
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (90, 11, 30, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (91, 11, 20, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (92, 11, 10, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (93, 2, 60, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (94, 9, 80, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (95, 9, 90, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (96, 9, 100, null);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (97, 9, 110, null);

ALTER TABLE modems ADD COLUMN managment_vlan int AFTER exp_point;

ALTER TABLE connection_types ADD COLUMN replace_by INT UNSIGNED, ADD FOREIGN KEY (replace_by) REFERENCES connection_types (id) ON DELETE SET NULL;

ALTER TABLE staff ADD COLUMN biom_id int AFTER stamp;

CREATE TABLE `biometric_auth_events` (
  `id` int(11) NOT NULL auto_increment,
  `user_id` int(11) default NULL,
  `auth_method` int(11) default NULL,
  `auth_time` datetime default NULL,
  `person` int(11) unsigned default NULL,
  PRIMARY KEY  (`id`),
  KEY `biometric_auth_events_ibfk_1` (`user_id`),
  KEY `person` (`person`),
  CONSTRAINT `person` FOREIGN KEY (`person`) REFERENCES `staff` (`id`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=cp1251;

alter table exp_points add unique(inv_number);

set names cp1251;
insert into system_entities (id, key_name, name) values (63, "biometrica", " ");
insert into menu_items (id, title, link, access_by, access_level) values (98, ' ', 'biometric_events.php', 63, 0);
insert into menu_operator_items (menu_item, menu_box, order_num, custom_title) values (98, 10, 14, null);

CREATE TABLE hosts_restricted_names (
	id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;
ALTER TABLE hosts_restricted_names ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

INSERT INTO `system_entities` (`id`,`key_name`,`name`) VALUES (64,'hosts_restricted_names',' ');
INSERT INTO `menu_items` (`id`,`title`,`link`,`access_by`,`access_level`) VALUES (99,' ','hosts_restricted_names.php','64','0');
INSERT INTO `menu_operator_items` (`id`,`menu_item`,`menu_box`,`custom_title`) VALUES (8491,'99','7',' ');

CREATE TABLE notifications (
	id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL,
	message TEXT,
	section INT,
	rank INT,
	show_begin DATETIME,
	show_end DATETIME,
	stamp BIGINT UNSIGNED UNIQUE,
	KEY(show_begin),
	KEY(show_end)
) ENGINE=InnoDB;
ALTER TABLE notifications 
	ADD FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	ADD FOREIGN KEY (section) REFERENCES ranks (id) ON DELETE SET NULL;
INSERT INTO `system_entities` (`id`,`key_name`,`name`) VALUES (65,'notifications','');
INSERT INTO `menu_items` (`id`,`title`,`link`,`access_by`) VALUES (101,'','notifications.php','65');
INSERT INTO `menu_operator_items` (`id`,`menu_item`,`menu_box`,`custom_title`,`order_num`) VALUES (NULL,'101','4','',200);

ALTER TABLE modems ADD COLUMN zabbix_id int AFTER managment_vlan;

ALTER TABLE notifications ADD COLUMN show_mode TINYINT NOT NULL;

ALTER TABLE jobs ADD COLUMN connection_type INT UNSIGNED,
	ADD FOREIGN KEY fk_con_type (connection_type) REFERENCES connection_types (id) ON DELETE SET NULL;

CREATE TABLE exp_movements (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	date DATETIME,
	src INT UNSIGNED,
	dst INT UNSIGNED,
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	KEY(date),
	KEY(src),
	KEY(dst)
) ENGINE=InnoDB;
ALTER TABLE exp_movements 
	ADD CONSTRAINT fk_exp_movements_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	ADD CONSTRAINT fk_exp_movements_src FOREIGN KEY (src) REFERENCES exp_storage_places (id),
	ADD CONSTRAINT fk_exp_movements_dst FOREIGN KEY (dst) REFERENCES exp_storage_places (id);
	
CREATE TABLE exp_movements_details (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	movement INT UNSIGNED NOT NULL,
	material INT UNSIGNED NOT NULL,
	amount BIGINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	KEY(material),
	KEY(movement)
) ENGINE=InnoDB;
ALTER TABLE exp_movements_details 
	ADD CONSTRAINT fk_exp_movements_details_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	ADD CONSTRAINT fk_exp_movements_details_movement FOREIGN KEY (movement) REFERENCES exp_movements (id) ON DELETE CASCADE,
	ADD CONSTRAINT fk_exp_movements_details_material FOREIGN KEY (material) REFERENCES exp_materials (id);

CREATE TABLE jobs_extra_types (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	charge_type INT UNSIGNED NOT NULL,
	worker_fee BIGINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;
ALTER TABLE jobs_extra_types
	ADD CONSTRAINT fk_jobs_extra_types_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	ADD CONSTRAINT fk_jobs_extra_types_charge_type FOREIGN KEY (charge_type) REFERENCES charge_types (id);
	
CREATE TABLE jobs_extras (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	job INT UNSIGNED NOT NULL,
	type INT UNSIGNED NOT NULL,
	amount INT NOT NULL,
	notes VARCHAR(255) NOT NULL,
	resource_use INT UNSIGNED,
	stamp BIGINT UNSIGNED UNIQUE,
	KEY (job),
	KEY (type)
) ENGINE=InnoDB;
ALTER TABLE jobs_extras
	ADD CONSTRAINT fk_jobs_extras_stamp  FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	ADD CONSTRAINT fk_jobs_extras_resource_use  FOREIGN KEY (resource_use) REFERENCES resource_uses (id) ON DELETE SET NULL,
	ADD CONSTRAINT fk_jobs_extras_type  FOREIGN KEY (type) REFERENCES jobs_extra_types (id);

ALTER TABLE jobs ADD COLUMN worker_bonus BIGINT NOT NULL;

INSERT INTO `system_entities` (`id`,`key_name`,`name`) VALUES (66,'jobs_extra_types','  ');
INSERT INTO `menu_items` (`id`,`title`,`link`,`access_by`,`access_level`) VALUES (102,' . ','jobs_extra_types.php','66','0');
INSERT INTO `menu_operator_items` (`id`,`menu_item`,`menu_box`,`order_num`) VALUES (NULL,'102','4','90');

INSERT INTO `menu_items` (`id`,`title`,`link`,`access_by`) VALUES (103,' ','exp_moyements.php','51');
INSERT INTO `menu_operator_items` (`id`,`menu_item`,`menu_box`,`order_num`) VALUES (NULL,'103','5','90');

CREATE TABLE entrances (
	house INT UNSIGNED NOT NULL,
	number INT NOT NULL,
	type TINYINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	PRIMARY KEY(house, number),
	KEY(house)
) ENGINE=InnoDB;
ALTER TABLE entrances
	ADD CONSTRAINT fk_entrances_stamp  FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	ADD CONSTRAINT fk_entrances_house  FOREIGN KEY (house) REFERENCES houses (id) ON DELETE CASCADE;
	
UPDATE menu_items SET link='exp_movements.php' WHERE link='exp_moyements.php';

ALTER TABLE config ADD column list VARCHAR(255) AFTER control;
UPDATE config SET control=4, list='getChargeTypes' WHERE id='DRWEB_CHARGE_TYPE';
UPDATE config SET control=4, list='getSystemDstAccounts' WHERE id='DRWEB_ACCOUNT';
UPDATE config SET control=4, list='getResourceTypes' WHERE id='DRWEB_RESOURCE_TYPE';

SET NAMES cp1251;
INSERT IGNORE INTO resource_types(name) VALUES ('');
SELECT id INTO @a FROM resource_types WHERE name='';
insert into config (id, value, control, list, actual) values ('CONNECTION_RESOURCE_TYPE', @a, 4, 'getResourceTypes', 1);

SELECT account INTO @a FROM system_accounts WHERE name='';
insert into config (id, value, control, list, actual) values ('CONNECTION_ACCOUNT', @a, 4, 'getSystemDstAccounts', 1);

INSERT INTO `system_entities` (`id`,`key_name`,`name`) VALUES (67,'salary',' ');
INSERT INTO `menu_items` (`id`,`title`,`link`,`access_by`) VALUES (104,' ','salary.php','67');
INSERT INTO `menu_operator_items` (`id`,`menu_item`,`menu_box`,`order_num`,`custom_title`) VALUES (NULL, '104','4','210',' ');

ALTER TABLE connection_types ADD COLUMN first_charge_amount BIGINT NOT NULL;
ALTER TABLE connection_types ADD COLUMN first_free_amount BIGINT NOT NULL;
ALTER TABLE connection_types ADD COLUMN first_free_limit BIGINT NOT NULL;

ALTER TABLE entrances DROP PRIMARY KEY, ADD COLUMN id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY FIRST, ADD UNIQUE (house, number);

ALTER TABLE modems_states CHANGE COLUMN loss_in loss_in INT, CHANGE COLUMN loss_in_prev loss_in_prev INT, CHANGE COLUMN loss_out loss_out INT, CHANGE COLUMN loss_out_prev loss_out_prev INT;

ALTER TABLE modems ADD COLUMN auto_update ENUM("0", "1") AFTER zabbix_id;

ALTER TABLE modems_models ADD COLUMN file text AFTER notes;

ALTER TABLE jobs ADD COLUMN cost BIGINT NOT NULL;
ALTER TABLE jobs ADD COLUMN entrance_type TINYINT NOT NULL;

UPDATE jobs j LEFT JOIN houses h ON h.street=j.street AND h.house=j.house LEFT JOIN entrances e ON e.house=h.id AND e.number=j.entrance 
SET j.cost=IF(j.type=2, 0, IF(h.district=7, 2500, IF(j.type=4, 5000, IF(e.type=1, 5000, 9500)))), j.entrance_type=e.type;

DELETE FROM `menu_operator_items` WHERE (`id`='51');
DELETE FROM `menu_items` WHERE (`id`='51');

ALTER TABLE exp_storage_places ADD column notes text AFTER stamp;
ALTER TABLE exp_storage_places ADD column account enum("0","1") AFTER notes;

ALTER TABLE exp_tasks ADD COLUMN type TINYINT NOT NULL;

ALTER TABLE exp_tasks 
	ADD COLUMN point INT UNSIGNED,
	ADD KEY(point),
	ADD CONSTRAINT fk_exp_tasks_point  FOREIGN KEY (point) REFERENCES exp_points (id) ON DELETE SET NULL;
	
ALTER TABLE exp_tasks 
	ADD COLUMN trunk INT UNSIGNED,
	ADD KEY(trunk),
	ADD CONSTRAINT fk_exp_tasks_trunk  FOREIGN KEY (trunk) REFERENCES exp_trunks (id) ON DELETE SET NULL;
	
ALTER TABLE exp_tasks ADD COLUMN crash_date DATETIME;

CREATE TABLE staff_tools (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	person INT UNSIGNED NOT NULL,
	tool INT UNSIGNED NOT NULL,
	amount BIGINT NOT NULL,
	issue_date DATETIME,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;
ALTER TABLE staff_tools
	ADD CONSTRAINT fk_staff_tools_stamp  FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	ADD CONSTRAINT fk_staff_tools_tool FOREIGN KEY (tool) REFERENCES exp_tools (id),
	ADD CONSTRAINT fk_staff_tools_person FOREIGN KEY (person) REFERENCES staff (id) ON DELETE CASCADE;

CREATE TABLE coil_supplies (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	date DATETIME NOT NULL,
	code VARCHAR(64) UNIQUE,
	tech_zone INT UNSIGNED,
	cable_type TINYINT NOT NULL,
	amount INT NOT NULL,
	length BIGINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;
ALTER TABLE coil_supplies
	ADD CONSTRAINT fk_coil_supplies_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	ADD CONSTRAINT fk_coil_supplies_tech_zone FOREIGN KEY (tech_zone) REFERENCES tech_zones (id) ON DELETE SET NULL;

CREATE TABLE coil_items (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	supply INT UNSIGNED,
	code VARCHAR(32),
	rest BIGINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	UNIQUE (supply, code)
) ENGINE=InnoDB;
ALTER TABLE coil_items
	ADD CONSTRAINT fk_coil_items_stamp  FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	ADD CONSTRAINT fk_coil_items_supply  FOREIGN KEY (supply) REFERENCES coil_supplies (id) ON DELETE CASCADE;

CREATE TABLE coil_uses (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	item INT UNSIGNED,
	person INT UNSIGNED,
	begin_date DATETIME,
	begin_length BIGINT NOT NULL,
	end_date DATETIME,
	end_length BIGINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	KEY (person),
	KEY (item)
) ENGINE=InnoDB;
ALTER TABLE coil_uses
	ADD CONSTRAINT fk_coil_uses_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	ADD CONSTRAINT fk_coil_uses_item FOREIGN KEY (item) REFERENCES coil_items (id) ON DELETE SET NULL,
	ADD CONSTRAINT fk_coil_uses_person FOREIGN KEY (person) REFERENCES staff (id) ON DELETE SET NULL;

INSERT INTO `system_entities` (`id`,`key_name`,`name`) VALUES ('68','coil_supplies',' ');
INSERT INTO `menu_items` (`id`,`title`,`link`,`access_by`) VALUES (105,' ','coil_supplies.php','68');
INSERT INTO `menu_operator_items` (`id`,`menu_item`,`menu_box`,`order_num`,`custom_title`) VALUES (NULL, '105','4','220',' ');

INSERT INTO `system_entities` (`id`,`key_name`,`name`) VALUES ('69','coil_uses',' ');
INSERT INTO `menu_items` (`id`,`title`,`link`,`access_by`) VALUES (106,' ','coil_uses.php','69');
INSERT INTO `menu_operator_items` (`id`,`menu_item`,`menu_box`,`order_num`,`custom_title`) VALUES (NULL, '105','4','220',' ');

delimiter //
CREATE PROCEDURE complete_jobs ()
BEGIN
UPDATE jobs j 
INNER JOIN clients c ON c.id=j.client_id
SET j.state=1
WHERE j.date < now() AND (j.state=0 OR j.state=2) AND (j.type=1 OR j.type=4) AND c.connection_type!=19 AND (
SELECT count(h.id) c FROM hosts h WHERE h.client=j.client_id) > 0;
END//
delimiter ;

ALTER TABLE connection_types ADD COLUMN igmp_profile INT NOT NULL, ADD COLUMN igmp_max_groups INT NOT NULL;
UPDATE connection_types SET igmp_profile=1, igmp_max_groups=1;

CREATE TABLE ip_use_days (
	ip_use INT UNSIGNED NOT NULL,
	day DATE NOT NULL,
	bytes BIGINT UNSIGNED NOT NULL,
	PRIMARY KEY (ip_use, day),
	KEY (day)
) ENGINE=InnoDB;
ALTER TABLE ip_use_days 
	ADD CONSTRAINT fk_ip_use_days_ip_use FOREIGN KEY (ip_use) REFERENCES ip_uses (id) ON DELETE CASCADE;

CREATE TABLE ip_use_hours (
	ip_use INT UNSIGNED NOT NULL,
	hour TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
	bytes BIGINT UNSIGNED NOT NULL,
	PRIMARY KEY (ip_use, hour),
	KEY (hour)
) ENGINE=InnoDB;
ALTER TABLE ip_use_hours 
	ADD CONSTRAINT fk_ip_use_hours_ip_use FOREIGN KEY (ip_use) REFERENCES ip_uses (id) ON DELETE CASCADE;

CREATE TABLE ip_use_time (
	ip_use INT UNSIGNED NOT NULL,
	from_time TIMESTAMP NOT NULL DEFAULT 0,
	to_time TIMESTAMP NOT NULL DEFAULT 0,
	bytes BIGINT UNSIGNED NOT NULL,
	PRIMARY KEY (ip_use, from_time),
	KEY (from_time),
	KEY (to_time)
) ENGINE=InnoDB;
ALTER TABLE ip_use_time 
	ADD CONSTRAINT fk_ip_use_time_ip_use FOREIGN KEY (ip_use) REFERENCES ip_uses (id) ON DELETE CASCADE;

INSERT INTO `system_entities` (`id`,`key_name`,`name`) VALUES (70,'clientssupport','()');
INSERT INTO `menu_items` (`id`,`title`,`link`,`access_by`) VALUES (107,'()','clients2.php?r=1','70');
INSERT INTO `menu_operator_items` (`id`,`menu_item`,`menu_box`,`order_num`,`custom_title`) VALUES (NULL, '107','1','15','()');

INSERT INTO `system_entities` (`id`,`key_name`,`name`) VALUES (71,'con_statistic',' ');
INSERT INTO `menu_items` (`id`,`title`,`link`,`access_by`) VALUES (108,' ','con_statistic.php','71');
INSERT INTO `menu_operator_items` (`id`,`menu_item`,`menu_box`,`order_num`,`custom_title`) VALUES (NULL, '108','10','12',' ');

alter table ip_use_time drop primary key, drop key from_time, add primary key(from_time, ip_use), add key(ip_use);
alter table ip_use_days drop primary key, drop key day, add primary key (day, ip_use), add key(ip_use);
alter table ip_use_hours drop primary key, drop key hour, add primary key (hour, ip_use), add key(ip_use);

DELETE FROM ip_use_days;
INSERT INTO ip_use_days (ip_use, day, bytes) 
SELECT l.ip_use, CAST(ii.from_time as DATE), sum(l.bytes)
FROM ip_use_log l 
LEFT JOIN ip_use_intervals ii ON ii.id=l.ip_use_interval
GROUP BY 1,2;

DELETE FROM ip_use_hours;
INSERT INTO ip_use_hours (ip_use, hour, bytes) 
SELECT l.ip_use, from_unixtime(floor(unix_timestamp(ii.from_time)/3600)*3600), sum(l.bytes)
FROM ip_use_log l 
LEFT JOIN ip_use_intervals ii ON ii.id=l.ip_use_interval
WHERE ii.from_time > SUBDATE(NOW(), INTERVAL 1 MONTH)
GROUP BY 1,2;

DELETE FROM ip_use_time;
INSERT INTO ip_use_time (ip_use, from_time, to_time, bytes)
SELECT l.ip_use, ii.from_time, ii.to_time, sum(l.bytes)
FROM ip_use_log l 
LEFT JOIN ip_use_intervals ii ON ii.id=l.ip_use_interval
WHERE ii.from_time > SUBDATE(NOW(), INTERVAL 1 DAY)
GROUP BY 1,2;

INSERT INTO `system_entities` (`id`,`key_name`,`name`) VALUES (72,'polls','');
INSERT INTO `menu_items` (`id`,`title`,`link`,`access_by`) VALUES (109,'','polls.php','72');
INSERT INTO `menu_operator_items` (`id`,`menu_item`,`menu_box`,`order_num`,`custom_title`) VALUES (NULL, '109','1','102','');

ALTER TABLE connection_types
	ADD COLUMN credit_con_type INT UNSIGNED,
	ADD KEY(credit_con_type),
	ADD CONSTRAINT fk_connection_types_credit_con_type FOREIGN KEY (credit_con_type) REFERENCES connection_types (id) ON DELETE SET NULL;
	
alter table staff add column place tinyint after biom_id;

INSERT INTO `system_entities` (`id`,`key_name`,`name`) VALUES (73,'blocks','');


INSERT INTO `system_entities` (`id`,`key_name`,`name`) VALUES (74,'notes','');


INSERT INTO `system_entities` (`id`,`key_name`,`name`) VALUES (75,'calls','');


INSERT INTO `system_entities` (`id`,`key_name`,`name`) VALUES (76,'client_shaper_patches',' ');


UPDATE menu_items SET title='asterisk' WHERE link='stats_asterisk.php';

CREATE TABLE asterisk_schedule_plans (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	manager_begin TIME NOT NULL,
	manager_end TIME NOT NULL,
	tech_begin TIME NOT NULL,
	tech_end TIME NOT NULL,
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;
ALTER TABLE asterisk_schedule_plans
	ADD CONSTRAINT fk_asterisk_schedule_plans_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

CREATE TABLE asterisk_schedule_labels (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;
ALTER TABLE asterisk_schedule_labels
	ADD CONSTRAINT fk_asterisk_schedule_labels_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

CREATE TABLE asterisk_schedule (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	day DATE NOT NULL UNIQUE,
	schedule_plan INT UNSIGNED,
	manager_begin TIME NOT NULL,
	manager_end TIME NOT NULL,
	tech_begin TIME NOT NULL,
	tech_end TIME NOT NULL,
	day_type TINYINT NOT NULL,
	label INT UNSIGNED,
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	KEY(schedule_plan),
	KEY(label)
) ENGINE=InnoDB;
ALTER TABLE asterisk_schedule
	ADD CONSTRAINT fk_asterisk_schedule_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	ADD CONSTRAINT fk_asterisk_schedule_plan FOREIGN KEY (schedule_plan) REFERENCES asterisk_schedule_plans (id) ON DELETE SET NULL,
	ADD CONSTRAINT fk_asterisk_schedule_label FOREIGN KEY (label) REFERENCES asterisk_schedule_labels (id) ON DELETE SET NULL;


-- Table "houses_foto" DDL

CREATE TABLE `houses_foto` (
  `id` int(11) NOT NULL auto_increment,
  `house` int(10) unsigned default NULL,
  `foto` int(10) unsigned default NULL,
  `stamp` bigint(20) unsigned default NULL,
  PRIMARY KEY  (`id`),
  KEY `house` (`house`),
  KEY `houses_foto_ibfk_1` (`stamp`),
  CONSTRAINT `houses_foto_ibfk_1` FOREIGN KEY (`stamp`) REFERENCES `stamps` (`id`) ON DELETE SET NULL,
  CONSTRAINT `house` FOREIGN KEY (`house`) REFERENCES `houses` (`id`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=cp1251;



-- Table "modems_foto" DDL

CREATE TABLE `modems_foto` (
  `id` int(11) NOT NULL auto_increment,
  `modem` int(10) unsigned default NULL,
  `foto` int(10) unsigned default NULL,
  `stamp` bigint(20) unsigned default NULL,
  PRIMARY KEY  (`id`),
  KEY `modem` (`modem`),
  KEY `houses_foto_ibfk_1` (`stamp`),
  CONSTRAINT `modems_foto_ibfk_1` FOREIGN KEY (`stamp`) REFERENCES `stamps` (`id`) ON DELETE SET NULL,
  CONSTRAINT `modem` FOREIGN KEY (`modem`) REFERENCES `modems` (`id`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=cp1251;

-- Table "modems_foto" DDL

CREATE TABLE `staff_foto` (
  `id` int(11) NOT NULL auto_increment,
  `staff` int(10) unsigned default NULL,
  `foto` int(10) unsigned default NULL,
  `stamp` bigint(20) unsigned default NULL,
  PRIMARY KEY  (`id`),
  KEY `staff` (`staff`),
  KEY `staff_foto_ibfk_1` (`stamp`),
  CONSTRAINT `staff_foto_ibfk_1` FOREIGN KEY (`stamp`) REFERENCES `stamps` (`id`) ON DELETE SET NULL,
  CONSTRAINT `staff` FOREIGN KEY (`staff`) REFERENCES `staff` (`id`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=cp1251;

ALTER TABLE houses ADD COLUMN house_state INT AFTER dispatching;

INSERT INTO `system_entities` (`id`,`key_name`,`name`) VALUES (77,'my_coil',' ');
INSERT INTO `menu_items` (`id`,`title`,`link`,`access_by`) VALUES (110,' ','my_coil.php','77');
INSERT INTO `menu_operator_items` (`id`,`menu_item`,`menu_box`,`order_num`,`custom_title`) VALUES (NULL, '110','4','22',' ');

alter table exp_movements add column state tinyint not null AFTER stamp;

insert into config (id, value, control, list, actual) values ('PLATEZHKA_ACCOUNT', 0, 4, 'getSystemSrcAccounts', 1);
insert into config (id, value, control, actual) values ('PLATEZHKA_THEIRS_PUBLIC', '', 0, 1);
insert into config (id, value, control, actual) values ('PAYGATE_OUR_PRIVATE', '', 0, 1);
insert into config (id, value, control, actual) values ('PLATEZHKA_ALLOW', '', 0, 1);

drop table if exists gate_pays;
CREATE TABLE gate_pays (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	token VARCHAR(64) NOT NULL,
	client INT UNSIGNED,
	movement BIGINT UNSIGNED,
	order_id VARCHAR(64) NOT NULL,
	amount BIGINT UNSIGNED,
	date datetime not null,
	KEY movement (movement),
	UNIQUE (token, order_id)
) ENGINE=InnoDB;
ALTER TABLE gate_pays ADD FOREIGN KEY (movement) REFERENCES movements (id) ON DELETE SET NULL;
ALTER TABLE gate_pays ADD FOREIGN KEY (client) REFERENCES clients (id) ON DELETE SET NULL;

ALTER TABLE staff ADD COLUMN department TINYINT NOT NULL;

set names cp1251;
INSERT INTO `system_entities` (`id`,`key_name`,`name`) VALUES (78,'gatepays','');
INSERT INTO `menu_items` (`id`,`title`,`link`,`access_by`) VALUES (111,'','gatepays.php','78');
INSERT INTO `menu_operator_items` (`id`,`menu_item`,`menu_box`,`order_num`,`custom_title`) VALUES (NULL, '111','1','175','');


ALTER TABLE asterisk_schedule_objects ADD COLUMN code VARCHAR(64) UNIQUE AFTER name;


DELIMITER //
CREATE PROCEDURE asterisk_check_schedule (IN objectCode VARCHAR(255), IN scheduleTime DATETIME) 
BEGIN
SELECT IF(count(*) > 0, 1, 0) online
FROM asterisk_schedule_details d 
LEFT JOIN asterisk_schedule s ON s.id=d.schedule 
LEFT JOIN asterisk_schedule_objects o ON o.id=d.object 
WHERE s.day=CAST(scheduleTime as DATE) AND o.code=objectCode AND CAST(scheduleTime as TIME) BETWEEN d.time_begin AND d.time_end;
END //
DELIMITER ;

CREATE TABLE tech_zone_job_types (
	tech_zone INT UNSIGNED NOT NULL,
	type TINYINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	PRIMARY KEY(tech_zone, type),
	KEY (type)
) ENGINE=InnoDB;
ALTER TABLE tech_zone_job_types
	ADD CONSTRAINT fk_tech_zone_job_types_zone FOREIGN KEY  (tech_zone) REFERENCES tech_zones (id) ON DELETE CASCADE,
	ADD CONSTRAINT fk_tech_zone_job_types_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

ALTER TABLE tech_zone_districts DROP KEY district_2;


CREATE VIEW tech_zones_view AS
SELECT t.tech_zone, t.id tzt, d.district, j.type job_type
FROM tech_zone_times t
LEFT JOIN tech_zone_districts d ON t.tech_zone=d.tech_zone
LEFT JOIN tech_zone_job_types j ON j.tech_zone=t.tech_zone;

alter table tech_zone_times add column from_time_ TIME NOT NULL, add column to_time_ TIME NOT NULL;
UPDATE tech_zone_times SET from_time_ = SEC_TO_TIME(from_time * 60 * 60), to_time_ = SEC_TO_TIME(to_time * 60 * 60);
ALTER TABLE tech_zone_times CHANGE column from_time from_time TIME NOT NULL, CHANGE column to_time to_time TIME NOT NULL;
UPDATE tech_zone_times SET from_time = from_time_, to_time = to_time_;
ALTER TABLE tech_zone_times DROP COLUMN from_time_, DROP COLUMN to_time_;

INSERT INTO `menu_items` (`id`,`title`,`link`,`access_by`) VALUES (112,'','gateways.php','15');
INSERT INTO `menu_operator_items` (`id`,`menu_item`,`menu_box`,`order_num`,`custom_title`) VALUES (NULL, '112','8','70','');

CREATE TABLE gateways (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	address VARCHAR(255) NOT NULL,
	gate VARCHAR(255) NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;
ALTER TABLE gateways
	ADD CONSTRAINT fk_gateways_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

CREATE TABLE coil_uses_details (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	coil_use INT UNSIGNED,
	expense INT NOT NULL,
	cable INT NOT NULL,
	cable_in INT NOT NULL,
	KEY (coil_use),
	KEY (expense),
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;
ALTER TABLE coil_uses_details
	ADD CONSTRAINT fk_coil_uses_details_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	ADD CONSTRAINT fk_coil_uses_details_coil_use FOREIGN KEY (coil_use) REFERENCES coil_uses (id) ON DELETE SET NULL,
	ADD CONSTRAINT fk_coil_uses_details_expense FOREIGN KEY (expense) REFERENCES client_expenses (id) ON DELETE CASCADE;

CREATE TABLE expense_material_types (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL,
	job_type TINYINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	UNIQUE (job_type, name)
) ENGINE=InnoDB;
ALTER TABLE expense_material_types
	ADD CONSTRAINT fk_expense_materials_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

CREATE TABLE expense_material_types (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL,
	job_type TINYINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	UNIQUE (job_type, name)
) ENGINE=InnoDB;
ALTER TABLE expense_material_types
	ADD CONSTRAINT fk_expense_materials_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

CREATE TABLE client_expenses_details (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	expense INT NOT NULL,
	material INT UNSIGNED NOT NULL,
	amount INT NOT NULL,
	KEY (expense),
	KEY (material),
	UNIQUE(expense, material),
	stamp BIGINT UNSIGNED UNIQUE
) ENGINE=InnoDB;
ALTER TABLE client_expenses_details
	ADD CONSTRAINT fk_client_expenses_details_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	ADD CONSTRAINT fk_client_expenses_details_expense FOREIGN KEY (expense) REFERENCES client_expenses (id) ON DELETE CASCADE,
	ADD CONSTRAINT fk_client_expenses_details_material FOREIGN KEY (material) REFERENCES expense_material_types (id);

SET names cp1251;
INSERT INTO expense_material_types (id, name) VALUES (1,  '');
INSERT INTO expense_material_types (id, name) VALUES (2,  '');
INSERT INTO expense_material_types (id, name) VALUES (3,  '');
INSERT INTO expense_material_types (id, name) VALUES (4,  'F-');
INSERT INTO expense_material_types (id, name) VALUES (5,  'TV-');
INSERT INTO expense_material_types (id, name) VALUES (6,  'F-');
INSERT INTO expense_material_types (id, name) VALUES (7,  'F- - F-');
INSERT INTO expense_material_types (id, name) VALUES (8,  ' - 2');
INSERT INTO expense_material_types (id, name) VALUES (9,  ' - 3');
INSERT INTO expense_material_types (id, name) VALUES (10, ' - 4');

INSERT INTO client_expenses_details (expense, material, amount)
SELECT id, 1, connectors FROM client_expenses WHERE connectors>0
UNION
SELECT id, 2, ties FROM client_expenses WHERE ties>0
UNION
SELECT id, 3, clips FROM client_expenses WHERE clips>0;

CREATE TABLE asterisk_trunks (
	id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(64) NOT NULL,
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	key(name)
) ENGINE=InnoDB;
ALTER TABLE asterisk_trunks
	ADD CONSTRAINT fk_asterisk_trunks_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

SET NAMES cp1251;
INSERT INTO asterisk_trunks (id, name) VALUES
(4, 'BeeLine'),
(1, 'Life'),
(7, 'Life'),
(2, ''),
(5, ''),
(3, 'KyivStar'),
(6, 'KyivStar'),
(12, 'Golden'),
(11, ''),
(100, ''),
(13, 'FAX'),
(14, ''),
(15, '.'),
(16, 'IVR');
	
INSERT INTO `system_entities` (`id`,`key_name`,`name`) VALUES (79,'expense_material_types','  ');
INSERT INTO `menu_items` (`id`,`title`,`link`,`access_by`) VALUES (113,'  ','expense_material_types.php','79');
INSERT INTO `menu_operator_items` (`id`,`menu_item`,`menu_box`,`order_num`,`custom_title`) VALUES (NULL, '113','4','230','  ');

alter table gateways add column count_ip int after stamp;

alter table houses add column internet tinyint after house_state;
alter table houses add column link_up_internet date after internet;
alter table houses add column tv tinyint after link_up_internet;
alter table houses add column link_up_tv date after tv;

UPDATE `houses` SET `internet`='1' WHERE (`state`='1');
UPDATE `houses` SET `internet`='2' WHERE (`state`='2');

alter table modems_models change column file file_firmware text;
alter table modems_models add column file_loader text after file_firmware;
alter table modems_models add column file_diag text after file_loader;

CREATE TABLE expense_material_job_types (
	material_type INT UNSIGNED NOT NULL,
	job_type TINYINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	PRIMARY KEY (material_type, job_type)
) ENGINE=InnoDB;
ALTER TABLE expense_material_job_types
	ADD CONSTRAINT fk_expense_material_job_types_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

ALTER TABLE client_expenses 
	ADD COLUMN job INT UNSIGNED, 
	ADD KEY(job), 
	ADD CONSTRAINT fk_client_expenses_job FOREIGN KEY (job) REFERENCES jobs (id) ON DELETE SET NULL; 
	
alter table client_expenses drop FOREIGN KEY client_expenses_ibfk_1;
alter table client_expenses change column client client int unsigned;
alter table client_expenses add CONSTRAINT fk_client_expenses_client FOREIGN KEY (client)  REFERENCES `clients` (`id`) ON DELETE set null;

alter table coil_items add column tech_zone int unsigned, add column state tinyint not null, add column cable_type tinyint not null after supply, add column length bigint not null after cable_type;
alter table coil_items add CONSTRAINT fk_coil_items_tech_zone FOREIGN KEY (tech_zone) REFERENCES tech_zones (id) ON DELETE SET NULL;
alter table coil_supplies change column code code_from varchar(64) not null;

-- Table "penalty_and_bonus" DDL

CREATE TABLE `penalty_and_bonus` (
  `id` int(11) NOT NULL auto_increment,
  `oper` int(10) unsigned default NULL,
  `type` tinyint(4) default NULL,
  `amount` int(11) default NULL,
  `date` date default NULL,
  `notes` text,
  `stamp` bigint(10) unsigned default NULL,
  PRIMARY KEY  (`id`),
  KEY `fk_stamp` (`stamp`),
  KEY `fk_oper` (`oper`),
  CONSTRAINT `fk_oper` FOREIGN KEY (`oper`) REFERENCES `staff` (`id`) ON DELETE SET NULL,
  CONSTRAINT `fk_stamp` FOREIGN KEY (`stamp`) REFERENCES `stamps` (`id`) ON DELETE SET NULL
) ENGINE=InnoDB AUTO_INCREMENT=3 DEFAULT CHARSET=cp1251;

INSERT INTO `system_entities` (`id`,`key_name`,`name`) VALUES (80,'penalty_and_bonus','  ');
INSERT INTO `menu_items` (`id`,`title`,`link`,`access_by`) VALUES (114,'  ','penalty_and_bonus.php','80');
INSERT INTO `menu_operator_items` (`id`,`menu_item`,`menu_box`,`order_num`,`custom_title`) VALUES (NULL, '114','4','215','  ');

UPDATE `menu_items` SET `title`='' WHERE (`id`='56');
UPDATE `system_entities` SET `name`=': ' WHERE (`id`='49');

ALTER TABLE action_log ADD COLUMN item_id BIGINT AFTER object, ADD KEY(item_id);
ALTER TABLE action_log DROP KEY object, ADD KEY (object, item_id);

alter table coil_items add key (supply);
alter table coil_items drop key supply;
alter table coil_items add UNIQUE (code);

ALTER TABLE staff ADD COLUMN asterisk_agent VARCHAR(64) UNIQUE;

CREATE TABLE job_types (
	id INT NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(64) NOT NULL UNIQUE,
	notes VARCHAR(255),
	actual TINYINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	key(name)
) ENGINE=InnoDB;
ALTER TABLE job_types
	ADD CONSTRAINT fk_job_types_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

SET NAMES CP1251;
INSERT INTO job_types (id, name, actual) VALUES
	(1, ' Internet', 1),
	(2, '  Internet', 1),
	(3, ' ', 1),
	(4, '  Internet', 1),
	(5, ' TV', 1),
	(6, '  TV', 1);
	
INSERT INTO `system_entities` (`id`,`key_name`,`name`) VALUES (81,'jobs_types',' ');
INSERT INTO `menu_items` (`id`,`title`,`link`,`access_by`) VALUES (115,' ','jobs_types.php','81');
INSERT INTO `menu_operator_items` (`id`,`menu_item`,`menu_box`,`order_num`,`custom_title`) VALUES (NULL, '115','4','240',' ');

INSERT INTO config (id, value, control, actual) VALUES ('PRIVATE_MER_ID', '1', 1, 1);
SET NAMES CP1251;
INSERT INTO accounts (type, name) VALUES ('S', ' ');
INSERT INTO system_accounts (account, code, name, type) VALUES (LAST_INSERT_ID(), 'PAY_PRIVATE', ' ', 1);

INSERT INTO config (id, value, control, actual) VALUES ('PRIVATE_PASSWORD', '1', 1, 1);

CREATE TABLE private_pays (
	id BIGINT NOT NULL AUTO_INCREMENT PRIMARY KEY,
	client INT UNSIGNED,
	date DATETIME NOT NULL,
	amount BIGINT UNSIGNED NOT NULL,
	order_id VARCHAR(255) NOT NULL UNIQUE,
	movement BIGINT UNSIGNED,
	eci TINYINT NOT NULL,
	onus TINYINT NOT NULL,
	response_code INT NOT NULL,
	reason_code INT NOT NULL,
	reason_code_desc VARCHAR(255) NOT NULL,
	reference_no VARCHAR(255) NOT NULL,
	padded_card_no VARCHAR(16) NOT NULL,
	auth_code VARCHAR(32) NOT NULL,
	KEY(client),
	KEY(movement),
	KEY(date)
)type=InnoDB;
alter table private_pays add CONSTRAINT fk_private_pays_client FOREIGN KEY (client)  REFERENCES clients (id) ON DELETE set null;
alter table private_pays add CONSTRAINT fk_private_pays_movement FOREIGN KEY (movement)  REFERENCES movements (id) ON DELETE set null;

SET NAMES CP1251;
INSERT INTO system_entities (id, key_name, name) VALUES (82,'private_pays',' 24');
INSERT INTO menu_items (id, title, link, access_by) VALUES (116,' 24','private_pays.php','82');
INSERT INTO menu_operator_items (menu_item, menu_box, order_num) VALUES ('116','1','177');

alter table private_pays add key(response_code);

SET NAMES CP1251;
insert into system_entities (id, key_name, name) values (83, 'workers_photo', ': ');
alter table coil_items drop FOREIGN KEY fk_coil_items_supply;
alter table coil_items add CONSTRAINT `fk_coil_items_supply` FOREIGN KEY (`supply`) REFERENCES `coil_supplies` (`id`) ON DELETE SET NULL;

ALTER TABLE job_types ADD COLUMN type TINYINT NOT NULL AFTER name;

alter table coil_items drop FOREIGN KEY fk_coil_items_supply;
alter table coil_items add CONSTRAINT `fk_coil_items_supply` FOREIGN KEY (`supply`) REFERENCES `coil_supplies` (`id`) ON DELETE CASCADE;

ALTER TABLE streets ADD COLUMN name_alt VARCHAR(32) UNIQUE AFTER name;
UPDATE streets SET name_alt=name; 

alter table jobs_extra_types add column salary_group VARCHAR(255) NOT NULL;
alter table jobs_extra_types add column salary_title VARCHAR(255) NOT NULL;
SET NAMES cp1251;
UPDATE system_entities SET name=": asterisk" WHERE id=56;

alter table houses add column coords VARCHAR(255) NOT NULL;

SET NAMES CP1251;
INSERT INTO system_entities (id, key_name, name) VALUES (84,'netstruct',' ');
INSERT INTO operators_access (operator, entity, access)
SELECT operator, 84, access FROM operators_access WHERE entity=8;

INSERT INTO groups_access (group_id, entity, access)
SELECT group_id, 84, access FROM groups_access WHERE entity=8;

UPDATE menu_items SET access_by=84 WHERE id=64;

CREATE TABLE osmp_pays (
	id BIGINT NOT NULL AUTO_INCREMENT PRIMARY KEY,
	client INT UNSIGNED,
	date DATETIME NOT NULL,
	amount BIGINT UNSIGNED NOT NULL,
	order_id VARCHAR(255) NOT NULL UNIQUE,
	movement BIGINT UNSIGNED,
	KEY(client),
	KEY(movement),
	KEY(date)
)type=InnoDB;
alter table osmp_pays add CONSTRAINT fk_osmp_pays_client FOREIGN KEY (client)  REFERENCES clients (id) ON DELETE set null;
alter table osmp_pays add CONSTRAINT fk_osmp_pays_movement FOREIGN KEY (movement)  REFERENCES movements (id) ON DELETE set null;

SET NAMES CP1251;
INSERT INTO accounts (type, name) VALUES ('S', ' ');
INSERT INTO system_accounts (account, code, name, type) VALUES (LAST_INSERT_ID(), 'PAY_OSMP', ' ', 1);
INSERT INTO system_entities (id, key_name, name) VALUES (85,'osmp_pays',' ');
INSERT INTO menu_items (id, title, link, access_by) VALUES (117, '', 'osmp_pays.php', '85');
INSERT INTO menu_operator_items (menu_item, menu_box, order_num) VALUES ('117','1','178');

alter table exp_tasks drop key name, add key(name);
 
alter table client_expenses drop column ties, drop column clips, drop column connectors;
alter table client_expenses add column no_materials TINYINT NOT NULL;

ALTER TABLE troubles ADD COLUMN replies INT NOT NULL;

CREATE TRIGGER trouble_answers_insert AFTER INSERT ON trouble_answers FOR EACH ROW
UPDATE troubles SET replies=(SELECT count(*) FROM trouble_answers WHERE trouble=NEW.trouble) WHERE id=NEW.trouble;

CREATE TRIGGER trouble_answers_delete AFTER DELETE ON trouble_answers FOR EACH ROW
UPDATE troubles SET replies=(SELECT count(*) FROM trouble_answers WHERE trouble=OLD.trouble) WHERE id=OLD.trouble;

UPDATE troubles t 
LEFT JOIN (SELECT trouble, count(id) cnt FROM trouble_answers GROUP BY trouble) x ON x.trouble=t.id
SET t.replies=IFNULL(x.cnt,0);

CREATE TABLE mail_messages_replies (
	message INT UNSIGNED NOT NULL PRIMARY KEY,
	replies INT NOT NULL
)type=InnoDB;
ALTER TABLE mail_messages_replies ADD CONSTRAINT fk_mail_messages_replies FOREIGN KEY(message) REFERENCES mail_messages (id) ON DELETE CASCADE;

delimiter //

CREATE TRIGGER mail_messages_insert AFTER INSERT ON mail_messages 
FOR EACH ROW
BEGIN
IF NEW.in_reply_to > '' THEN
SET @m = (SELECT id FROM mail_messages WHERE msg_id=NEW.in_reply_to);
IF @m > 0 THEN
REPLACE INTO mail_messages_replies (message, replies)
VALUES (@m, (SELECT count(*) FROM mail_messages WHERE in_reply_to=NEW.in_reply_to));
END IF;
END IF;
END;//

CREATE TRIGGER mail_messages_update AFTER UPDATE ON mail_messages 
FOR EACH ROW
BEGIN
IF OLD.in_reply_to > '' THEN
SET @m = (SELECT id FROM mail_messages WHERE msg_id=OLD.in_reply_to);
IF @m > 0 THEN
REPLACE INTO mail_messages_replies (message, replies)
VALUES (@m, (SELECT count(*) FROM mail_messages WHERE in_reply_to=OLD.in_reply_to));
END IF;
END IF;
IF NEW.in_reply_to > '' THEN
SET @m = (SELECT id FROM mail_messages WHERE msg_id=NEW.in_reply_to);
IF @m > 0 THEN
REPLACE INTO mail_messages_replies (message, replies)
VALUES (@m, (SELECT count(*) FROM mail_messages WHERE in_reply_to=NEW.in_reply_to));
END IF;
END IF;
END;//

CREATE TRIGGER mail_messages_delete AFTER DELETE ON mail_messages 
FOR EACH ROW
BEGIN
IF OLD.in_reply_to > '' THEN
SET @m = (SELECT id FROM mail_messages WHERE msg_id=OLD.in_reply_to);
IF @m > 0 THEN
REPLACE INTO mail_messages_replies (message, replies)
VALUES (@m, (SELECT count(*) FROM mail_messages WHERE in_reply_to=OLD.in_reply_to));
END IF;
END IF;
END;//

delimiter ;	

REPLACE INTO mail_messages_replies (message, replies)
SELECT t.id, count(t.id) 
FROM mail_messages m
LEFT JOIN mail_messages t ON m.in_reply_to=t.msg_id
WHERE t.id IS NOT NULL AND t.msg_id > ''
GROUP BY t.id;

CREATE TABLE asterisk_call_info (
	id BIGINT UNSIGNED NOT NULL PRIMARY KEY,
	notes VARCHAR(255),
	rating INT NOT NULL DEFAULT 0,
	stamp BIGINT UNSIGNED UNIQUE
)type=InnoDB;
ALTER TABLE asterisk_call_info
	ADD CONSTRAINT fk_asterisk_call_info_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

update staff set sip_id=null where sip_id='';
alter table staff add unique (sip_id);
ALTER TABLE staff CHANGE COLUMN sip_id sip_id VARCHAR(32);

CREATE TABLE cdr (
	id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	billid INT UNSIGNED NOT NULL,
	clid VARCHAR(10) NOT NULL,
	src VARCHAR(25) NOT NULL,
	caller VARCHAR(25) NOT NULL,
	callie VARCHAR(25) NOT NULL,
	intrunk INT NOT NULL,
	outtrunk INT NOT NULL,
	dst VARCHAR(25) NOT NULL,
	dcontext VARCHAR(25) NOT NULL,
	channel VARCHAR(50) NOT NULL,
	dstchannel VARCHAR(50) NOT NULL,
	lastapp VARCHAR(25) NOT NULL,
	lastdata VARCHAR(75) NOT NULL,
	start DATETIME NOT NULL,
	answer DATETIME NOT NULL,
	end DATETIME NOT NULL,
	duration SMALLINT UNSIGNED NOT NULL,
	billsec SMALLINT UNSIGNED NOT NULL,
	disposition VARCHAR(10) NOT NULL,
	amaflags TINYINT UNSIGNED NOT NULL,
	accountcode VARCHAR(10) NOT NULL,
	flags TINYINT UNSIGNED NOT NULL,
	uniqueid VARCHAR(25) NOT NULL,
	userfield VARCHAR(50) NOT NULL,
	status TINYINT UNSIGNED NOT NULL,
	reason TINYINT UNSIGNED NOT NULL,
	record CHAR(80) NOT NULL,
	notes VARCHAR(255),
	rating INT NOT NULL DEFAULT 0,
	stamp BIGINT UNSIGNED UNIQUE,
	KEY billid (billid),
	KEY accountcode (accountcode, start),
	KEY uniqueid (uniqueid),
	KEY start (start)
) ENGINE=InnoDB;
ALTER TABLE cdr
	ADD CONSTRAINT fk_cdr_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

INSERT INTO cdr (id,billid,clid,src,caller,callie,intrunk,outtrunk,dst,dcontext,channel,dstchannel,lastapp,lastdata,start,answer,
	end,duration,billsec,disposition,
	amaflags,accountcode,flags,uniqueid,userfield,status,reason,record)
SELECT id,billid,CONVERT(clid USING cp1251),CONVERT(src USING cp1251),CONVERT(caller USING cp1251),CONVERT(callie USING cp1251),
	intrunk,outtrunk,CONVERT(dst USING cp1251),CONVERT(dcontext USING cp1251),CONVERT(channel USING cp1251),
	CONVERT(dstchannel USING cp1251),CONVERT(lastapp USING cp1251),CONVERT(lastdata USING cp1251),start,answer,end,duration,billsec,
	CONVERT(disposition USING cp1251),
	amaflags,CONVERT(accountcode USING cp1251),flags,CONVERT(uniqueid USING cp1251),
	CONVERT(userfield USING cp1251),status,reason,
	CONVERT(record  USING cp1251)
FROM asterisk_test.cdr2 ORDER BY id;


ALTER TABLE system_accounts ADD column label VARCHAR(32);

ALTER TABLE departments ADD COLUMN hidden TINYINT NOT NULL;

CREATE TABLE houses_notes (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	house INT UNSIGNED NOT NULL,
	note VARCHAR(255) NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE
) type=InnoDB;
ALTER TABLE houses_notes
	ADD CONSTRAINT fk_houses_notes_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	ADD CONSTRAINT fk_houses_notes_house FOREIGN KEY (house) REFERENCES houses (id) ON DELETE CASCADE;

ALTER TABLE osmp_pays ADD COLUMN pay_type INT NOT NULL;

INSERT INTO config (id, value, control, actual) VALUES ('OSMP_ACCESS_ZONE', '213.186.115.164/27', 0, 1);

SET NAMES CP1251;
INSERT INTO system_entities (id, key_name, name) VALUES (86,'my_calls',':  ');
INSERT INTO menu_items (id, title, link, access_by) VALUES (118, ' ', 'my_calls.php', '86');
INSERT INTO menu_operator_items (menu_item, menu_box, order_num) VALUES ('118','2','70');

alter table access_tracker_queue add column task bigint;
alter table access_tracker_queue add key(task);
alter table access_tracker_queue add constraint fk_access_tracker_queue_task foreign key(task) references delayed_tasks(id) on delete set null;

SET NAMES CP1251;
INSERT INTO accounts (type, name) VALUES ('S', ' Aval');
INSERT INTO system_accounts (account, code, name, type, label) VALUES (LAST_INSERT_ID(), 'PAY_AVAL', ' Aval', 1, '  ');

update menu_items set access_by=3 where id=18;

CREATE TABLE tv_types (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(64) NOT NULL,
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	key(name)
) ENGINE=InnoDB;
ALTER TABLE tv_types
	ADD CONSTRAINT fk_tv_types_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL;

CREATE TABLE connection_type_tv (
  connection_type INT UNSIGNED NOT NULL,
  tv_type INT UNSIGNED NOT NULL,
  cost BIGINT NOT NULL,
  message VARCHAR(255),
  stamp BIGINT UNSIGNED UNIQUE,
  PRIMARY KEY  (connection_type, tv_type),
  KEY shaper (tv_type),
  CONSTRAINT fk_connection_type_tv_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
  CONSTRAINT fk_connection_type_tv_connection_type FOREIGN KEY (connection_type) REFERENCES connection_types (id) ON DELETE CASCADE,
  CONSTRAINT fk_connection_type_tv_tv_type FOREIGN KEY (tv_type) REFERENCES tv_types (id) ON DELETE CASCADE
) ENGINE=InnoDB;

ALTER TABLE clients 
	ADD COLUMN tv_type INT UNSIGNED AFTER connection_type,
	ADD KEY(tv_type),
	ADD CONSTRAINT fk_clients_tv_type FOREIGN KEY(tv_type) REFERENCES tv_types (id);
	
ALTER TABLE personal_periods 
	ADD COLUMN tv_type INT UNSIGNED AFTER connection_type,
	ADD KEY(tv_type),
	ADD CONSTRAINT fk_personal_periods_tv_type FOREIGN KEY(tv_type) REFERENCES tv_types (id) ON DELETE SET NULL;

	
SET NAMES cp1251;
INSERT IGNORE INTO resource_types(name) VALUES ('');
SELECT id INTO @a FROM resource_types WHERE name='';
insert into config (id, value, control, list, actual) values ('TV_RESOURCE_TYPE', @a, 4, 'getResourceTypes', 1);

INSERT IGNORE INTO accounts (type, name) VALUES ('S', ' TV');
SELECT id INTO @a FROM accounts WHERE name=' TV';
INSERT INTO system_accounts (account, name, type) VALUES (@a, ' TV', 2);

insert into config (id, value, control, list, actual) values ('TV_ACCOUNT', @a, 4, 'getSystemDstAccounts', 1);

ALTER TABLE ecalls
	ADD column tariff_tv VARCHAR(128),
	ADD COLUMN tv_type INT UNSIGNED,
	ADD KEY(tv_type),
	ADD CONSTRAINT fk_ecalls_tv_type FOREIGN KEY (tv_type) REFERENCES tv_types (id) ON DELETE SET NULL;
	
CREATE TABLE faxes (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	number VARCHAR(64) NOT NULL,
	client INT UNSIGNED,
	type TINYINT NOT NULL,
	status TINYINT NOT NULL,
	file INT UNSIGNED,
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	key(client),
	key(file)
) ENGINE=InnoDB;
ALTER TABLE faxes
	ADD CONSTRAINT fk_faxes_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	ADD CONSTRAINT fk_faxes_client FOREIGN KEY (client) REFERENCES clients (id) ON DELETE SET NULL;

SET NAMES CP1251;
INSERT INTO system_entities (id, key_name, name) VALUES (87,'fax','Fax');
INSERT INTO menu_items (id, title, link, access_by) VALUES (119, 'fax', 'faxes.php', '87');
INSERT INTO menu_operator_items (menu_item, menu_box, order_num) VALUES ('119','2','80');

ALTER TABLE entrances
	ADD COLUMN internet TINYINT NOT NULL,
	ADD COLUMN tv TINYINT NOT NULL;

UPDATE entrances e LEFT JOIN houses h ON h.id=e.house SET e.internet=1 WHERE h.internet=1;
UPDATE entrances e LEFT JOIN houses h ON h.id=e.house SET e.tv=1 WHERE h.tv=1 AND e.type=1;

alter table con_requests add column type tinyint default 1;

ALTER TABLE con_requests 
	ADD COLUMN client_id INT UNSIGNED, 
	ADD KEY(client_id), 
	ADD CONSTRAINT fk_con_requests_client FOREIGN KEY (client_id) REFERENCES clients (id) ON DELETE SET NULL;
	
ALTER TABLE houses
	ADD COLUMN kpreo1 INT UNSIGNED AFTER kpreo,
	ADD KEY(kpreo1),
	ADD CONSTRAINT fk_houses_kpreo1 FOREIGN KEY (kpreo1) REFERENCES kpreos (id) ON DELETE SET NULL,
	ADD COLUMN koop1 INT UNSIGNED AFTER koop,
	ADD KEY(koop1),
	ADD CONSTRAINT fk_houses_koop1 FOREIGN KEY (koop1) REFERENCES koops (id) ON DELETE SET NULL;
	
CREATE TABLE documents (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	file INT UNSIGNED,
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	KEY(file),
	CONSTRAINT fk_documents_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL
) ENGINE=InnoDB;

CREATE TABLE documents_access (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	document INT UNSIGNED NOT NULL,
	access_to TINYINT NOT NULL,
	group_id INT UNSIGNED,
	rank INT,
	section INT,
	person INT UNSIGNED,
	access_level TINYINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	UNIQUE(person, document),
	UNIQUE(group_id, document),
	UNIQUE(rank, document),
	UNIQUE(section, document),
	CONSTRAINT fk_documents_access_document FOREIGN KEY (document) REFERENCES documents (id) ON DELETE CASCADE,
	CONSTRAINT fk_documents_access_group_id FOREIGN KEY (group_id) REFERENCES groups (id) ON DELETE SET NULL,
	CONSTRAINT fk_documents_access_rank FOREIGN KEY (rank) REFERENCES ranks (id) ON DELETE SET NULL,
	CONSTRAINT fk_documents_access_person FOREIGN KEY (person) REFERENCES staff (id) ON DELETE SET NULL,
	CONSTRAINT fk_documents_access_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL
) ENGINE=InnoDB;

SET NAMES CP1251;
INSERT INTO system_entities (id, key_name, name) VALUES (88,'documents','');
INSERT INTO menu_items (id, title, link, access_by) VALUES (120, '', 'documents.php', '88');
INSERT INTO menu_operator_items (menu_item, menu_box, order_num) VALUES ('120','4','250');

REPLACE INTO menu_operator_items (operator, menu_item, menu_box, order_num)
SELECT o.id, 120, (SELECT b.id FROM menu_operator_boxes b WHERE b.operator=o.id AND b.title=''), 250
FROM operators o
WHERE o.group_id=6;

DROP PROCEDURE IF EXISTS `complete_jobs`;
delimiter //
CREATE PROCEDURE complete_jobs ()
BEGIN
-- complete internet
UPDATE jobs j 
INNER JOIN clients c ON c.id=j.client_id
SET j.state=1
WHERE j.date < now() AND (j.state IN (0,2,5)) AND (j.type=1 OR j.type=4) AND c.connection_type!=19 AND (
SELECT count(h.id) c FROM hosts h WHERE h.client=j.client_id) > 0;
-- complete tv
UPDATE jobs j 
INNER JOIN clients c ON c.id=j.client_id
SET j.state=1
WHERE j.date < now() AND (j.state IN (0,2,5)) AND (j.type=5) AND c.connection_type!=19 AND (
SELECT count(e.id) c FROM client_expenses e WHERE e.job=j.id) > 0;
-- complete repair
UPDATE jobs j 
SET j.state=1
WHERE j.date < now() AND (j.state IN (0,2,5)) AND (j.type=2 OR j.type=6);
-- to in-process
UPDATE jobs j 
SET j.state=5
WHERE j.date < now() AND (j.state IN (0,2));
END//
delimiter ;

ALTER TABLE jobs ADD COLUMN contract VARCHAR(16) UNIQUE;
-- ALTER TABLE jobs DROP FOREIGN KEY jobs_ibfk_6;
ALTER TABLE jobs DROP COLUMN worker_op;

ALTER TABLE connection_types ADD COLUMN replace_after INT;

SET NAMES CP1251;
INSERT INTO system_entities (id, key_name, name) VALUES (89,'gateways','');
UPDATE menu_items SET access_by=89 WHERE id=112;

SET NAMES CP1251;
update system_entities set name=": 24" WHERE id=82;
update system_entities set name=": " WHERE id=85;

SET NAMES CP1251;
INSERT INTO system_entities (id, key_name, name) VALUES (90,'portmone',': Portmone');
UPDATE menu_items SET access_by=90 WHERE id=18;

CREATE TABLE ipcad.ipcad_log (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT,
	src INT UNSIGNED NOT NULL,
	dst INT UNSIGNED NOT NULL,
	port SMALLINT UNSIGNED NOT NULL,
	dir TINYINT UNSIGNED NOT NULL,
	proto TINYINT UNSIGNED NOT NULL,
	bytes INT UNSIGNED NOT NULL,
	date TIMESTAMP NOT NULL,
	PRIMARY KEY(src, date, id),
	UNIQUE (id)
) ENGINE=InnoDB;

-- INSERT INTO ipcad.ipcad_log SELECT * FROM ipcad.ipcad_log_26;

CREATE TABLE force_offline (
	client INT UNSIGNED NOT NULL PRIMARY KEY,
	CONSTRAINT fk_force_offline_client FOREIGN KEY (client) REFERENCES clients (id) ON DELETE CASCADE
) ENGINE=InnoDB;

ALTER TABLE modems_states CHANGE COLUMN date date TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP;
ALTER TABLE modems_states 
ADD COLUMN prev BIGINT, 
ADD KEY (prev), 
ADD CONSTRAINT fk_modems_states_prev FOREIGN KEY (prev) REFERENCES modems_states (id) ON DELETE SET NULL;

ALTER TABLE modems ADD COLUMN off_all BIGINT NOT NULL, ADD COLUMN off_month BIGINT NOT NULL;

DELETE FROM mailboxes;
INSERT INTO mailboxes(id,login,pass,domain,type,boxsize,mailsize)
	SELECT m.id,m.login,m.password,t.domain,m.type,t.boxsize,t.mailsize
	FROM client_mailboxes m LEFT JOIN mailbox_types t ON t.id=m.type WHERE m.active
ALTER TABLE mailboxes add CONSTRAINT fk_mailboxes_id  FOREIGN KEY (id) REFERENCES client_mailboxes(id) ON DELETE CASCADE;

ALTER TABLE client_calls
	ADD COLUMN start_ticket tinyint NOT NULL,
	ADD COLUMN start_iptv tinyint NOT NULL,
	ADD COLUMN start_rating tinyint NOT NULL;
	
ALTER TABLE jobs DROP KEY contract, ADD key(contract);

ALTER TABLE blocks ADD COLUMN type TINYINT NOT NULL DEFAULT 1;

ALTER TABLE blocks ADD COLUMN prolonged TINYINT NOT NULL;

UPDATE blocks SET type=1;

CREATE TABLE modems_nest (
	id INT UNSIGNED NOT NULL PRIMARY KEY,
	parent INT UNSIGNED,
	level INT NOT NULL,
	left_key INT NOT NULL,
	right_key INT NOT NULL,
	KEY(left_key),
	KEY(right_key)
) type=InnoDB;

CREATE TABLE job_types_connection_types (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	job_type INT NOT NULL,
	connection_type INT UNSIGNED NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	UNIQUE(job_type, connection_type),
	KEY(connection_type),
	CONSTRAINT fk_job_types_connection_types_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	CONSTRAINT fk_job_types_connection_types_job_type FOREIGN KEY (job_type) REFERENCES job_types (id) ON DELETE CASCADE,
	CONSTRAINT fk_job_types_connection_types_connection_type FOREIGN KEY (connection_type) REFERENCES connection_types (id) ON DELETE CASCADE
) type=InnoDB;


CREATE TABLE block_schedule (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	host INT UNSIGNED NOT NULL,
	type TINYINT NOT NULL,
	date DATE,
	day_bits TINYINT NOT NULL,
	from_time TIME NOT NULL,
	to_time TIME NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	KEY(host),
	KEY(date),
	CONSTRAINT fk_block_schedule_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	CONSTRAINT fk_block_schedule_host FOREIGN KEY (host) REFERENCES hosts (id) ON DELETE CASCADE
) type=InnoDB;

set names cp1251;
insert into menu_operator_boxes (id, title, order_num) values (12, 'asterisk', 25);
UPDATE menu_operator_items SET menu_box=12, order_num=20 WHERE menu_item=33;
UPDATE menu_items SET title=' ' WHERE id=33;

INSERT INTO menu_items (id, title, link, access_by) VALUES (121, '', 'asterisk_calls.php', 56);
INSERT INTO menu_operator_items (menu_item, menu_box, order_num) VALUES (121, 12, 10);

INSERT INTO menu_items (id, title, link, access_by) VALUES (122, '', 'asterisk_trunks.php', 56);
INSERT INTO menu_operator_items (menu_item, menu_box, order_num) VALUES (122, 12, 30);

INSERT INTO menu_items (id, title, link, access_by) VALUES (123, '', 'asterisk_schedule.php', 56);
INSERT INTO menu_operator_items (menu_item, menu_box, order_num) VALUES (123, 12, 40);

CREATE TABLE asterisk_address_book (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	number VARCHAR(32) NOT NULL UNIQUE,
	type TINYINT NOT NULL,
	host INT UNSIGNED,
	person INT UNSIGNED,
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	KEY(host),
	KEY(person),
	CONSTRAINT fk_asterisk_address_book_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	CONSTRAINT fk_asterisk_address_book_host FOREIGN KEY (host) REFERENCES hosts (id) ON DELETE SET NULL,
	CONSTRAINT fk_asterisk_address_book_person FOREIGN KEY (person) REFERENCES staff (id) ON DELETE SET NULL
) type=InnoDB;

set names cp1251;
INSERT INTO menu_items (id, title, link, access_by) VALUES (124, ' ', 'asterisk_address_book.php', 56);
INSERT INTO menu_operator_items (menu_item, menu_box, order_num) VALUES (124, 12, 50);

ALTER TABLE clients 
	ADD column personal_period INT UNSIGNED,
	ADD KEY(personal_period),
	ADD CONSTRAINT fk_clients_personal_period FOREIGN KEY(personal_period) REFERENCES personal_periods(id) ON DELETE SET NULL;

DROP TRIGGER IF EXISTS personal_periods_insert;
delimiter //
CREATE TRIGGER personal_periods_insert AFTER INSERT ON personal_periods 
FOR EACH ROW
BEGIN
IF NEW.marker=1 THEN
UPDATE clients SET personal_period=NEW.id WHERE id=NEW.client;
END IF;
END;//
delimiter ;

DROP TRIGGER IF EXISTS personal_periods_update;
delimiter //
CREATE TRIGGER personal_periods_update AFTER UPDATE ON personal_periods 
FOR EACH ROW
BEGIN
IF NEW.marker=1 THEN
UPDATE clients SET personal_period=NEW.id WHERE id=NEW.client;
END IF;
END;//
delimiter ;

UPDATE clients SET personal_period = NULL;

UPDATE personal_periods pp 
LEFT JOIN clients c ON c.id=pp.client
SET c.personal_period=pp.id WHERE pp.marker=1;

CREATE TABLE clients_states (
	client INT UNSIGNED NOT NULL PRIMARY KEY,
	block TINYINT NOT NULL,
	firewall TINYINT NOT NULL,
	limits TINYINT NOT NULL,
	use_plan BIGINT NOT NULL,
	use_actual BIGINT NOT NULL,
	incomplete BIGINT NOT NULL,
	subscriptions BIGINT NOT NULL,
	credit BIGINT NOT NULL,
	CONSTRAINT fk_clients_states_client FOREIGN KEY(client) REFERENCES clients(id) ON DELETE CASCADE
) type=InnoDB;

delimiter //
CREATE TRIGGER clients_insert_states AFTER INSERT ON clients
FOR EACH ROW
BEGIN
INSERT IGNORE INTO clients_states (client) VALUES (NEW.id);
END;//
delimiter ;

INSERT IGNORE INTO clients_states (client) SELECT id FROM clients;

ALTER TABLE modems_actions ADD COLUMN hidden TINYINT NOT NULL;
ALTER TABLE hosts_actions ADD COLUMN hidden TINYINT NOT NULL;

ALTER TABLE modems 
	ADD COLUMN check_power TINYINT NOT NULL DEFAULT 1,
	ADD COLUMN vlan INT NOT NULL;
	
ALTER TABLE exp_movements_details ADD COLUMN src_rest BIGINT NOT NULL;
ALTER TABLE exp_movements_details ADD COLUMN dst_rest BIGINT NOT NULL;

ALTER TABLE queries ADD COLUMN is_filter TINYINT NOT NULL;

alter table menu_operator_boxes add CONSTRAINT fk_menu_operator_boxes_operator FOREIGN KEY (`operator`) REFERENCES `operators` (`id`) ON DELETE CASCADE;

ALTER TABLE ecalls ADD COLUMN src TINYINT NOT NULL DEFAULT 1;

CREATE TABLE _houses_back SELECT * FROM houses;

ALTER TABLE houses CHANGE COLUMN link_up_internet link_up_internet DATETIME;
ALTER TABLE houses CHANGE COLUMN link_up_tv link_up_tv DATETIME;

ALTER TABLE clients_states ADD COLUMN modem_state TINYINT;

INSERT INTO config (id, value, control, list, actual) values ('CLIENT_LOGIN_PATTERN', '|^[0-9a-z\-\.]*$|i', 1, null, 1);

CREATE TABLE cities (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(64) NOT NULL UNIQUE,
	stamp BIGINT UNSIGNED UNIQUE,
	CONSTRAINT fk_cities_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL
) type=InnoDB;

SET NAMES CP1251;
INSERT INTO system_entities (id, key_name, name) VALUES (91,'cities','');
INSERT INTO menu_items (id, title, link, access_by) VALUES (125, '', 'cities.php', '91');
INSERT INTO menu_operator_items (menu_item, menu_box, order_num) VALUES ('125','3','5');

INSERT INTO cities(id, name) VALUES (1, '');
INSERT INTO cities(id, name) VALUES (2, '');

ALTER TABLE streets 
	ADD column city INT UNSIGNED AFTER id,
	ADD KEY(city);
UPDATE streets SET city=1;	
ALTER TABLE streets ADD CONSTRAINT fk_streets_city FOREIGN KEY (city) REFERENCES cities (id);

ALTER TABLE admin_districts
	ADD column city INT UNSIGNED AFTER id,
	ADD KEY(city);
UPDATE admin_districts SET city=1;	
ALTER TABLE admin_districts ADD CONSTRAINT fk_admin_districts_city FOREIGN KEY (city) REFERENCES cities (id);

ALTER TABLE districts
	ADD column city INT UNSIGNED AFTER id,
	ADD KEY(city);
UPDATE districts SET city=1;	
ALTER TABLE districts ADD CONSTRAINT fk_districts_city FOREIGN KEY (city) REFERENCES cities (id);

ALTER TABLE kpreos
	ADD column city INT UNSIGNED AFTER id,
	ADD KEY(city);
UPDATE kpreos SET city=1;	
ALTER TABLE kpreos ADD CONSTRAINT fk_kpreos_city FOREIGN KEY (city) REFERENCES cities (id);
ALTER TABLE kpreos DROP KEY number, ADD UNIQUE(city, number);

ALTER TABLE koops
	ADD column city INT UNSIGNED AFTER id,
	ADD KEY(city);
UPDATE koops SET city=1;	
ALTER TABLE koops ADD CONSTRAINT fk_koops_city FOREIGN KEY (city) REFERENCES cities (id);

ALTER TABLE dispatching
	ADD column city INT UNSIGNED AFTER id,
	ADD KEY(city);
UPDATE dispatching SET city=1;	
ALTER TABLE dispatching ADD CONSTRAINT fk_dispatching_city FOREIGN KEY (city) REFERENCES cities (id);

ALTER TABLE houses
	ADD column city INT UNSIGNED  AFTER id,
	ADD KEY(city);
UPDATE houses SET city=1;
ALTER TABLE houses ADD CONSTRAINT fk_houses_city FOREIGN KEY (city) REFERENCES cities (id);

alter table houses drop key street_2, add unique(city, house, street);

ALTER TABLE exp_points
	ADD column city INT UNSIGNED AFTER id,
	ADD KEY(city);
UPDATE exp_points SET city=1;
ALTER TABLE exp_points ADD CONSTRAINT fk_exp_points_city FOREIGN KEY (city) REFERENCES cities (id);

ALTER TABLE exp_trunks
	ADD column city INT UNSIGNED AFTER id,
	ADD KEY(city);
UPDATE exp_trunks SET city=1;
ALTER TABLE exp_trunks ADD CONSTRAINT fk_exp_trunks_city FOREIGN KEY (city) REFERENCES cities (id);

ALTER TABLE modems
	ADD column city INT UNSIGNED AFTER id,
	ADD KEY(city);
UPDATE modems SET city=1;
ALTER TABLE modems ADD CONSTRAINT fk_modems_city FOREIGN KEY (city) REFERENCES cities (id);

alter table streets drop key name, add unique(city, name);
alter table streets drop key name_alt, add unique(city, name_alt);
alter table admin_districts drop key name, add unique(city, name);
alter table districts drop key name, add unique(city, name);
alter table koops drop key name, add unique(city, name);

ALTER TABLE clients
	ADD column city INT UNSIGNED AFTER id,
	ADD KEY(city);
UPDATE clients SET city=1;
ALTER TABLE clients ADD CONSTRAINT fk_clients_city FOREIGN KEY (city) REFERENCES cities (id);

ALTER TABLE jobs
	ADD column city INT UNSIGNED AFTER id,
	ADD KEY(city);
UPDATE jobs SET city=1;
ALTER TABLE jobs ADD CONSTRAINT fk_jobs_city FOREIGN KEY (city) REFERENCES cities (id);

ALTER TABLE entrances ADD column tie TINYINT NOT NULL

UPDATE entrances SET tie=IF(type=0, 3, 1);

CREATE TABLE operators_cities (
	operator INT UNSIGNED NOT NULL,
	city INT UNSIGNED NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	KEY (operator),
	KEY (city),
	CONSTRAINT fk_operators_cities_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	CONSTRAINT fk_operators_cities_operator FOREIGN KEY (operator) REFERENCES operators (id) ON DELETE CASCADE,
	CONSTRAINT fk_operators_cities_city FOREIGN KEY (city) REFERENCES cities (id) ON DELETE CASCADE
) ENGINE=InnoDB;

ALTER TABLE modem_profiles ADD COLUMN hidden TINYINT NOT NULL;

ALTER TABLE operators ADD COLUMN city_mode TINYINT NOT NULL;

ALTER TABLE staff
	ADD column city INT UNSIGNED AFTER id,
	ADD KEY(city);
UPDATE staff SET city=1;
ALTER TABLE staff ADD CONSTRAINT fk_staff_city FOREIGN KEY (city) REFERENCES cities (id);

ALTER TABLE movements
	ADD column city INT UNSIGNED AFTER id,
	ADD KEY(city);
UPDATE movements SET city=1;
ALTER TABLE movements ADD CONSTRAINT fk_movements_city FOREIGN KEY (city) REFERENCES cities (id) ON DELETE SET NULL;

DELIMITER //
CREATE FUNCTION get_account_city(x INT UNSIGNED) RETURNS INT UNSIGNED
BEGIN
DECLARE t CHAR(1);
DECLARE r INT UNSIGNED;
SET r=NULL;
SELECT type INTO t FROM accounts WHERE id=x;
IF t='D' THEN SELECT city INTO r FROM clients WHERE account=x; END IF;
IF t='B' THEN SELECT c.city INTO r FROM bonus_accounts ba LEFT JOIN clients c ON c.id=ba.client WHERE ba.account=x; END IF;
RETURN r;
END //
DELIMITER ;

DELIMITER //
CREATE TRIGGER movement_insert BEFORE INSERT ON movements
FOR EACH ROW
IF NEW.city IS NULL 
THEN SET NEW.city = IFNULL(get_account_city(NEW.src), get_account_city(NEW.dst));
END IF //
DELIMITER ;

DELIMITER //
CREATE TRIGGER movement_update BEFORE UPDATE ON movements
FOR EACH ROW
IF NEW.city IS NULL 
THEN SET NEW.city = IFNULL(get_account_city(NEW.src), get_account_city(NEW.dst));
END IF //
DELIMITER ;

ALTER TABLE houses ADD COLUMN lat DECIMAL(12,9), ADD COLUMN lng DECIMAL(12,9), ADD KEY (lat, lng); 
UPDATE houses SET lng=substring_index(coords, ',', 1),  lat=substring_index(coords, ',', -1);

CREATE TRIGGER house_insert BEFORE INSERT ON houses FOR EACH ROW
SET NEW.lng=substring_index(NEW.coords, ',', 1), NEW.lat=substring_index(NEW.coords, ',', -1);

CREATE TRIGGER house_update BEFORE UPDATE ON houses FOR EACH ROW
SET NEW.lng=substring_index(NEW.coords, ',', 1), NEW.lat=substring_index(NEW.coords, ',', -1);

ALTER TABLE jobs ADD column skip_graph TINYINT NOT NULL;

SET NAMES CP1251;
INSERT INTO system_entities (id, key_name, name) VALUES (92,'asterisk_ban',' ');
INSERT INTO menu_items (id, title, link, access_by) VALUES (126, ' ', 'asterisk_ban.php', '92');
INSERT INTO menu_operator_items (menu_item, menu_box, order_num) VALUES ('126','12','60');

UPDATE menu_items SET title='call-center' WHERE id=93;

DROP PROCEDURE IF EXISTS complete_ecalls;
delimiter //
CREATE PROCEDURE complete_ecalls()
BEGIN
DROP TABLE IF EXISTS _client_links;
CREATE TEMPORARY TABLE _client_links SELECT max(account) account, house_id, apartment FROM clients WHERE connection_type!=19 GROUP BY house_id, apartment;
DROP TABLE IF EXISTS _has_pay;
CREATE TEMPORARY TABLE _has_pay (account INT UNSIGNED NOT NULL PRIMARY KEY) SELECT DISTINCT dst as account FROM movements;

UPDATE _client_links x
INNER JOIN _has_pay p ON p.account=x.account
INNER JOIN ecalls e ON x.house_id = e.assumed_house AND x.apartment = e.apartment
SET e.state=5 WHERE e.state!=5 AND e.state!=4;

END//
delimiter ;

alter table delayed_tasks add column delay int not null;

ALTER TABLE jobs
	ADD COLUMN resource_use INT UNSIGNED,
	ADD KEY(resource_use),
	ADD CONSTRAINT fk_jobs_resource_use FOREIGN KEY (resource_use) REFERENCES resource_uses (id) ON DELETE SET NULL;
	
ALTER TABLE job_types
	ADD COLUMN charge_type INT UNSIGNED,
	ADD KEY(charge_type),
	ADD CONSTRAINT fk_job_types_charge_type FOREIGN KEY (charge_type) REFERENCES charge_types (id) ON DELETE SET NULL;
	
alter table modems_nest drop key left_key, add key left_key(left_key, right_key);

ALTER TABLE modems_nest 
	ADD COLUMN l3 INT UNSIGNED, 
	ADD KEY(l3),
	ADD CONSTRAINT fk_modems_nest FOREIGN KEY(l3) REFERENCES modems(id) ON DELETE SET NULL;
	
alter table notifications drop foreign key notifications_ibfk_2;
alter table notifications add CONSTRAINT fk_notifications_rank FOREIGN KEY(rank) REFERENCES ranks (id) ON DELETE SET NULL;

alter table mail_messages 
	add column client int unsigned, 
	add key(client), 
	add constraint fk_mail_messages_client foreign key(client) references clients(id) on delete set null;
	
ALTER TABLE job_types ADD COLUMN abbr VARCHAR(8) UNIQUE;
ALTER TABLE job_types ADD COLUMN number INT NOT NULL;

ALTER TABLE job_types
	ADD COLUMN fee_0 INT NOT NULL, 
	ADD COLUMN fee_1 INT NOT NULL;
	
ALTER TABLE job_types ADD COLUMN auto_complete INT NOT NULL;


DROP FUNCTION IF EXISTS `complete_job`;
delimiter //
CREATE FUNCTION complete_job (jid INT UNSIGNED) RETURNS INT
BEGIN
DECLARE ac INT;
DECLARE complete INT;
SET complete = 1;
SELECT auto_complete INTO ac FROM job_types WHERE id=(SELECT type FROM jobs WHERE id=jid);
-- check client
IF complete AND ac & 1  AND NOT EXISTS (SELECT id FROM jobs WHERE id=jid AND client_id IS NOT NULL) THEN SET complete = 0; END IF;
-- check tv_type
IF complete AND ac & 2  AND NOT EXISTS (SELECT id FROM clients WHERE id=(SELECT client_id FROM jobs WHERE id=jid) AND tv_type IS NOT NULL) THEN SET complete = 0; END IF;
-- check hosts
IF complete AND ac & 4  AND NOT EXISTS (SELECT id FROM hosts WHERE client=(SELECT id FROM clients WHERE id=(SELECT client_id FROM jobs WHERE id=jid)) LIMIT 1) THEN SET complete = 0; END IF;
-- check expenses
IF complete AND ac & 8  AND NOT EXISTS (SELECT id FROM client_expenses WHERE job=jid LIMIT 1) THEN SET complete = 0; END IF;
-- check address
IF complete AND ac & 16 AND NOT EXISTS (SELECT j.id FROM jobs j LEFT JOIN clients c ON c.id=j.client_id WHERE j.id=jid AND j.house_id=c.house_id AND j.apartment=c.apartment) THEN SET complete = 0; END IF;
-- set state
IF complete THEN UPDATE jobs SET state=1 WHERE id=jid; END IF;
RETURN complete;
END//
delimiter ;

DROP PROCEDURE IF EXISTS `complete_jobs2`;
delimiter //
CREATE PROCEDURE complete_jobs2 ()
BEGIN
-- complete
DROP TABLE IF EXISTS _to_complete;
CREATE TEMPORARY TABLE _to_complete SELECT id FROM jobs WHERE date < now() AND state IN (0,2,5);
SELECT complete_job(id) FROM _to_complete;
-- to in-process
UPDATE jobs j 
SET j.state=5
WHERE j.date < now() AND (j.state IN (0,2));
END//
delimiter ;

ALTER TABLE ecalls ADD COLUMN in_client INT UNSIGNED;
ALTER TABLE ecalls ADD COLUMN in_state TINYINT NOT NULL;

CREATE TABLE actions (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(64) NOT NULL UNIQUE,
	notes VARCHAR(255),
	link VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	CONSTRAINT fk_actions_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL
) type=InnoDB;

SET NAMES CP1251;
INSERT INTO system_entities (id, key_name, name) VALUES (93, 'actions','');
INSERT INTO menu_items (id, title, link, access_by) VALUES (127, '', 'actions.php', 93);
INSERT INTO menu_operator_items (menu_item, menu_box, order_num) VALUES (127, 1, 230);

ALTER TABLE connection_types
	ADD COLUMN action INT UNSIGNED,
	ADD KEY(action),
	ADD CONSTRAINT fk_connection_types_action FOREIGN KEY (action) REFERENCES actions (id) ON DELETE SET NULL;

alter table ads add column city int unsigned, add key(city), add constraint fk_ads_city foreign key(city) references cities(id) on delete set null;

alter table polls add column city int unsigned, add key(city), add constraint fk_polls_city foreign key(city) references cities(id) on delete set null;

alter table con_requests add column city int unsigned, add key(city), add constraint fk_con_requests_city foreign key(city) references cities(id) on delete set null;

alter table asterisk_address_book add column city int unsigned, add key(city), add constraint fk_asterisk_address_book_city foreign key(city) references cities(id) on delete set null;

alter table asterisk.permissions add column city int unsigned, add key(city);

alter table asterisk_schedule add column city int unsigned, add key(city), add constraint fk_asterisk_schedule_city foreign key(city) references cities(id) on delete cascade;

DROP PROCEDURE IF EXISTS `asterisk_check_schedule`;
DELIMITER //
CREATE PROCEDURE asterisk_check_schedule (IN objectCode VARCHAR(255), IN scheduleTime DATETIME) 
BEGIN
SELECT IF(count(*) > 0, 1, 0) online
FROM asterisk_schedule_details d 
LEFT JOIN asterisk_schedule s ON s.id=d.schedule 
LEFT JOIN asterisk_schedule_objects o ON o.id=d.object 
WHERE s.city=1 AND s.day=CAST(scheduleTime as DATE) AND o.code=objectCode AND CAST(scheduleTime as TIME) BETWEEN d.time_begin AND d.time_end;
END //
DELIMITER ;

DROP PROCEDURE IF EXISTS `asterisk_check_schedule2`;
DELIMITER //
CREATE PROCEDURE asterisk_check_schedule2 (IN city INT UNSIGNED, IN objectCode VARCHAR(255), IN scheduleTime DATETIME) 
BEGIN
SELECT IF(count(*) > 0, 1, 0) online
FROM asterisk_schedule_details d 
LEFT JOIN asterisk_schedule s ON s.id=d.schedule 
LEFT JOIN asterisk_schedule_objects o ON o.id=d.object 
WHERE s.city=city AND s.day=CAST(scheduleTime as DATE) AND o.code=objectCode AND CAST(scheduleTime as TIME) BETWEEN d.time_begin AND d.time_end;
END //
DELIMITER ;

CREATE TABLE exp_complaints (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	city INT UNSIGNED,
	house INT UNSIGNED,
	entrance INT,
	floor INT,
	apartment INT,
	name VARCHAR(64) NOT NULL,
	phone VARCHAR(64) NOT NULL,
	state TINYINT NOT NULL,
	executor INT UNSIGNED,
	reason VARCHAR(255),
	decision VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	KEY(city),
	KEY(house),
	KEY(executor),
	CONSTRAINT fk_exp_complaints_city FOREIGN KEY (city) REFERENCES cities (id) ON DELETE SET NULL,
	CONSTRAINT fk_exp_complaints_house FOREIGN KEY (house) REFERENCES houses (id) ON DELETE SET NULL,
	CONSTRAINT fk_exp_complaints_executor FOREIGN KEY (executor) REFERENCES exp_executors (id) ON DELETE SET NULL,
	CONSTRAINT fk_exp_complaints_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL
) type=InnoDB;

ALTER TABLE connection_types ADD COLUMN enter_cost INT NOT NULL;

INSERT INTO config (id, value, control, list, actual) values ('PRIVATE_MIN_PAY', '10', 1, null, 1);

SET NAMES CP1251;
INSERT INTO system_entities (id, key_name, name) VALUES (94, 'exp_complaints',': ');
INSERT INTO menu_items (id, title, link, access_by) VALUES (128, '', 'exp_complaints.php', 94);
INSERT INTO menu_operator_items (menu_item, menu_box, order_num) VALUES (128, 5, 100);

CREATE TABLE exp_reports (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	type TINYINT NOT NULL,
	date DATETIME,
	city INT UNSIGNED,
	house INT UNSIGNED,
	executor INT UNSIGNED,
	
	entrance INT,
	floor INT,
	number VARCHAR(64) UNIQUE,
	place VARCHAR(255),
	
	notes VARCHAR(255),
	cable_hor TINYINT NOT NULL,
	cable_ver TINYINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	KEY(city),
	KEY(house),
	KEY(executor),
	KEY(date),
	CONSTRAINT fk_exp_reports_city FOREIGN KEY (city) REFERENCES cities (id) ON DELETE SET NULL,
	CONSTRAINT fk_exp_reports_house FOREIGN KEY (house) REFERENCES houses (id) ON DELETE SET NULL,
	CONSTRAINT fk_exp_reports_executor FOREIGN KEY (executor) REFERENCES exp_executors (id) ON DELETE SET NULL,
	CONSTRAINT fk_exp_reports_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL
) ENGINE=InnoDB;

SET NAMES CP1251;
INSERT INTO system_entities (id, key_name, name) VALUES (95, 'exp_reports',': ');
INSERT INTO menu_items (id, title, link, access_by) VALUES (129, '', 'exp_reports.php', 95);
INSERT INTO menu_operator_items (menu_item, menu_box, order_num) VALUES (129, 5, 110);

CREATE TABLE exp_reports_entrances (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	report INT UNSIGNED NOT NULL,
	entrance INT UNSIGNED NOT NULL,
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	KEY(report),
	KEY(entrance),
	CONSTRAINT fk_exp_reports_entrances_stamp  FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	CONSTRAINT fk_exp_reports_entrances_report  FOREIGN KEY (report) REFERENCES exp_reports (id) ON DELETE CASCADE,
	CONSTRAINT fk_exp_reports_entrances_entrance  FOREIGN KEY (entrance) REFERENCES entrances (id)
) ENGINE=InnoDB;

CREATE TABLE account_types (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	priority TINYINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	CONSTRAINT fk_account_types_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL
) ENGINE=InnoDB; 

CREATE TABLE account_resource_types (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	account_type INT UNSIGNED NOT NULL,
	resource_type INT UNSIGNED NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	UNIQUE (account_type, resource_type),
	KEY (resource_type),
	CONSTRAINT fk_account_resource_types_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	CONSTRAINT fk_account_resource_types_account_type FOREIGN KEY (account_type) REFERENCES account_types (id) ON DELETE CASCADE,
	CONSTRAINT fk_account_resource_types_resource_type FOREIGN KEY (resource_type) REFERENCES resource_types (id) ON DELETE CASCADE
) ENGINE=InnoDB;


SET NAMES CP1251;
INSERT INTO system_entities (id, key_name, name) VALUES (96, 'account_types',' ');
INSERT INTO menu_items (id, title, link, access_by) VALUES (130, ' ', 'account_types.php', 96);
INSERT INTO menu_operator_items (menu_item, menu_box, order_num) VALUES (130, 9, 15);

CREATE TABLE connection_type_bonuses (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	connection_type INT UNSIGNED NOT NULL,
	account_type INT UNSIGNED NOT NULL,
	start_src INT UNSIGNED NOT NULL,
	start_amount BIGINT NOT NULL,
	end_dst INT UNSIGNED,
	end_amount BIGINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	UNIQUE (connection_type, account_type),
	KEY (account_type),
	CONSTRAINT fk_connection_type_bonuses_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	CONSTRAINT fk_connection_type_bonuses_account_type FOREIGN KEY (account_type) REFERENCES account_types (id) ON DELETE CASCADE,
	CONSTRAINT fk_connection_type_bonuses_connection_type FOREIGN KEY (connection_type) REFERENCES connection_types (id) ON DELETE CASCADE,
	CONSTRAINT fk_connection_type_bonuses_start_src FOREIGN KEY (start_src) REFERENCES accounts (id),
	CONSTRAINT fk_connection_type_bonuses_end_dst FOREIGN KEY (end_dst) REFERENCES accounts (id)
) ENGINE=InnoDB;

ALTER TABLE bonus_accounts
	ADD COLUMN account_type INT UNSIGNED AFTER account,
	ADD KEY(account_type),
	ADD CONSTRAINT fk_bonus_accounts_account_type FOREIGN KEY (account_type) REFERENCES account_types(id),
	ADD UNIQUE (client, account_type);

alter table bonus_accounts drop key client;

ALTER TABLE clients_states ADD COLUMN threshold BIGINT NOT NULL DEFAULT 0;

SET NAMES cp1251;
INSERT INTO account_types (name) VALUES (' ');

UPDATE bonus_accounts ba 
LEFT JOIN calendar_account_states cas ON cas.account=ba.account AND cas.calendar_period=(SELECT max(id) FROM calendar_periods)
SET ba.account_type=LAST_INSERT_ID()
WHERE (ba.account_type IS NULL) AND (ba.actual_to IS NULL OR ba.actual_to > now()) AND (cas.state_end > 0);

CREATE TABLE _bonuses_back
SELECT ba.* 
FROM bonus_accounts ba 
LEFT JOIN calendar_account_states cas ON cas.account=ba.account AND cas.calendar_period=(SELECT max(id) FROM calendar_periods)
WHERE (ba.account_type IS NULL) AND NOT( (ba.actual_to IS NULL OR ba.actual_to > now()) AND (cas.state_end > 0));

DELETE FROM bonus_accounts WHERE account IN (SELECT account FROM _bonuses_back);

SET NAMES cp1251;
UPDATE bonus_accounts ba 
LEFT JOIN clients c ON c.id=ba.client
LEFT JOIN account_types at ON at.id=ba.account_type
LEFT JOIN accounts a ON a.id=ba.account
SET a.name = CONCAT(c.last_name, " ", c.first_name, " (",c.login,") - ", at.name); 

ALTER TABLE account_types ADD COLUMN front_page TINYINT NOT NULL DEFAULT 0;
ALTER TABLE account_types ADD COLUMN description VARCHAR(255);

ALTER TABLE exp_reports ADD COLUMN exp_point INT UNSIGNED AFTER executor, ADD KEY(exp_point), ADD CONSTRAINT fk_exp_reports_exp_point FOREIGN KEY (exp_point) REFERENCES exp_points(id);
--     exp_reports ( )
ALTER TABLE exp_reports DROP COLUMN number, DROP COLUMN entrance, DROP COLUMN floor, DROP COLUMN place; 

ALTER TABLE exp_reports ADD COLUMN exp_trunk INT UNSIGNED AFTER exp_point, ADD KEY(exp_trunk), ADD CONSTRAINT fk_exp_reports_exp_trunk FOREIGN KEY (exp_trunk) REFERENCES exp_trunks(id);

ALTER TABLE exp_reports ADD COLUMN entrances VARCHAR(255);

alter table connection_types add column city int unsigned, add key(city), add constraint fk_connection_types_city foreign key(city) references cities(id) on delete set null;

alter table job_types add column city int unsigned, add key(city), add constraint fk_job_types_city foreign key(city) references cities(id) on delete set null;
alter table job_types drop key name, add unique (name, city);

alter table tech_zones add column city int unsigned, add key(city), add constraint fk_tech_zones_city foreign key(city) references cities(id) on delete set null;
alter table tech_zones drop key name, add unique (name, city);

alter table tech_zones_graph drop FOREIGN KEY tech_zones_graph_ibfk_1;
alter table tech_zones_graph drop column operator;

alter table tech_zones_graph_template add column city int unsigned, add key(city), add constraint fk_tech_zones_graph_template_city foreign key(city) references cities(id) on delete set null;
alter table tech_zones_graph add column city int unsigned, add key(city), add constraint fk_tech_zones_graph_city foreign key(city) references cities(id) on delete set null;

update tech_zones_graph_template t left join staff s on s.id=t.person set t.city=s.city;
update tech_zones_graph t left join staff s on s.id=t.person set t.city=s.city;

alter table coil_supplies add column city int unsigned, add key(city), add constraint fk_coil_supplies_city foreign key(city) references cities(id) on delete set null;
update coil_supplies s left join tech_zones z on z.id=s.tech_zone set s.city=z.city where z.city is not null;
update coil_supplies set city=1 where city is null;

alter table coil_items add column city int unsigned, add key(city), add constraint fk_coil_items_city foreign key(city) references cities(id) on delete set null;
update coil_items i left join coil_supplies s on s.id=i.supply set i.city=s.city where s.city is not null;
update coil_items set city=1 where city is null;

alter table coil_items add column used_by int unsigned, add key(used_by), add constraint fk_coil_items_used_by FOREIGN KEY (used_by) REFERENCES coil_uses (id) ON DELETE SET NULL;

alter table documents_access add column city int unsigned, add key(city), add constraint fk_documents_access_city foreign key(city) references cities(id) on delete set null;
update documents_access set city=1;

alter table documents_access drop key person, add key(person);
alter table documents_access drop key group_id, add key(group_id);
alter table documents_access drop key rank, add key(rank);
alter table documents_access drop key section, add key(section);

alter table expense_material_job_types 
	change column job_type job_type int not null,
	add key (job_type),
	add  constraint fk_expense_material_job_types_job_type foreign key (job_type) references job_types(id) on delete cascade;

delete from expense_material_job_types where material_type not in (select id from expense_material_types);
alter table expense_material_job_types 	add  constraint fk_expense_material_job_types_material_type foreign key (material_type) references expense_material_types(id) on delete cascade;

alter table job_types drop key abbr;
alter table job_types add unique (abbr, city);

CREATE TABLE exp_complaints_files (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	complaint INT UNSIGNED NOT NULL,
	file INT UNSIGNED NOT NULL,
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	key(complaint),
	key(file),
	CONSTRAINT fk_exp_complaints_files_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	CONSTRAINT fk_exp_complaints_files_complaints FOREIGN KEY (complaint) REFERENCES exp_complaints (id) ON DELETE CASCADE
) ENGINE=InnoDB;

CREATE TABLE dhcp_templates (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	subnet VARCHAR(32),
	mask VARCHAR(32),
	gate varchar(255),
	broadcast varchar(32),
	notes VARCHAR(255),
	domain INT UNSIGNED,
	stamp BIGINT UNSIGNED UNIQUE,
	key(domain),
	CONSTRAINT fk_dhcp_templates_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	CONSTRAINT fk_dhcp_templates_domain FOREIGN KEY (domain) REFERENCES domains (id)
) ENGINE=InnoDB;

ALTER TABLE entrances ADD column notes VARCHAR(255);

ALTER TABLE snapshots
ADD COLUMN int_clients INT NOT NULL,
ADD COLUMN int_clients_diff INT NOT NULL,
ADD COLUMN int_new INT NOT NULL,
ADD COLUMN int_return INT NOT NULL,
ADD COLUMN tv_clients INT NOT NULL,
ADD COLUMN tv_clients_diff INT NOT NULL,
ADD COLUMN tv_new INT NOT NULL,
ADD COLUMN tv_return INT NOT NULL,
ADD COLUMN both_clients INT NOT NULL,
ADD COLUMN both_clients_diff INT NOT NULL,
ADD COLUMN both_new INT NOT NULL,
ADD COLUMN both_return INT NOT NULL,
ADD COLUMN cover_tv INT NOT NULL,
ADD COLUMN cover_tv_diff INT NOT NULL,
ADD COLUMN cover_int INT NOT NULL,
ADD COLUMN cover_int_diff INT NOT NULL;

ALTER TABLE snapshots ADD COLUMN city INT UNSIGNED, ADD KEY(city), ADD CONSTRAINT fk_snapshots_city FOREIGN KEY(city) REFERENCES cities(id) ON DELETE SET NULL;

ALTER TABLE job_types ADD COLUMN abon_incremental TINYINT NOT NULL;

ALTER TABLE clients ADD COLUMN allow_ip_login TINYINT NOT NULL;

CREATE TABLE connection_type_boost (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	connection_type INT UNSIGNED NOT NULL,
	boost INT UNSIGNED NOT NULL,
	name VARCHAR(255) NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	UNIQUE (connection_type, boost),
	KEY (boost),
	CONSTRAINT fk_connection_type_boost_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	CONSTRAINT fk_connection_type_boost_connection_type FOREIGN KEY (connection_type) REFERENCES connection_types (id) ON DELETE CASCADE,
	CONSTRAINT fk_connection_type_boost_boost FOREIGN KEY (boost) REFERENCES connection_types (id) ON DELETE CASCADE
) ENGINE=InnoDB;

update trunk_modems set ip_core_route=NULL where ip_core_route='';
alter table trunk_modems add unique(ip_core_route);

SET names cp1251;
INSERT INTO connection_type_boost (connection_type, boost, name)
SELECT ct.id, x.id, ' ' FROM connection_types x  LEFT JOIN connection_types ct ON CONCAT(ct.name, ' ( 2)')=x.name WHERE ct.id IS NOT NULL;
INSERT INTO connection_type_boost (connection_type, boost, name)
SELECT ct.id, x.id, ' ' FROM connection_types x  LEFT JOIN connection_types ct ON CONCAT(ct.name, ' ( x2)')=x.name WHERE ct.id IS NOT NULL;
INSERT INTO connection_type_boost (connection_type, boost, name)
SELECT ct.id, x.id, ' ' FROM connection_types x  LEFT JOIN connection_types ct ON CONCAT(ct.name, ' ( 2)')=x.name WHERE ct.id IS NOT NULL;
INSERT INTO connection_type_boost (connection_type, boost, name)
SELECT ct.id, x.id, ' ' FROM connection_types x  LEFT JOIN connection_types ct ON CONCAT(ct.name, ' ( x2)')=x.name WHERE ct.id IS NOT NULL;

ALTER TABLE snapshots 
	ADD COLUMN int_payed BIGINT NOT NULL AFTER int_return,
	ADD COLUMN tv_payed BIGINT NOT NULL AFTER tv_return,
	ADD COLUMN both_payed BIGINT NOT NULL AFTER both_return;
	
ALTER TABLE snapshots 
	ADD COLUMN arpu BIGINT NOT NULL AFTER payed,
	ADD COLUMN int_arpu BIGINT NOT NULL AFTER int_payed,
	ADD COLUMN tv_arpu BIGINT NOT NULL AFTER tv_payed,
	ADD COLUMN both_arpu BIGINT NOT NULL AFTER both_payed;

ALTER TABLE dhcp_templates ADD COLUMN name_server VARCHAR(255);

alter table exp_complaints change column reason reason TEXT;
alter table exp_complaints change column decision decision TEXT;

ALTER TABLE con_requests ADD COLUMN source TINYINT NOT NULL;
UPDATE con_requests SET source=1;

ALTER TABLE con_requests ADD COLUMN recall_date DATETIME, ADD COLUMN accept_date DATETIME;

ALTER TABLE con_requests ADD COLUMN house_id INT UNSIGNED, ADD KEY(house_id), ADD CONSTRAINT fk_con_requests_house_id FOREIGN KEY(house_id) REFERENCES houses(id) ON DELETE SET NULL;

ALTER TABLE asterisk_trunks ADD COLUMN type TINYINT NOT NULL;
ALTER TABLE asterisk_trunks 
ADD COLUMN gsm_number VARCHAR(255),
ADD COLUMN gsm_prefix VARCHAR(255),
ADD COLUMN gsm_trunk VARCHAR(255),
ADD COLUMN gsm_ussd_query VARCHAR(255);

ALTER TABLE asterisk_trunks CHANGE COLUMN notes notes TEXT;

ALTER TABLE connection_types
ADD COLUMN unshape_charge_type INT UNSIGNED,
ADD KEY (unshape_charge_type),
ADD CONSTRAINT fk_connection_types_charge_type FOREIGN KEY (unshape_charge_type) REFERENCES charge_types (id) ON DELETE SET NULL;

ALTER TABLE config ADD COLUMN allow_city_alt TINYINT NOT NULL AFTER actual;

CREATE TABLE config_city (
	id varchar(64) NOT NULL,
	city INT UNSIGNED NOT NULL,
  	value mediumtext NOT NULL,
	actual tinyint NOT NULL,
	stamp bigint unsigned UNIQUE,
	PRIMARY KEY (id, city),
	KEY(city),
	CONSTRAINT fk_config_alt_id FOREIGN KEY (id) REFERENCES config (id) ON DELETE CASCADE,
	CONSTRAINT fk_config_alt_city FOREIGN KEY (city) REFERENCES cities (id) ON DELETE CASCADE,
	CONSTRAINT fk_config_alt_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL
) ENGINE=InnoDB;

UPDATE config SET allow_city_alt=1 WHERE id LIKE 'WM%';
UPDATE config SET allow_city_alt=1 WHERE id LIKE 'PRIVATE%';
UPDATE config SET allow_city_alt=1 WHERE id LIKE 'PORTMONE%';
UPDATE config SET allow_city_alt=1 WHERE id LIKE 'UKRPAYS%';

SET NAMES cp1251;
INSERT INTO config (id, value, control, list, actual, allow_city_alt) values ('PAY_ORDER_TARGET_NAME', ' " "', 1, null, 1, 1);
INSERT INTO config (id, value, control, list, actual, allow_city_alt) values ('PAY_ORDER_TARGET_ACCOUNT', '2600000460246', 1, null, 1, 1);
INSERT INTO config (id, value, control, list, actual, allow_city_alt) values ('PAY_ORDER_TARGET_CODE', '33440550', 1, null, 1, 1);
INSERT INTO config (id, value, control, list, actual, allow_city_alt) values ('PAY_ORDER_BANK_NAME', ' "-"', 1, null, 1, 1);
INSERT INTO config (id, value, control, list, actual, allow_city_alt) values ('PAY_ORDER_BANK_MFO', '321983', 1, null, 1, 1);

CREATE TABLE tv_node_types (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	notes TEXT,
	stamp BIGINT UNSIGNED UNIQUE,
	CONSTRAINT fk_tv_node_types_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL
) ENGINE=InnoDB;

CREATE TABLE tv_nodes (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	parent INT UNSIGNED,
	type INT UNSIGNED,
	city INT UNSIGNED,
	point INT UNSIGNED,
	entrances VARCHAR(255),
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	CONSTRAINT fk_tv_nodes_type FOREIGN KEY (type) REFERENCES tv_node_types (id) ON DELETE SET NULL,
	CONSTRAINT fk_tv_nodes_city FOREIGN KEY (city) REFERENCES cities (id) ON DELETE SET NULL,
	CONSTRAINT fk_tv_nodes_point FOREIGN KEY (point) REFERENCES exp_points (id) ON DELETE SET NULL,
	CONSTRAINT fk_tv_nodes_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL
) ENGINE=InnoDB;

SET NAMES CP1251;
INSERT INTO system_entities (id, key_name, name) VALUES (97, 'tv_node_types', ' -');
INSERT INTO menu_items (id, title, link, access_by) VALUES (131, ' -', 'tv_node_types.php', 97);
INSERT INTO menu_operator_items (menu_item, menu_box, order_num) VALUES (131, 8, 75);

INSERT INTO system_entities (id, key_name, name) VALUES (98, 'tv_nodes', '-');
INSERT INTO menu_items (id, title, link, access_by) VALUES (132, '-', 'tv_nodes.php', 98);
INSERT INTO menu_operator_items (menu_item, menu_box, order_num) VALUES (132, 8, 80);

INSERT INTO system_entities (id, key_name, name) VALUES (99, 'tv_struct', ' -');
INSERT INTO menu_items (id, title, link, access_by) VALUES (133, ' -', 'tv_struct.php', 99);
INSERT INTO menu_operator_items (menu_item, menu_box, order_num) VALUES (133, 8, 90);

CREATE TABLE exp_repair_requests (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	type TINYINT NOT NULL,
	city INT UNSIGNED,
	point INT UNSIGNED,
	address VARCHAR(255),
	state TINYINT NOT NULL,
	priority TINYINT NOT NULL,
	crash_date DATETIME,
	description TEXT,
	solution TEXT,
	stamp BIGINT UNSIGNED UNIQUE,
	CONSTRAINT fk_exp_repair_requests_city FOREIGN KEY (city) REFERENCES cities (id) ON DELETE SET NULL,
	CONSTRAINT fk_exp_repair_requests_point FOREIGN KEY (point) REFERENCES exp_points (id) ON DELETE SET NULL,
	CONSTRAINT fk_exp_repair_requests_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL
) ENGINE=InnoDB;

SET NAMES CP1251;
INSERT INTO system_entities (id, key_name, name) VALUES (100, 'exp_repair_requests',':   ');
INSERT INTO menu_items (id, title, link, access_by) VALUES (134, '  ', 'exp_repair_requests.php', 100);
INSERT INTO menu_operator_items (menu_item, menu_box, order_num) VALUES (134, 5, 120);


ALTER TABLE connection_types ADD COLUMN replace_date DATETIME AFTER replace_after, ADD KEY(replace_date);

ALTER TABLE modems_clients ADD COLUMN apartments INT NOT NULL;

CREATE TABLE tv_boxes (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	house INT UNSIGNED NOT NULL,
	entrance INT,
	floor INT,
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	UNIQUE(house, entrance, floor),
	CONSTRAINT fk_tv_boxes_house FOREIGN KEY (house) REFERENCES houses (id) ON DELETE CASCADE,
	CONSTRAINT fk_tv_boxes_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL
) ENGINE=InnoDB;

CREATE TABLE tv_connections (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	client INT UNSIGNED NOT NULL,
	tv_box INT UNSIGNED NOT NULL,
	notes VARCHAR(255),
	state TINYINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	KEY(client),
	KEY(tv_box),
	CONSTRAINT fk_tv_connections_client FOREIGN KEY (client) REFERENCES clients (id) ON DELETE CASCADE,
	CONSTRAINT fk_tv_connections_tv_box FOREIGN KEY (tv_box) REFERENCES tv_boxes (id),
	CONSTRAINT fk_tv_connections_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL
) ENGINE=InnoDB;

SET NAMES CP1251;
INSERT INTO system_entities (id, key_name, name) VALUES (101, 'tv_connections',' ');
INSERT INTO menu_items (id, title, link, access_by) VALUES (135, ' ', 'tv_connections.php', 101);
INSERT INTO menu_operator_items (menu_item, menu_box, order_num) VALUES (135, 1, 25);

ALTER TABLE modems CHANGE COLUMN vlan vlan VARCHAR(32);

DROP FUNCTION IF EXISTS `complete_job`;
delimiter //
CREATE FUNCTION complete_job (jid INT UNSIGNED) RETURNS INT
BEGIN
DECLARE ac INT;
DECLARE complete INT;
SET complete = 1;
SELECT auto_complete INTO ac FROM job_types WHERE id=(SELECT type FROM jobs WHERE id=jid);
-- check client
IF complete AND ac & 1  AND NOT EXISTS (SELECT id FROM jobs WHERE id=jid AND client_id IS NOT NULL) THEN SET complete = 0; END IF;
-- check tv_type
IF complete AND ac & 2  AND NOT EXISTS (SELECT id FROM clients WHERE id=(SELECT client_id FROM jobs WHERE id=jid) AND tv_type IS NOT NULL) THEN SET complete = 0; END IF;
-- check hosts
IF complete AND ac & 4  AND NOT EXISTS (SELECT id FROM hosts WHERE client=(SELECT id FROM clients WHERE id=(SELECT client_id FROM jobs WHERE id=jid)) LIMIT 1) THEN SET complete = 0; END IF;
-- check expenses
IF complete AND ac & 8  AND NOT EXISTS (SELECT id FROM client_expenses WHERE job=jid LIMIT 1) THEN SET complete = 0; END IF;
-- check address
IF complete AND ac & 16 AND NOT EXISTS (SELECT j.id FROM jobs j LEFT JOIN clients c ON c.id=j.client_id WHERE j.id=jid AND j.house_id=c.house_id AND j.apartment=c.apartment) THEN SET complete = 0; END IF;
-- check tv_connections
IF complete AND ac & 32  AND NOT EXISTS (SELECT id FROM tv_connections WHERE client=(SELECT id FROM clients WHERE id=(SELECT client_id FROM jobs WHERE id=jid)) LIMIT 1) THEN SET complete = 0; END IF;
-- set state
IF complete THEN UPDATE jobs SET state=1 WHERE id=jid; END IF;
RETURN complete;
END//
delimiter ;

ALTER TABLE staff_tools ADD COLUMN notes VARCHAR(255);

ALTER TABLE notifications ADD COLUMN city INT UNSIGNED, ADD KEY(city), ADD CONSTRAINT fk_notifications_city FOREIGN KEY (city) REFERENCES cities (id) ON DELETE SET NULL;

ALTER TABLE streets ADD COLUMN code VARCHAR(64) AFTER abbr;

SET NAMES CP1251;
INSERT INTO system_entities (id, key_name, name) VALUES (102, 'clients_export', ': ');

ALTER TABLE connection_type_tv ADD COLUMN replace_by INT UNSIGNED, ADD FOREIGN KEY (replace_by) REFERENCES tv_types (id) ON DELETE SET NULL;
ALTER TABLE connection_type_tv ADD COLUMN replace_after INT;
ALTER TABLE connection_type_tv ADD COLUMN replace_date DATETIME, ADD KEY(replace_date);

INSERT INTO config (id, value, control, actual) values ('ZABBIX_URL', 'http://zabbix.la.net.ua', 1, 1);
INSERT INTO config (id, value, control, actual) values ('ZABBIX_USER', 'billing', 1, 1);
INSERT INTO config (id, value, control, actual) values ('ZABBIX_PASSWORD', 'GSIeDOVZQ4', 3, 1);
INSERT INTO config (id, value, control, actual) values ('ZABBIX_KEYS', 'ifInErrors, ifHCInMulticastPkts, ifHCOutMulticastPkts, ifHCInOctets, ifHCOutOctets', 0, 1);

CREATE TABLE job_types_tv_types (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	job_type INT NOT NULL,
	tv_type INT UNSIGNED NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	UNIQUE(job_type, tv_type),
	KEY(tv_type),
	CONSTRAINT fk_job_types_tv_types_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	CONSTRAINT fk_job_types_tv_types_job_type FOREIGN KEY (job_type) REFERENCES job_types (id) ON DELETE CASCADE,
	CONSTRAINT fk_job_types_tv_types_tv_type FOREIGN KEY (tv_type) REFERENCES tv_types (id) ON DELETE CASCADE
) ENGINE=InnoDB;


ALTER TABLE tv_types ADD COLUMN city INT UNSIGNED, ADD KEY (city), ADD CONSTRAINT fk_tv_types_city FOREIGN KEY (city) REFERENCES cities (id) ON DELETE SET NULL;

ALTER TABLE ecalls_back ADD COLUMN refuse_reason TINYINT NOT NULL;

ALTER TABLE client_calls ADD COLUMN sale_status TINYINT NOT NULL;

ALTER TABLE client_calls ADD COLUMN complaint TINYINT NOT NULL;

ALTER TABLE client_calls ADD COLUMN recall_date DATETIME;

CREATE TABLE portmone_bills (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	client INT UNSIGNED,
	contract VARCHAR(32) NOT NULL,
	payee VARCHAR(16),
	period VARCHAR(16),
	date DATETIME,
	amount BIGINT NOT NULL,
	KEY(client),
	KEY(date),
	CONSTRAINT fk_portmone_bills_client FOREIGN KEY (client) REFERENCES clients (id) ON DELETE SET NULL
) ENGINE=InnoDB;

CREATE TABLE portmone_bills_pays (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	bill INT UNSIGNED NOT NULL,
	portmone_id VARCHAR(32) UNIQUE,
	date DATETIME,
	amount BIGINT NOT NULL,
	commission BIGINT NOT NULL,
	auth_code VARCHAR(16),
	movement BIGINT UNSIGNED,
	KEY(bill),
	KEY(date),
	KEY(movement),
	CONSTRAINT fk_portmone_bills_pays_bill FOREIGN KEY (bill) REFERENCES portmone_bills (id),
	CONSTRAINT fk_portmone_bills_pays_movement FOREIGN KEY (movement) REFERENCES movements (id) ON DELETE SET NULL
) ENGINE=InnoDB;

CREATE TABLE portmone_bills_orders (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	order_id VARCHAR(32) UNIQUE,
	order_number VARCHAR(32),
	date DATETIME,
	amount BIGINT NOT NULL,
	payee_name VARCHAR(128),
	payee_code VARCHAR(16),
	bank_name  VARCHAR(128),
	bank_code VARCHAR(16),
	bank_account VARCHAR(32),
	KEY(date)
) ENGINE=InnoDB;

CREATE TABLE portmone_bills_orders_details (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	portmone_order INT UNSIGNED,
	pay INT UNSIGNED,
	amount BIGINT NOT NULL,
	commission BIGINT NOT NULL,
	portmone_id VARCHAR(32),
	auth_code VARCHAR(16),
	contract VARCHAR(32),
	KEY(portmone_order),
	KEY(pay),
	CONSTRAINT fk_portmone_bills_orders_details_order FOREIGN KEY (portmone_order) REFERENCES portmone_bills_orders (id) ON DELETE CASCADE,
	CONSTRAINT fk_portmone_bills_orders_details_pay FOREIGN KEY (pay) REFERENCES portmone_bills_pays (id)
) ENGINE=InnoDB;

INSERT INTO config (id, value, control, actual) VALUES ('PORTMONE_ACCESS_ZONE', '194.50.85.2', 0, 1);

ALTER TABLE portmone_bills_orders ADD COLUMN created DATETIME, ADD KEY (created);

ALTER TABLE portmone_bills_pays
ADD COLUMN bank_name  VARCHAR(128),
ADD COLUMN bank_code VARCHAR(16),
ADD COLUMN bank_account VARCHAR(32);

ALTER TABLE portmone_bills ADD COLUMN code VARCHAR(32) UNIQUE AFTER id;
UPDATE portmone_bills SET code=id;

INSERT INTO config (id, value, control, actual) values ('COPINY_DOMAIN', 'rada.my.lanet.ua', 1, 1);
INSERT INTO config (id, value, control, actual) values ('COPINY_PRIVATE_KEY', '1234567890', 1, 1);

ALTER TABLE entrances ADD COLUMN master_key TINYINT NOT NULL;

ALTER TABLE rules ADD COLUMN critical TINYINT NOT NULL;
UPDATE rules SET critical = 1;

DROP TRIGGER IF EXISTS movement_update;
DELIMITER //
CREATE TRIGGER movement_update BEFORE UPDATE ON movements FOR EACH ROW SET NEW.city = IFNULL(get_account_city(NEW.src), get_account_city(NEW.dst));
//
DELIMITER ;

alter table poll_options add column code varchar(32);

alter table houses change column notes notes text;

CREATE TABLE ecall_report_groups (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE
) ENGINE=InnoDB;

CREATE TABLE ecall_report_fields (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	gid INT UNSIGNED NOT NULL,
	name VARCHAR(255) NOT NULL,
	UNIQUE(gid, name),
	CONSTRAINT fk_ecall_report_fields_gid FOREIGN KEY (gid) REFERENCES ecall_report_groups (id) ON DELETE CASCADE
) ENGINE=InnoDB;

CREATE TABLE ecall_reports (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	src TINYINT NOT NULL,
	date date,
	filename VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	KEY(date),
	CONSTRAINT fk_ecall_reports_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL
) ENGINE=InnoDB;

CREATE TABLE ecall_report_values (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	report INT UNSIGNED NOT NULL,
	field INT UNSIGNED NOT NULL,
	value INT NOT NULL,
	KEY(report),
	KEY(field),
	CONSTRAINT fk_ecall_report_values_report FOREIGN KEY (report) REFERENCES ecall_reports (id) ON DELETE CASCADE,
	CONSTRAINT fk_ecall_report_values_field FOREIGN KEY (field) REFERENCES ecall_report_fields (id)
) ENGINE=InnoDB;

CREATE TABLE concurrents (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	city INT UNSIGNED,
	name VARCHAR(64),
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	UNIQUE(city, name),
	CONSTRAINT fk_concurrents_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL
);

INSERT INTO concurrents (id, city, name) VALUES 
(1, 1, ''),
(2, 1, ''),
(3, 1, ''),
(4, 1, ' '),
(5, 1, ''),
(6, 1, ' ()'),
(7, 1, ''),
(8, 1, '');

ALTER TABLE client_calls 
ADD COLUMN start_uniform TINYINT NOT NULL,
ADD COLUMN concurrent INT UNSIGNED,
ADD KEY(concurrent),
ADD CONSTRAINT fk_client_calls_concurrent FOREIGN KEY (concurrent) REFERENCES concurrents (id) ON DELETE SET NULL,
ADD COLUMN debts_reason TINYINT NOT NULL,
ADD COLUMN debts_promise DATE,
ADD COLUMN refuse_reason TINYINT NOT NULL;

alter table houses_notes change column note note text;

CREATE TABLE entrance_notes (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
  	entrance INT UNSIGNED NOT NULL,
	note TEXT,
	stamp BIGINT UNSIGNED UNIQUE,
	KEY(entrance),
	CONSTRAINT fk_entrance_notes_entrance FOREIGN KEY (entrance) REFERENCES entrances (id) ON DELETE CASCADE,
	CONSTRAINT fk_entrance_notes_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL
) ENGINE=InnoDB;

CREATE TABLE koops_files (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	koop INT UNSIGNED NOT NULL,
	file INT UNSIGNED NOT NULL,
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	key(koop),
	key(file),
	CONSTRAINT fk_koops_files_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	CONSTRAINT fk_koops_files_koops FOREIGN KEY (koop) REFERENCES koops (id) ON DELETE CASCADE
) ENGINE=InnoDB;

insert into koops_files (koop, file, notes) select id, contract_file, CONCAT(" ", IFNULL(contract,"")) FROM koops WHERE contract_file IS NOT NULL;

CREATE TABLE exp_complaints_decisions (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	complaint INT UNSIGNED NOT NULL,
	decision TEXT,
	stamp BIGINT UNSIGNED UNIQUE,
	key(complaint),
	CONSTRAINT fk_exp_complaints_decisions_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	CONSTRAINT fk_exp_complaints_decisions_complaints FOREIGN KEY (complaint) REFERENCES exp_complaints (id) ON DELETE CASCADE
) ENGINE=InnoDB;

insert into exp_complaints_decisions (complaint, decision)  select id, decision from exp_complaints where decision is not null and decision > "";

ALTER TABLE tv_types ADD COLUMN client_notes VARCHAR(255);

ALTER TABLE clients CHANGE COLUMN allow_ip_login allow_ip_login TINYINT NOT NULL DEFAULT 1;

ALTER TABLE rules ADD COLUMN auto_firewall TINYINT NOT NULL;

alter table ip_use_time drop foreign key fk_ip_use_time_ip_use, drop key ip_use;
alter table ip_use_hours drop foreign key fk_ip_use_hours_ip_use, drop key ip_use;
alter table ip_use_days drop foreign key fk_ip_use_days_ip_use, drop key ip_use;

ALTER TABLE client_calls ADD COLUMN status TINYINT NOT NULL;

UPDATE client_calls SET status=CASE WHEN type=5 OR type=6 THEN (
CASE sale_status
when 1 then 4
when 2 then 3
when 3 then 1
when 4 then 2
else 0 end)
WHEN type=2 THEN (
CASE start_status
when 1 then 5
when 2 then 4
when 3 then 3
else 0 end
) ELSE 0 END;

alter table modems drop key name, add unique(domain, name);
alter table cities add column abbr varchar(16) unique; 

alter table clients  change column house_id house_id INT UNSIGNED;
update clients c left join houses h on h.id=c.house_id  set c.house_id=null where h.id is null;
alter table clients add constraint fk_client_houses foreign key(house_id) references houses (id);

ALTER TABLE connection_types ADD COLUMN check_offline TINYINT NOT NULL;
UPDATE connection_types SET check_offline=1;

CREATE TABLE jobs_finders (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(64) NOT NULL UNIQUE,
	actual TINYINT NOT NULL DEFAULT 1,
	show_memo TINYINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	CONSTRAINT fk_jobs_finders_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL
) ENGINE=InnoDB;

SET NAMES cp1251;
INSERT INTO jobs_finders (id, name) VALUES 
(1, '  '),
(2, '  '),
(3, '  '),
(4, '    '),
(5, ' '),
(6, '  '),
(7, ''),
(8, ''),
(9, ' '),
(10, '  '),
(11, ' '),
(12, '  '),
(13, ' ');

SET NAMES CP1251;
INSERT INTO system_entities (id, key_name, name) VALUES (103, 'jobs_finders', ':  ');
INSERT INTO menu_items (id, title, link, access_by) VALUES (136, ' ', 'jobs_finders.php', 103);
INSERT INTO menu_operator_items (menu_item, menu_box, order_num) VALUES (136, 4, 260);

ALTER TABLE jobs CHANGE COLUMN found_by found_by INT UNSIGNED;
UPDATE jobs SET found_by=NULL WHERE found_by=0;
ALTER TABLE jobs ADD CONSTRAINT fk_jobs_found_by FOREIGN KEY (found_by) REFERENCES jobs_finders(id);

CREATE TABLE jobs_notes (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	job INT UNSIGNED,
	note text,
	stamp BIGINT UNSIGNED UNIQUE,
	CONSTRAINT fk_jobs_notes_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	CONSTRAINT fk_jobs_notes_job FOREIGN KEY (job) REFERENCES jobs (id) ON DELETE CASCADE
) ENGINE=InnoDB;

SET NAMES CP1251;
INSERT INTO system_entities (id, key_name, name) VALUES (104, 'jobs_notes', ': ');

DELIMITER //
CREATE FUNCTION get_account_client(x INT UNSIGNED) RETURNS INT UNSIGNED
BEGIN
DECLARE t CHAR(1);
DECLARE r INT UNSIGNED;
SET r=NULL;
SELECT type INTO t FROM accounts WHERE id=x;
IF t='D' THEN SELECT id INTO r FROM clients WHERE account=x; END IF;
IF t='B' THEN SELECT client INTO r FROM bonus_accounts WHERE account=x; END IF;
RETURN r;
END //
DELIMITER ;


ALTER TABLE movements
ADD column client INT UNSIGNED AFTER city,
ADD KEY(client);

DROP TRIGGER IF EXISTS movement_insert;
DELIMITER //
CREATE TRIGGER movement_insert BEFORE INSERT ON movements
FOR EACH ROW
BEGIN
SET NEW.city = IFNULL(get_account_city(NEW.src), get_account_city(NEW.dst));
SET NEW.client = IFNULL(get_account_client(NEW.src), get_account_client(NEW.dst));
END //
DELIMITER ;

DROP TRIGGER IF EXISTS movement_update;
DELIMITER //
CREATE TRIGGER movement_update BEFORE UPDATE ON movements
FOR EACH ROW
BEGIN
SET NEW.city = IFNULL(get_account_city(NEW.src), get_account_city(NEW.dst));
SET NEW.client = IFNULL(get_account_client(NEW.src), get_account_client(NEW.dst));
END //
DELIMITER ;

UPDATE movements m INNER JOIN clients c ON c.account=m.src SET m.client=c.id;
UPDATE movements m INNER JOIN clients c ON c.account=m.dst SET m.client=c.id;
UPDATE movements m INNER JOIN bonus_accounts ba ON ba.account=m.src SET m.client=ba.client;
UPDATE movements m INNER JOIN bonus_accounts ba ON ba.account=m.dst SET m.client=ba.client;

ALTER TABLE tv_node_types ADD COLUMN feedback TINYINT NOT NULL;

ALTER TABLE tv_nodes ADD COLUMN power_in INT NOT NULL, ADD COLUMN power_out INT NOT NULL, ADD COLUMN audit TINYINT NOT NULL;

ALTER TABLE domains ADD COLUMN hidden TINYINT NOT NULL;

CREATE TABLE clients_documents (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	number INT NOT NULL,
	city INT UNSIGNED,
	query INT UNSIGNED,
	file INT UNSIGNED,
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	KEY(city),
	KEY(query),
	KEY(file),
	CONSTRAINT fk_clients_documents_city FOREIGN KEY (city) REFERENCES cities (id) ON DELETE SET NULL,
	CONSTRAINT fk_clients_documents_query FOREIGN KEY (query) REFERENCES queries (id) ON DELETE SET NULL,
	CONSTRAINT fk_clients_documents_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL
) ENGINE=InnoDB;

SET NAMES CP1251;
INSERT INTO system_entities (id, key_name, name) VALUES (105, 'clients_documents', ': ');
INSERT INTO menu_items (id, title, link, access_by) VALUES (137, '', 'clients_documents.php', 105);
INSERT INTO menu_operator_items (menu_item, menu_box, order_num) VALUES (137, 1, 260);

ALTER TABLE houses ADD COLUMN show_on_map TINYINT NOT NULL;
UPDATE houses SET show_on_map=1;

alter table tv_connections add column tv_node int unsigned, add key(tv_node), add constraint fk_tv_connections_tv_node foreign key(tv_node) references tv_nodes(id);

CREATE TABLE tv_nodes_nest (
  id INT UNSIGNED NOT NULL PRIMARY KEY,
  parent int unsigned DEFAULT NULL,
  level int NOT NULL,
  left_key int NOT NULL,
  right_key int NOT NULL,
  KEY right_key (right_key),
  KEY left_key (left_key,right_key)
) ENGINE=InnoDB;

CREATE TABLE ivr_messages (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	city INT UNSIGNED,
	name VARCHAR(64) NOT NULL,
	filename VARCHAR(255),
	number INT NOT NULL,
	notes VARCHAR(255),
	actual TINYINT NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	UNIQUE(city, name),
	CONSTRAINT fk_messages_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL
) ENGINE=InnoDB;

CREATE TABLE ivr_autoreply_tv (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	city INT UNSIGNED,
	tv_node INT UNSIGNED NOT NULL,
	message INT UNSIGNED,
	scope TINYINT NOT NULL,
	actual_from DATETIME,
	actual_to DATETIME,
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	KEY(city),
	KEY(tv_node),
	KEY(message),
	CONSTRAINT fk_ivr_autoreply_tv_city FOREIGN KEY (city) REFERENCES cities (id) ON DELETE SET NULL,
	CONSTRAINT fk_ivr_autoreply_tv_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	CONSTRAINT fk_ivr_autoreply_tv_node FOREIGN KEY (tv_node) REFERENCES tv_nodes (id) ON DELETE CASCADE,
	CONSTRAINT fk_ivr_autoreply_tv_message FOREIGN KEY (message) REFERENCES ivr_messages (id)
) ENGINE=InnoDB;

SET NAMES CP1251;
INSERT INTO system_entities (id, key_name, name) VALUES (106, 'ivr_messages', 'IVR ');
INSERT INTO menu_items (id, title, link, access_by) VALUES (138, 'IVR ', 'ivr_messages.php', 106);
INSERT INTO menu_operator_items (menu_item, menu_box, order_num) VALUES (138, 12, 70);

SET NAMES CP1251;
INSERT INTO system_entities (id, key_name, name) VALUES (107, 'ivr_autoreply_tv', 'IVR  ');
INSERT INTO menu_items (id, title, link, access_by) VALUES (139, 'IVR  ', 'ivr_autoreply_tv.php', 107);
INSERT INTO menu_operator_items (menu_item, menu_box, order_num) VALUES (139, 12, 80);

DELIMITER ;;
DROP PROCEDURE IF EXISTS get_client_ivr_messages;;
CREATE PROCEDURE get_client_ivr_messages (_clid INT UNSIGNED)
BEGIN
DECLARE _tv_left INT;
DECLARE _tv_right INT;
DECLARE _tv_node INT;

SELECT left_key, right_key, id INTO _tv_left, _tv_right, _tv_node
FROM tv_nodes_nest
WHERE id = (SELECT tv_node FROM tv_connections WHERE client=_clid LIMIT 1);

SELECT im.filename 
FROM ivr_autoreply_tv as iat 
LEFT JOIN ivr_messages as im ON im.id=iat.message 
LEFT JOIN tv_nodes_nest as tnn ON tnn.id=iat.tv_node 
WHERE (iat.tv_node=_tv_node OR (tnn.left_key < _tv_left AND tnn.right_key > _tv_right AND iat.scope=1))
AND (iat.actual_from IS NULL OR iat.actual_from <= NOW()) AND (iat.actual_to IS NULL OR iat.actual_to > NOW())
ORDER BY im.number, iat.id;
END ;;
DELIMITER ;

ALTER TABLE connection_types ADD COLUMN service_type TINYINT NOT NULL DEFAULT 3;
ALTER TABLE tv_types ADD COLUMN offline TINYINT NOT NULL;

ALTER TABLE connection_types 
ADD COLUMN offline_type INT UNSIGNED, 
ADD KEY(offline_type), 
ADD CONSTRAINT fk_connection_types_offline_type FOREIGN KEY(offline_type) REFERENCES connection_types(id) ON DELETE SET NULL;

-- UPDATE connection_types SET offline_type=19 WHERE check_offline=1 AND city=1;
-- UPDATE connection_types SET offline_type=490 WHERE check_offline=1 AND city=2;

CREATE TABLE asterisk_queues (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(64) NOT NULL UNIQUE,
	query INT UNSIGNED,
	target_cmd VARCHAR(255),
	notes VARCHAR(255),
	actual TINYINT NOT NULL,
	last_sent DATETIME,
	last_sender INT UNSIGNED,
	stamp BIGINT UNSIGNED UNIQUE,
	KEY(query),
	KEY(last_sender),
	CONSTRAINT fk_asterisk_queues_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	CONSTRAINT fk_asterisk_queues_query FOREIGN KEY (query) REFERENCES queries (id),
	CONSTRAINT fk_asterisk_queues_last_sender FOREIGN KEY (last_sender) REFERENCES operators (id) ON DELETE SET NULL
) ENGINE=InnoDB;

SET NAMES CP1251;
INSERT INTO system_entities (id, key_name, name) VALUES (108, 'asterisk_queues', 'Asterisk: ');
INSERT INTO menu_items (id, title, link, access_by) VALUES (140, '', 'asterisk_queues.php', 108);
INSERT INTO menu_operator_items (menu_item, menu_box, order_num) VALUES (140, 12, 90);


SET NAMES CP1251;
INSERT INTO system_entities (id, key_name, name) VALUES (109, 'snapshots', '');
UPDATE menu_items SET access_by=109 WHERE id=85;

CREATE TABLE tv_nodes_clients (
	tv_node INT UNSIGNED NOT NULL PRIMARY KEY,
	clients INT NOT NULL,
	clients_total INT NOT NULL,
	apartments INT NOT NULL,
	CONSTRAINT fk_tv_nodes_clients_tv_node FOREIGN KEY(tv_node) REFERENCES tv_nodes(id) ON DELETE CASCADE
) ENGINE =InnoDB;


ALTER TABLE troubles ADD COLUMN callback TINYINT NOT NULL, ADD COLUMN callback_phone VARCHAR(32); 

alter table tv_connections add column card_code varchar(64), add unique(card_code);

DELIMITER ;;
DROP PROCEDURE IF EXISTS ivr_get_client_info;;
CREATE PROCEDURE ivr_get_client_info (_clid INT UNSIGNED)
BEGIN
DECLARE _incomplete BIGINT;
DECLARE _blocked BIGINT;
DECLARE _deposit BIGINT;
SELECT MIN(type) INTO _blocked FROM blocks WHERE client=_clid AND (actual_from IS NULL OR actual_from < NOW()) AND (actual_to IS NULL OR actual_to > NOW()) AND type IN (1,2);
SELECT incomplete INTO _incomplete FROM clients_states cs WHERE client=_clid;
SELECT state_end - IFNULL(_incomplete, 0) INTO _deposit
FROM calendar_account_states 
WHERE account=(SELECT account FROM clients WHERE id=_clid) AND calendar_period=(SELECT id FROM calendar_periods WHERE marker=1);
SELECT IFNULL(_deposit,0) as deposit, IFNULL(_blocked, 0) as blocked;
END ;;
DELIMITER ;

alter table clients add column lang tinyint not null;

ALTER TABLE connection_types ADD COLUMN gigabit TINYINT NOT NULL;

SET NAMES CP1251;
INSERT INTO system_entities (id, key_name, name) VALUES (110, 'con_requests', '');
INSERT INTO operators_access  (operator, entity, access)
SELECT operator, 110, access FROM operators_access WHERE entity=41;
INSERT INTO groups_access  (group_id, entity, access)
SELECT group_id, 110, access FROM groups_access WHERE entity=41;
UPDATE menu_items SET access_by=110 WHERE id=32;


SET NAMES CP1251;
INSERT INTO system_entities (id, key_name, name) VALUES (111, 'jobs_extras', ':  ');
INSERT INTO menu_items (id, title, link, access_by) VALUES (141, ' ', 'jobs_extras.php', 111);
INSERT INTO menu_operator_items (menu_item, menu_box, order_num) VALUES (141, 4, 270);

INSERT INTO operators_access  (operator, entity, access)
SELECT operator, 111, access FROM operators_access WHERE entity=33;
INSERT INTO groups_access  (group_id, entity, access)
SELECT group_id, 111, access FROM groups_access WHERE entity=33;

alter table jobs_extras change column job job int unsigned;
update jobs_extras je left join jobs j on j.id=je.job set je.job=null where j.id is null;
alter table jobs_extras add constraint fk_jobs_extras foreign key(job) references jobs (id) on delete cascade;

ALTER TABLE jobs_extras ADD COLUMN pay_state TINYINT NOT NULL;

ALTER TABLE jobs_extras ADD COLUMN ticket_state TINYINT NOT NULL;

alter table clients change column surveyor surveyor tinyint not null;

CREATE TABLE tv_node_profiles (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE,
	driver TINYINT NOT NULL DEFAULT 1,
	config TEXT,
	actual tinyint(4) NOT NULL,
	stamp BIGINT UNSIGNED UNIQUE,
	CONSTRAINT fk_tv_node_profiles_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL
) ENGINE =InnoDB;

SET NAMES CP1251;
INSERT INTO system_entities (id, key_name, name) VALUES (112, 'tv_node_profiles', '-: ');
INSERT INTO menu_items (id, title, link, access_by) VALUES (142, ' -', 'tv_node_profiles.php', 112);
INSERT INTO menu_operator_items (menu_item, menu_box, order_num) VALUES (142, 8, 100);

ALTER TABLE tv_nodes 
ADD COLUMN ip VARCHAR(32) UNIQUE,
ADD COLUMN mac VARCHAR(32) UNIQUE,
ADD COLUMN profile INT UNSIGNED,
ADD KEY(profile),
ADD CONSTRAINT fk_tv_nodes_profile FOREIGN KEY (profile) REFERENCES tv_node_profiles (id);

CREATE TABLE tv_nodes_power (
	tv_node INT UNSIGNED NOT NULL PRIMARY KEY,
	available TINYINT NOT NULL,
	input_power INT NOT NULL,
	output_power INT NOT NULL,
	CONSTRAINT fk_tv_nodes_power_tv_node FOREIGN KEY(tv_node) REFERENCES tv_nodes(id) ON DELETE CASCADE
) ENGINE=InnoDB;

ALTER TABLE tv_node_types ADD COLUMN power_out INT NOT NULL;

CREATE TABLE client_visits_reasons (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL UNIQUE
) ENGINE=InnoDB;

CREATE TABLE client_visits (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	city INT UNSIGNED,
	client INT UNSIGNED,
	date DATETIME,
	reason INT UNSIGNED,
	notes TEXT,
	stamp BIGINT UNSIGNED UNIQUE,
	KEY(city),
	KEY(client),
	KEY(date),
	KEY(reason),
	CONSTRAINT fk_client_visits_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	CONSTRAINT fk_client_visits_client FOREIGN KEY (client) REFERENCES clients (id) ON DELETE SET NULL,
	CONSTRAINT fk_client_visits_city FOREIGN KEY (city) REFERENCES cities (id),
	CONSTRAINT fk_client_visits_reason FOREIGN KEY (reason) REFERENCES client_visits_reasons (id)
) ENGINE =InnoDB;

SET NAMES CP1251;
INSERT INTO system_entities (id, key_name, name) VALUES (113, 'client_visits', ': ');
INSERT INTO menu_items (id, title, link, access_by) VALUES (143, '', 'client_visits.php', 113);
INSERT INTO menu_operator_items (menu_item, menu_box, order_num) VALUES (143, 1, 270);

INSERT INTO client_visits_reasons(id,name) VALUES
(1, ''),
(2, ''),
(3, ''),
(4, ''),
(5, ''),
(6, ''),
(7, '');

ALTER TABLE client_visits_reasons ADD COLUMN actual TINYINT NOT NULL DEFAULT 1;

insert into config (id, value, control, actual) values ('UKRCARD_PROVIDER_PRIVATE', '', 0, 0);
insert into config (id, value, control, actual) values ('UKRCARD_PSA_PUBLIC', '', 0, 0);

UPDATE houses h LEFT JOIN koops k on k.id=h.koop SET h.koop=NULL WHERE h.koop IS NOT NULL AND k.id IS NULL;

ALTER TABLE houses CHANGE COLUMN dispatching dispatching INT UNSIGNED;

ALTER TABLE houses 
ADD CONSTRAINT fk_houses_street FOREIGN KEY (street) REFERENCES streets(id),
ADD CONSTRAINT fk_houses_district FOREIGN KEY (district) REFERENCES districts(id),
ADD CONSTRAINT fk_houses_kpreo FOREIGN KEY (kpreo) REFERENCES kpreos(id) ON DELETE SET NULL,
ADD CONSTRAINT fk_houses_kpreo1 FOREIGN KEY (kpreo1) REFERENCES kpreos(id) ON DELETE SET NULL,
ADD CONSTRAINT fk_houses_koop FOREIGN KEY (koop) REFERENCES koops(id) ON DELETE SET NULL,
ADD CONSTRAINT fk_houses_koop1 FOREIGN KEY (koop1) REFERENCES koops(id) ON DELETE SET NULL,
ADD CONSTRAINT fk_houses_dispatching FOREIGN KEY (dispatching) REFERENCES dispatching(id) ON DELETE SET NULL,
ADD CONSTRAINT fk_houses_city FOREIGN KEY (city) REFERENCES cities(id),
ADD CONSTRAINT fk_houses_stamp FOREIGN KEY (stamp) REFERENCES stamps(id) ON DELETE SET NULL;

-- locked
ALTER TABLE domains ADD CONSTRAINT fk_domains_stamp FOREIGN KEY (stamp) REFERENCES stamps(id) ON DELETE SET NULL;

ALTER TABLE exp_tools ADD CONSTRAINT fk_exp_tools_stamp FOREIGN KEY (stamp) REFERENCES stamps(id) ON DELETE SET NULL;

ALTER TABLE ads
ADD CONSTRAINT fk_ads_stamp FOREIGN KEY (stamp) REFERENCES stamps(id) ON DELETE SET NULL,
ADD CONSTRAINT fk_ads_query FOREIGN KEY (query) REFERENCES queries(id) ON DELETE SET NULL; 

create table _unknown_browsers select bc.browser from browser_to_client bc left join browsers b on b.id=bc.browser where b.id is null;
delete from browser_to_client where browser in (select id from _unknown_browsers);
drop table _unknown_browsers;

create table _unknown_clients select bc.client from browser_to_client bc left join client c on c.id=bc.client where c.id is null;
delete from browser_to_client where client in (select id from _unknown_clients);
drop table _unknown_clients;

ALTER TABLE browser_to_client DROP KEY browser,
ADD CONSTRAINT fk_browser_to_client_client FOREIGN KEY (client) REFERENCES clients(id) ON DELETE CASCADE,
ADD CONSTRAINT fk_browser_to_client_browser FOREIGN KEY (browser) REFERENCES browsers(id) ON DELETE CASCADE; 

DELETE FROM browsers WHERE 
(adddate(last_visit, interval 2 month) < now()) and (visit_counter < 10) 
or adddate(last_visit, interval 6 month) < now();

ALTER TABLE charge_types
ADD CONSTRAINT fk_charge_types_resource_type FOREIGN KEY (resource_type) REFERENCES resource_types(id),
ADD CONSTRAINT fk_charge_types_account FOREIGN KEY (account) REFERENCES accounts(id);

ALTER TABLE client_shaper_patches
ADD CONSTRAINT fk_client_shaper_patches_client FOREIGN KEY (client) REFERENCES clients(id) ON DELETE CASCADE;

ALTER TABLE concurrents
ADD CONSTRAINT fk_concurrents_city FOREIGN KEY (city) REFERENCES cities(id) ON DELETE CASCADE;

ALTER TABLE expense_material_types DROP COLUMN job_type;

ALTER TABLE jobs CHANGE COLUMN type type INT;
UPDATE jobs j LEFT JOIN job_types t ON t.id=j.type SET j.type=NULL WHERE j.type IS NOT NULL AND t.id IS NULL;
UPDATE jobs j LEFT JOIN houses h ON h.id=j.house_id SET j.house_id=NULL WHERE j.house_id IS NOT NULL AND h.id IS NULL;
ALTER TABLE jobs 
ADD KEY(type),
ADD CONSTRAINT fk_jobs_type FOREIGN KEY (type) REFERENCES job_types(id),
ADD CONSTRAINT fk_jobs_house FOREIGN KEY (house_id) REFERENCES houses(id);

alter table koops drop foreign key koops_ibfk_2, drop column street, drop column house, drop column house_id;

delete from modems_states where date < '2011-07-01';
UPDATE modems m LEFT JOIN modems_states ms ON ms.id=m.state_id SET m.state_id = NULL WHERE ms.id IS NULL;

ALTER TABLE modems 
ADD KEY(state_id),
ADD CONSTRAINT fk_modems_state FOREIGN KEY (state_id) REFERENCES modems_states(id) ON DELETE SET NULL;

UPDATE offline_ports o LEFT JOIN hosts h ON h.id=o.host SET o.host=NULL WHERE h.id IS NULL;
ALTER TABLE offline_ports ADD CONSTRAINT fk_offline_ports_host FOREIGN KEY (host) REFERENCES hosts(id) ON DELETE SET NULL;

ALTER TABLE pays_import ADD CONSTRAINT fk_pays_import_calendar_period FOREIGN KEY (calendar_period) REFERENCES calendar_periods(id);

ALTER TABLE request_emails CHANGE COLUMN stamp stamp BIGINT UNSIGNED, ADD UNIQUE(stamp), ADD CONSTRAINT fk_request_emails_stamp FOREIGN KEY (stamp) REFERENCES stamps(id) ON DELETE SET NULL;

UPDATE snmp_status s LEFT JOIN clients c ON c.id=s.client_id SET s.client_id = NULL WHERE c.id IS NULL;
ALTER TABLE snmp_status CHANGE COLUMN client_id client_id INT UNSIGNED, ADD KEY(client_id), ADD CONSTRAINT fk_snmp_status_client FOREIGN KEY (client_id) REFERENCES clients (id) ON DELETE CASCADE;

ALTER TABLE tech_zone_job_types CHANGE COLUMN type type INT NOT NULL, ADD CONSTRAINT fk_tech_zone_job_types_type FOREIGN KEY(type) REFERENCES job_types(id);

ALTER TABLE tech_zones ADD CONSTRAINT fk_tech_zones_stamp FOREIGN KEY (stamp) REFERENCES stamps(id) ON DELETE SET NULL;

ALTER TABLE tv_nodes ADD KEY(parent), ADD CONSTRAINT fk_tv_nodes_parent FOREIGN KEY (parent) REFERENCES tv_nodes(id) ON DELETE SET NULL;

CREATE TABLE ukrcard_pays_regs (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	paysysid VARCHAR(255) NOT NULL,
	registerid VARCHAR(255) NOT NULL,
	mfo VARCHAR(255),
	registerdate DATETIME,
	orderno VARCHAR(255),
	orderdate DATETIME,
	ordersum BIGINT NOT NULL,
	allcount BIGINT NOT NULL,
	allsumma BIGINT NOT NULL,
	UNIQUE(paysysid, registerid)
) ENGINE=InnoDB;

CREATE TABLE ukrcard_pays (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	tranid VARCHAR(255) NOT NULL,
	paysysid VARCHAR(255) NOT NULL,
	serviceid VARCHAR(32) NOT NULL,
	payercode VARCHAR(64) NOT NULL,
	paydate DATETIME,
	amount BIGINT,
	client INT UNSIGNED,
	movement BIGINT UNSIGNED,
	state TINYINT NOT NULL,
	reg INT UNSIGNED,
	KEY(client),
	KEY(movement),
	KEY(reg),
	CONSTRAINT fk_ukrcard_pays_client FOREIGN KEY (client) REFERENCES clients (id) ON DELETE SET NULL,
	CONSTRAINT fk_ukrcard_pays_movement FOREIGN KEY (movement) REFERENCES movements (id) ON DELETE SET NULL,
	CONSTRAINT fk_ukrcard_pays_reg FOREIGN KEY (reg) REFERENCES ukrcard_pays_regs (id) ON DELETE SET NULL
) ENGINE=InnoDB;

DELETE FROM config WHERE id IN ('UKRCARD_PROVIDER_PRIVATE', 'UKRCARD_PSA_PUBLIC');

INSERT INTO config (id, value, control, actual) VALUES ('UKRCARD_PSA_LIST', 'UKRC=', 0, 0);
INSERT INTO config (id, value, control, actual) VALUES ('UKRCARD_SERVICE_LIST', 'INET=', 0, 0);

SET NAMES CP1251;
INSERT INTO system_entities (id, key_name, name) VALUES (114, 'ukrcard_pays', ': ukrcard');
INSERT INTO menu_items (id, title, link, access_by) VALUES (144, 'ukrcard', 'ukrcard_pays.php', 114);
INSERT INTO menu_operator_items (menu_item, menu_box, order_num) VALUES (144, 1, 195);

CREATE TABLE ibox_new_pays (
	id BIGINT NOT NULL AUTO_INCREMENT PRIMARY KEY,
	client INT UNSIGNED,
	date DATETIME NOT NULL,
	amount BIGINT UNSIGNED NOT NULL,
	order_id VARCHAR(255) NOT NULL UNIQUE,
	pay_type INT NOT NULL,
	movement BIGINT UNSIGNED,
	KEY(client),
	KEY(movement),
	KEY(date),
	CONSTRAINT fk_ibox_new_pays_client FOREIGN KEY (client)  REFERENCES clients (id) ON DELETE set null,
	CONSTRAINT fk_ibox_new_pays_movement FOREIGN KEY (movement)  REFERENCES movements (id) ON DELETE set null
)ENGINE=InnoDB;

INSERT INTO config (id, value, control, actual) VALUES ('IBOX_ACCESS_ZONE', '213.160.149.230', 0, 1);

ALTER TABLE houses ADD COLUMN tv_reason VARCHAR(255), ADD COLUMN internet_reason VARCHAR(255);

ALTER TABLE entrance_notes ADD COLUMN file INT UNSIGNED, ADD KEY (file);

ALTER TABLE asterisk_queues ADD COLUMN code VARCHAR(64) UNIQUE; 

INSERT INTO config (id, value, control, actual) VALUES ('ASTERISK_LIST_DISPATCHERS', "ssh billing@194.50.85.172 'sudo /usr/local/bin/list-dispatchers'", 0, 1);

ALTER TABLE asterisk_queues ADD COLUMN stop_cmd VARCHAR(255);

CREATE TABLE job_problem_types (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	name VARCHAR(255) NOT NULL,
	service_type TINYINT NOT NULL,
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
  	city INT UNSIGNED,
	UNIQUE KEY (name, city),
	KEY(city),
	CONSTRAINT fk_job_problem_types_city FOREIGN KEY (city) REFERENCES cities (id) ON DELETE SET NULL,
	CONSTRAINT fk_job_problem_types_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL
)Engine=InnoDB;

SET names cp1251;
INSERT INTO system_entities (id, key_name, name) VALUES (115, 'job_problem_types',':  ');
INSERT INTO menu_items (id, title, link, access_by) VALUES (145, ' ', 'job_problem_types.php', 115);
INSERT INTO menu_operator_items (menu_item, menu_box, order_num) VALUES (145, 4, 280);

ALTER TABLE jobs_notes
ADD COLUMN problem_type INT UNSIGNED,
ADD KEY(problem_type),
ADD CONSTRAINT fk_jobs_notes_problem_type FOREIGN KEY (problem_type) REFERENCES job_problem_types (id); 

CREATE TABLE ivr_autoreply_internet (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	city INT UNSIGNED,
	modem INT UNSIGNED NOT NULL,
	message INT UNSIGNED,
	scope TINYINT NOT NULL,
	actual_from DATETIME,
	actual_to DATETIME,
	notes VARCHAR(255),
	stamp BIGINT UNSIGNED UNIQUE,
	KEY(city),
	KEY(modem),
	KEY(message),
	CONSTRAINT fk_ivr_autoreply_internet_city FOREIGN KEY (city) REFERENCES cities (id) ON DELETE SET NULL,
	CONSTRAINT fk_ivr_autoreply_internet_stamp FOREIGN KEY (stamp) REFERENCES stamps (id) ON DELETE SET NULL,
	CONSTRAINT fk_ivr_autoreply_internet_modem FOREIGN KEY (modem) REFERENCES modems (id) ON DELETE CASCADE,
	CONSTRAINT fk_ivr_autoreply_internet_message FOREIGN KEY (message) REFERENCES ivr_messages (id)
) ENGINE=InnoDB;

SET NAMES CP1251;
INSERT INTO system_entities (id, key_name, name) VALUES (116, 'ivr_autoreply_internet', 'IVR  ');
INSERT INTO menu_items (id, title, link, access_by) VALUES (146, 'IVR  ', 'ivr_autoreply_internet.php', 116);
INSERT INTO menu_operator_items (menu_item, menu_box, order_num) VALUES (146, 12, 85);

DELIMITER ;;
DROP PROCEDURE IF EXISTS get_client_ivr_messages;;
CREATE PROCEDURE get_client_ivr_messages (_clid INT UNSIGNED)
BEGIN
DECLARE _tv_left INT;
DECLARE _tv_right INT;
DECLARE _tv_node INT;

DECLARE _int_left INT;
DECLARE _int_right INT;
DECLARE _int_node INT;

SELECT left_key, right_key, id INTO _tv_left, _tv_right, _tv_node
FROM tv_nodes_nest
WHERE id = (SELECT tv_node FROM tv_connections WHERE client=_clid LIMIT 1);

SELECT left_key, right_key, id INTO _int_left, _int_right, _int_node
FROM modems_nest
WHERE id = (SELECT modem FROM hosts WHERE client=_clid LIMIT 1);

SELECT DISTINCT filename FROM (
SELECT im.filename, im.number, iat.id, 2 as k
  FROM ivr_autoreply_tv as iat 
  LEFT JOIN ivr_messages as im ON im.id=iat.message 
  LEFT JOIN tv_nodes_nest as tnn ON tnn.id=iat.tv_node 
  WHERE (iat.tv_node=_tv_node OR (tnn.left_key < _tv_left AND tnn.right_key > _tv_right AND iat.scope=1))
  AND (iat.actual_from IS NULL OR iat.actual_from <= NOW()) AND (iat.actual_to IS NULL OR iat.actual_to > NOW())
UNION
  SELECT im.filename, im.number, iai.id, 1 as k
  FROM ivr_autoreply_internet as iai 
  LEFT JOIN ivr_messages as im ON im.id=iai.message 
  LEFT JOIN modems_nest as mn ON mn.id=iai.modem
  WHERE (iai.modem=_int_node OR (mn.left_key < _int_left AND mn.right_key > _int_right AND iai.scope=1))
  AND (iai.actual_from IS NULL OR iai.actual_from <= NOW()) AND (iai.actual_to IS NULL OR iai.actual_to > NOW())
) x
ORDER BY number,k,id;

END ;;
DELIMITER ;

ALTER TABLE client_calls
ADD COLUMN rating_tv_rating TINYINT NOT NULL,
ADD COLUMN rating_tv_bad TINYINT NOT NULL,
ADD COLUMN repair_done TINYINT NOT NULL,
ADD COLUMN repair_rating TINYINT NOT NULL,
ADD COLUMN repair_reason TINYINT NOT NULL;

ALTER TABLE job_types ADD COLUMN repair TINYINT NOT NULL;
UPDATE job_types SET repair=1 WHERE name LIKE '%%';

CREATE TABLE tv_use_amount (client INT UNSIGNED NOT NULL PRIMARY KEY, amount INT NOT NULL) ENGINE=InnoDB;

DELIMITER ;;
DROP PROCEDURE IF EXISTS calc_tv_use;;
CREATE PROCEDURE calc_tv_use ()
BEGIN
DELETE FROM tv_use_amount;
INSERT INTO tv_use_amount(client, amount)
SELECT pp.client, SUM(DATEDIFF(pp.period_end, pp.period_begin)) amount
FROM personal_periods pp 
LEFT JOIN connection_types ct ON ct.id=pp.connection_type
INNER JOIN tv_types tt ON tt.id=pp.tv_type
WHERE ct.service_type&2 AND pp.state=2 AND !tt.offline
GROUP BY pp.client;
END ;;
DELIMITER ;

ALTER TABLE staff ADD COLUMN client INT UNSIGNED, ADD KEY(client), ADD CONSTRAINT fk_staff_client FOREIGN KEY(client) REFERENCES clients (id) ON DELETE SET NULL;

ALTER TABLE client_calls
ADD COLUMN cabinet TINYINT NOT NULL,
ADD COLUMN consult TINYINT NOT NULL,
ADD COLUMN office TINYINT NOT NULL,
ADD COLUMN office_reason TINYINT NOT NULL,
ADD COLUMN office_rating TINYINT NOT NULL,
ADD COLUMN commercial TINYINT NOT NULL,
ADD COLUMN service_rating TINYINT NOT NULL;

alter table connection_types drop key name;
alter table connection_types add unique(name, city);

SET names cp1251;
-- INSERT INTO system_entities (id, key_name, name) VALUES (1246, "referal", ":  ");
INSERT INTO system_entities (id, key_name, name) VALUES (117, "privatb", ":  ");

ALTER TABLE clients ADD COLUMN referal VARCHAR(32) AFTER contract; 

alter table troubles drop key code;

ALTER TABLE exp_points
ADD COLUMN photo_file3 int unsigned AFTER photo_file2,
ADD COLUMN photo_file4 int unsigned AFTER photo_file3;

ALTER TABLE exp_points
ADD COLUMN in_proekt tinyint,
ADD COLUMN acting tinyint,
ADD COLUMN mob tinyint,
ADD COLUMN proekt varchar(40),
ADD COLUMN kod varchar(40),
ADD COLUMN power tinyint,
ADD COLUMN splices tinyint,
ADD COLUMN splices_list varchar(255),
ADD COLUMN checked tinyint;

ALTER TABLE houses ADD COLUMN verified TINYINT NOT NULL;

alter table biometric_auth_events drop foreign key person;
alter table biometric_auth_events add key (person);
alter table biometric_auth_events add constraint fk_biometric_auth_events_person foreign key (person) references staff(id) on delete cascade;

alter table modems_foto drop foreign key modem;
alter table modems_foto add key(modem);
alter table modems_foto add constraint fk_modems_foto_modem foreign key (modem) references modems (id) on delete cascade;

alter table staff_foto drop foreign key staff;
alter table staff_foto add key (staff);
alter table staff_foto add constraint fk_staff_foto_staff foreign key (staff) references staff(id) on delete cascade;

ALTER TABLE browser_properties DROP INDEX browser;
ALTER TABLE admin_districts DROP INDEX city;
ALTER TABLE calendar_account_states DROP INDEX account;
ALTER TABLE client_expenses_details DROP INDEX expense;
ALTER TABLE houses DROP INDEX city;
ALTER TABLE jobs DROP INDEX street;
ALTER TABLE koop_pays DROP INDEX koop;
ALTER TABLE koops DROP INDEX city;
ALTER TABLE kpreos DROP INDEX city;
ALTER TABLE modems_foto DROP INDEX modem_2;
ALTER TABLE staff_foto DROP INDEX staff;
ALTER TABLE streets DROP INDEX city;

CREATE TABLE dhcp_hosts (
	host INT UNSIGNED NOT NULL PRIMARY KEY,
	dhcp INT UNSIGNED NOT NULL,
	KEY(dhcp),
	CONSTRAINT fk_dhcp_hosts_host FOREIGN KEY(host) REFERENCES hosts(id) ON DELETE CASCADE,
	CONSTRAINT fk_dhcp_hosts_dhcp FOREIGN KEY(dhcp) REFERENCES dhcp_templates(id) ON DELETE CASCADE
) ENGINE=InnoDB;

CREATE TABLE dhcp_modems (
	modem INT UNSIGNED NOT NULL PRIMARY KEY,
	dhcp INT UNSIGNED NOT NULL,
	KEY(dhcp),
	CONSTRAINT fk_dhcp_modems_modem FOREIGN KEY(modem) REFERENCES modems(id) ON DELETE CASCADE,
	CONSTRAINT fk_dhcp_modems_dhcp FOREIGN KEY(dhcp) REFERENCES dhcp_templates(id) ON DELETE CASCADE
) ENGINE=InnoDB;

CREATE TABLE privatb (
	id INT UNSIGNED NOT NULL AUTO_INCREMENT PRIMARY KEY,
	client INT UNSIGNED,
	movement BIGINT UNSIGNED,
	pkey VARCHAR (32) NOT NULL UNIQUE,
	amount BIGINT UNSIGNED NOT NULL,
	date DATETIME NOT NULL,
	KEY client (client),
	KEY movement (movement),
	CONSTRAINT fk_privatb_client FOREIGN KEY (client) REFERENCES clients (id) ON DELETE SET NULL,
	CONSTRAINT fk_privatb_movement FOREIGN KEY (movement) REFERENCES movements (id) ON DELETE SET NULL
) ENGINE=InnoDB;

INSERT INTO config (id, value, control, actual) VALUES ('PRIVATE_CASH_ACCESS_ZONE', '0.0.0.0/0', 0, 1);

SET NAMES CP1251;
INSERT INTO menu_items (id, title, link, access_by) VALUES (147, '  ', 'privatb.php', 117);
INSERT INTO menu_operator_items (menu_item, menu_box, order_num) VALUES (147, 1, 178);
UPDATE menu_operator_items SET order_num=179 WHERE menu_item=117 AND operator IS NULL;

SET names cp1251;
INSERT INTO system_entities (id, key_name, name) VALUES (118, "delayed_tasks", " ");

INSERT INTO operators_access (operator, entity, access)
SELECT operator, 118, access FROM operators_access WHERE entity=15;

INSERT INTO groups_access (group_id, entity, access)
SELECT group_id, 118, access FROM groups_access WHERE entity=15;

UPDATE menu_items SET access_by=118 WHERE link='delayed_tasks.php';
